@@ -19,7 +19,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: arpcop.c,v 1.2 2001/08/31 13:32:28 alor Exp $
+    $Id: arpcop.c,v 1.3 2001/09/27 19:07:40 alor Exp $
 */
 
 #include <stdio.h>
@@ -36,11 +36,6 @@
 #include "../../src/include/ec_inet_forge.h"
 #include "../../src/include/ec_error.h"
 
-char Ettercap_Version[] = VERSION;                       // required -- for compatibility
-
-char PlugIn_Info[] = "report suspicious ARP activity";
-
-short PlugIn_Version = 10;
 
 typedef struct host_list_s host_list;
 
@@ -55,16 +50,42 @@ host_list *list;
 char ETH_BROADCAST[6] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
 char ARP_BROADCAST[6] = {0x0,0x0,0x0,0x0,0x0,0x0};
 
-// protos....
+// protos...
 
+int Plugin_Init(void *);
+int Plugin_Fini(void *);
+int arpcop_function(void *dummy);
 int ip_aton(const char *ip_str);
 host_list *arpcop_Host_in_LAN_list(void);
 void arpcop_Free_list(host_list *head);
 int Is_LAN_IP(int rem_ip);
 void Parse_packet(char *buffer);
-void PlugIn_Start(char**argv);
 
-// ---------------------------
+// plugin operation
+
+struct plugin_ops ops = {
+   ettercap_version: VERSION,
+   plug_info:        "Report suspicious ARP activity",
+   plug_version:     10,
+   plug_type:        PT_EXT,
+   hook_point:       HOOK_NONE,
+   hook_function:    &arpcop_function,
+};
+
+//==================================
+
+int Plugin_Init(void *params)
+{
+   return Plugin_Register(params, &ops);
+}
+
+int Plugin_Fini(void *params)
+{
+   return 0;
+}
+
+// =================================
+
 
 int ip_aton(const char *ip_str)
 {
@@ -253,7 +274,7 @@ void Parse_packet(char *buffer)
 }
 
 
-void PlugIn_Start(char**argv)
+int arpcop_function(void *dummy)
 {
     int sock, MTU = 1500;
     char *recv_pck;
@@ -287,6 +308,8 @@ void PlugIn_Start(char**argv)
 
     Inet_Forge_packet_destroy( recv_pck );
     Inet_CloseRawSock(sock);
+
+    return 0;
 }
 
 /* EOF */
