@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: leech.c,v 1.2 2001/08/31 13:32:28 alor Exp $
+    $Id: leech.c,v 1.3 2001/09/27 19:07:40 alor Exp $
 */
 
 #include <unistd.h>
@@ -28,19 +28,38 @@
 #include "../../src/include/ec_inet.h"
 #include "../../src/include/ec_inet_forge.h"
 
-char Ettercap_Version[] = VERSION;                       // required -- for compatibility
-
-char PlugIn_Info[] = "Isolate a host from the LAN";
-
-short PlugIn_Version = 17;
-
 // protos...
 
-void PlugIn_Start(char**argv);
+int Plugin_Init(void *);
+int Plugin_Fini(void *);
+int leech_function(void *dummy);
 
-//--------------------
+// plugin operation
 
-void PlugIn_Start(char**argv)
+struct plugin_ops ops = {
+   ettercap_version: VERSION,
+   plug_info:        "Isolate a host from the LAN",
+   plug_version:     17,
+   plug_type:        PT_EXT,
+   hook_point:       HOOK_NONE,
+   hook_function:    &leech_function,
+};
+
+//==================================
+
+int Plugin_Init(void *params)
+{
+   return Plugin_Register(params, &ops);
+}
+
+int Plugin_Fini(void *params)
+{
+   return 0;
+}
+
+// =================================
+
+int leech_function(void *dummy)
 {
     int IPS, IPD, sock, i;
     unsigned char MACS[6];
@@ -51,7 +70,7 @@ void PlugIn_Start(char**argv)
     if (!strcmp(Host_Dest.ip,""))
     {
       Plugin_Output("Please select a Dest...\n");
-      return;
+      return 0;
     }
 
     memset(cont, 0, 4);
@@ -61,7 +80,7 @@ void PlugIn_Start(char**argv)
     if (strcmp(cont, "yes"))  // not sure... ;)
     {
       Plugin_Output("\nIt is safe!  for now...\n");
-      return;
+      return 0;
     }
     Plugin_Output("\nBuilding host list for netmask %s, please wait...\n", Inet_MySubnet());
          number_of_hosts_in_lan = Inet_HostInLAN();
@@ -99,7 +118,7 @@ void PlugIn_Start(char**argv)
       {
          Inet_Forge_packet_destroy( pck_to_send );
          Inet_CloseRawSock(sock);
-         return;
+         return 0;
       }
     }
 }
