@@ -1,5 +1,5 @@
 /*
-    triton -- ettercap plugin -- spoof DNS requests
+    phantom -- ettercap plugin -- spoof DNS requests
 
     Copyright (C) 2001  NaGoR
 
@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: phantom.c,v 1.2 2001/08/31 13:32:28 alor Exp $
+    $Id: phantom.c,v 1.5 2001/10/16 19:56:00 alor Exp $
 */
 
 #include <stdlib.h>
@@ -39,12 +39,6 @@
 #include "../../src/include/ec_parser.h"
 #include "../../src/include/ec_queue.h"
 
-char Ettercap_Version[] = VERSION;                       // required -- for compatibility
-
-char PlugIn_Info[] = "Sniff/Spoof DNS requests";
-
-short PlugIn_Version = 14;
-
 
 /*
  *                                     1  1  1  1  1  1
@@ -115,14 +109,39 @@ SLIST_HEAD(, dns_file_entry) dns_entries;
 
 // protos...
 
+int Plugin_Init(void *);
+int Plugin_Fini(void *);
 void Parse_Packet(char *buf, int sock, char *MyMAC);
 char * GetType(short t);
 int Load_DNS_entries(void);
 u_long dns_spoof_a(const char *name);
 char * dns_spoof_ptr(const char *name);
-void PlugIn_Start(char**argv);
+int phantom(void *);
 
-// ---------------------------
+// plugin operation
+
+struct plugin_ops ops = {
+   ettercap_version: VERSION,
+   plug_info:        "Sniff/Spoof DNS requests",
+   plug_version:     14,
+   plug_type:        PT_EXT,
+   hook_point:       HOOK_NONE,
+   hook_function:    &phantom,
+};
+
+//==================================
+
+int Plugin_Init(void *params)
+{
+   return Plugin_Register(params, &ops);
+}
+
+int Plugin_Fini(void *params)
+{
+   return 0;
+}
+
+// =================================
 
 
 char * GetType(short t)
@@ -358,7 +377,7 @@ void Parse_Packet(char *buf, int sock, char *MyMAC)
 
 
 
-void PlugIn_Start(char**argv)
+int phantom(void *dummy)
 {
    int sock, MTU, len=0;
    char MyMAC[6];
@@ -368,7 +387,7 @@ void PlugIn_Start(char**argv)
    if (Load_DNS_entries() == 1) //there was an error
    {
       Plugin_Output("\n");
-      return;
+      return 0;
    }
 
    sock = Inet_OpenRawSock(Options.netiface);
@@ -403,7 +422,7 @@ void PlugIn_Start(char**argv)
       {
          Inet_Forge_packet_destroy( buf );
          Inet_CloseRawSock(sock);
-         return;
+         return 0;
       }
 
       if (len > 0) Parse_Packet(buf, sock, MyMAC);
@@ -426,14 +445,14 @@ int Load_DNS_entries(void)
    fto = fopen( "./" DNS_FILE, "r");
    if (fto == NULL)
    {
-      fto = fopen( INSTALL_PATH DNS_FILE, "r");
+      fto = fopen( DATA_PATH DNS_FILE, "r");
       if (fto == NULL)
       {
-         Plugin_Output("\nCan't find " DNS_FILE " in ./ or " INSTALL_PATH);
+         Plugin_Output("\nCan't find " DNS_FILE " in ./ or " DATA_PATH);
          return 1;
       }
       else
-         Plugin_Output("\nLoading DNS entries from " INSTALL_PATH DNS_FILE "...\n\n");
+         Plugin_Output("\nLoading DNS entries from " DATA_PATH DNS_FILE "...\n\n");
    }
    else
       Plugin_Output("\nLoading DNS entries from ./" DNS_FILE "...\n\n");
