@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: shadow.c,v 1.2 2001/08/31 13:32:28 alor Exp $
+    $Id: shadow.c,v 1.3 2001/09/27 19:07:40 alor Exp $
 */
 
 #include <stdlib.h>
@@ -33,13 +33,6 @@
 #include "../../src/include/ec_inet_forge.h"
 #include "../../src/include/ec_decodedata.h"
 
-char Ettercap_Version[] = VERSION;                       // required -- for compatibility
-
-char PlugIn_Info[] = "A very simple SYN/TCP port scanner";
-
-short PlugIn_Version = 18;
-
-
 #define SYN_SEQ 6969
 
 // global data...
@@ -47,12 +40,37 @@ short PlugIn_Version = 18;
 int IPS, IPD, port_index=0;
 int *PORTREP;
 
-// protos....
+// protos...
 
+int Plugin_Init(void *);
+int Plugin_Fini(void *);
+int shadow_main(void *dummy);
 void Parse_packet(char *buffer);
-void PlugIn_Start(char**argv);
 
-// ---------------------------
+// plugin operation
+
+struct plugin_ops ops = {
+   ettercap_version: VERSION,
+   plug_info:        "A very simple SYN/TCP port scanner",
+   plug_version:     18,
+   plug_type:        PT_EXT,
+   hook_point:       HOOK_NONE,
+   hook_function:    &shadow_main,
+};
+
+//==================================
+
+int Plugin_Init(void *params)
+{
+   return Plugin_Register(params, &ops);
+}
+
+int Plugin_Fini(void *params)
+{
+   return 0;
+}
+
+// =================================
 
 void Parse_packet(char *buffer)
 {
@@ -79,7 +97,7 @@ void Parse_packet(char *buffer)
 }
 
 
-void PlugIn_Start(char**argv)
+int shadow_main(void *dummy)
 {
     int i, sock, MTU, startP, finP, OldIndex=0;
     unsigned short PORTS, IP_ID;
@@ -92,13 +110,13 @@ void PlugIn_Start(char**argv)
     if (!strcmp(Host_Dest.ip,""))
     {
       Plugin_Output("Please select a Dest...\n");
-      return;
+      return 0;
     }
 
     if (!strcmp(Host_Dest.ip, Inet_MyIPAddress()))
     {
        Plugin_Output("\nThis plugin doesn't work on myself !!\n");
-       return;
+       return 0;
     }
 
     Plugin_Output("\nStarting Port: ");
@@ -112,7 +130,7 @@ void PlugIn_Start(char**argv)
     if (startP>finP)
     {
 		Plugin_Output("\nStopping Port must be greater than Starting Port\n");
-		return;
+		return 0;
     }
 
     IPD = inet_addr(Host_Dest.ip);
@@ -163,6 +181,7 @@ void PlugIn_Start(char**argv)
     Inet_Forge_packet_destroy( pck_to_send );
     free(PORTREP);
     Inet_CloseRawSock(sock);
+    return 0;
 }
 
 /* EOF */
