@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: spectre.c,v 1.2 2001/08/31 13:32:28 alor Exp $
+    $Id: spectre.c,v 1.3 2001/09/27 19:07:40 alor Exp $
 */
 
 #include <stdlib.h>
@@ -31,20 +31,39 @@
 #include "../../src/include/ec_inet.h"
 #include "../../src/include/ec_inet_forge.h"
 
-char Ettercap_Version[] = VERSION;                       // required -- for compatibility
+// protos...
 
-char PlugIn_Info[] = "Flood a switched LAN with random MAC addresses";
+int Plugin_Init(void *);
+int Plugin_Fini(void *);
+int spectre_function(void *dummy);
 
-short PlugIn_Version = 13;
+// plugin operation
 
-// protos....
+struct plugin_ops ops = {
+   ettercap_version: VERSION,
+   plug_info:        "Flood the LAN with random MAC addresses",
+   plug_version:     13,
+   plug_type:        PT_EXT,
+   hook_point:       HOOK_NONE,
+   hook_function:    &spectre_function,
+};
 
-void PlugIn_Start(char**argv);
+//==================================
 
-// ---------------------------
+int Plugin_Init(void *params)
+{
+   return Plugin_Register(params, &ops);
+}
+
+int Plugin_Fini(void *params)
+{
+   return 0;
+}
+
+// =================================
 
 
-void PlugIn_Start(char**argv)
+int spectre_function(void *dummy)
 {
    int sock, MTU;
    char c[2] = "";
@@ -61,7 +80,7 @@ void PlugIn_Start(char**argv)
    if (strncmp(cont, "yes", 3))  // not sure... ;)
    {
       Plugin_Output("\nIt is safe!  for now...\n");
-      return;
+      return 0;
    }
 
    gettimeofday(&seed, NULL);
@@ -111,7 +130,7 @@ void PlugIn_Start(char**argv)
       {
          Inet_Forge_packet_destroy( buf );
          Inet_CloseRawSock(sock);
-         return;
+         return 0;
       }
    }
 
