@@ -19,7 +19,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_inet_BSD.c,v 1.1.1.1 2001/08/27 19:27:19 alor Exp $
+    $Id: ec_inet_BSD.c,v 1.6 2001/12/08 16:38:43 alor Exp $
 */
 
 
@@ -201,7 +201,7 @@ int Inet_FindIFace(char *iface)
 
 
 
-char Inet_CorrectIface(char *iface)
+int Inet_CorrectIface(char *iface)
 {
 #ifdef HAVE_GETIFADDRS
    struct ifaddrs *ifalist, *ifa;
@@ -488,7 +488,12 @@ int Inet_GetRawPacket(int sock, char *buffer, int MTU, short *type)
    static char MyMAC[6]={0x65,0x74,0x74,0x65,0x72,0x63};
 
    if (SocketBuffer == -1)                   // only the first time
+   {
       SocketBuffer = Buffer_Create(1.0e5);   // 100 K buffer
+      #ifdef DEBUG
+         Debug_msg("Inet_GetRawPacket creates the buffer for the first time -- buf id = %d", SocketBuffer);
+      #endif
+   }
 
    Buffer_Get(SocketBuffer, &pktlen, sizeof(u_int));
    len = Buffer_Get(SocketBuffer, buffer, pktlen );
@@ -498,9 +503,9 @@ int Inet_GetRawPacket(int sock, char *buffer, int MTU, short *type)
        if (!strncmp(MyMAC,"etterc",6))    // only the first time...
            Inet_GetIfaceInfo(Options.netiface, NULL, MyMAC, NULL, NULL);
        if (!memcmp(MyMAC,buffer,6))
-           *type = 0; // PACKET_HOST
+           *type = PACKET_HOST;
        else
-           *type = 1; // !PACKET_HOST
+           *type = !PACKET_HOST;
    }
 
    if (len > 0) return len;                     // there was pending fata.
@@ -534,9 +539,9 @@ int Inet_GetRawPacket(int sock, char *buffer, int MTU, short *type)
    if (type != NULL)
    {
       if (!memcmp(MyMAC,buffer,6))
-         *type = 0; // PACKET_HOST
-      else
-         *type = 1; // !PACKET_HOST
+           *type = PACKET_HOST;
+       else
+           *type = !PACKET_HOST;
    }
 
    free(buf);
@@ -553,12 +558,19 @@ int Inet_SendRawPacket(int sock, char *buffer, int len)
    int sent;
 
    sent = write(sock, buffer, len);
-
    if (sent < len)
+   {
+      while (errno == ENOBUFS)
+      {
+         usleep(5000);
+         sent = write(sock, buffer, len);
+         if (sent == len) return (sent);
+      }
+
       Error_msg("ec_Inet_BSD:%d write() %d(%d) | ERRNO : %d | %s \n", __LINE__, len, sent, errno, strerror(errno));
+   }
 
    return (sent);
-
 }
 
 
@@ -609,7 +621,7 @@ void Inet_DisableForwarding(void)
    Debug_msg("Inet_DisableForwarding | net.inet.ip.forwarding = %d  old_value = %d\n", val, IpForward_status);
 #endif
 
-   exit_func(Inet_RestoreForwarding);
+   atexit(Inet_RestoreForwarding);
 
 }
 
@@ -625,7 +637,7 @@ void Inet_RestoreForwarding(void)
    mib[2] = IPPROTO_IP;
    mib[3] = IPCTL_FORWARDING;
 
-   if (!strstr(program_argv0, PROGRAM)) return;
+   if (strcmp(ECThread_getname(pthread_self()), PROGRAM)) return;
 
    if( (sysctl(mib, 4, NULL, NULL, &IpForward_status, sizeof(IpForward_status))) == -1)
       Error_msg("ec_inet_BSD:%d sysctl() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
@@ -909,7 +921,7 @@ int Inet_SetARPEntry(unsigned long IP, char MAC[6])
    SavedIP = Inet_Fake_Host();
    memcpy(SavedMAC, MAC, 6);
 
-   exit_func(Inet_UnSetARPEntry);
+   atexit(Inet_UnSetARPEntry);
 
    ret = rtmsg_arp(RTM_ADD);
 
@@ -940,7 +952,7 @@ typedef union sockunion *sup;
 int rtmsg_route(int cmd, int flags);
 void inet_makenetandmask(u_long net, struct sockaddr_in *sin, u_long bits);
 int getaddr(int which, char *s, struct hostent **hpp);
-void mask_addr();
+void mask_addr(void);
 int rtmsg_route(int cmd, int flags);
 
 //====================
@@ -1205,7 +1217,7 @@ void Inet_SetRoute(void)
    forcenet = 0;
 
    close(s);
-   exit_func(Inet_UnsetRoute);
+   atexit(Inet_UnsetRoute);
 }
 
 #endif
