@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_inet_macosx.c,v 1.1.1.1 2001/08/27 19:27:24 alor Exp $
+    $Id: ec_inet_macosx.c,v 1.4 2001/11/22 20:13:55 alor Exp $
 */
 
 
@@ -160,7 +160,7 @@ int Inet_FindIFace(char *iface)     // adapded from pcap_lookupdev
 }
 
 
-char Inet_CorrectIface(char *iface)
+int Inet_CorrectIface(char *iface)
 {
    int sock;
    struct ifreq ifr;
@@ -488,12 +488,19 @@ int Inet_SendRawPacket(int sock, char *buffer, int len)
    int sent;
 
    sent = write(sock, buffer, len);
-
    if (sent < len)
+   {
+      while (errno == ENOBUFS)
+      {
+         usleep(5000);
+         sent = write(sock, buffer, len);
+         if (sent == len) return (sent);
+      }
+
       Error_msg("ec_Inet_macosx:%d write() %d(%d) | ERRNO : %d | %s \n", __LINE__, len, sent, errno, strerror(errno));
+   }
 
    return (sent);
-
 }
 
 
@@ -544,7 +551,7 @@ void Inet_DisableForwarding(void)
    Debug_msg("Inet_DisableForwarding | net.inet.ip.forwarding = %d  old_value = %d\n", val, IpForward_status);
 #endif
 
-   exit_func(Inet_RestoreForwarding);
+   atexit(Inet_RestoreForwarding);
 
 }
 
@@ -560,7 +567,7 @@ void Inet_RestoreForwarding(void)
    mib[2] = IPPROTO_IP;
    mib[3] = IPCTL_FORWARDING;
 
-   if (!strstr(program_argv0, PROGRAM)) return;
+   if (strcmp(ECThread_getname(pthread_self()), PROGRAM)) return;
 
    if( (sysctl(mib, 4, NULL, NULL, &IpForward_status, sizeof(IpForward_status))) == -1)
       Error_msg("ec_Inet_macosx:%d sysctl() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
