@@ -19,7 +19,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_inet_linux.c,v 1.1.1.1 2001/08/27 19:27:30 alor Exp $
+    $Id: ec_inet_linux.c,v 1.5 2001/11/19 09:30:23 alor Exp $
 */
 
 #include <ctype.h>
@@ -135,7 +135,7 @@ int Inet_FindIFace(char *iface)     // adapded from pcap_lookupdev
 }
 
 
-char Inet_CorrectIface(char *iface)
+int Inet_CorrectIface(char *iface)
 {
    int sock;
    struct ifreq ifr;
@@ -368,7 +368,16 @@ int Inet_SendRawPacket(int sock, char *buffer, int len)
 
    sent = sendto(sock, buffer, len, 0, (struct sockaddr *)&dest, sizeof(dest));
    if (sent < len)
+   {
+      while (errno == ENOBUFS)
+      {
+         usleep(5000);
+         sent = sendto(sock, buffer, len, 0, (struct sockaddr *)&dest, sizeof(dest));
+         if (sent == len) return (sent);
+      }
+
       Error_msg("ec_Inet_linux:%d sendto() %d(%d) | ERRNO : %d | %s \n", __LINE__, len, sent, errno, strerror(errno));
+   }
 
    return (sent);
 
@@ -402,7 +411,7 @@ int Inet_SetPromisc(char *iface)
       ifr.ifr_flags |= IFF_PROMISC;
       if ( ioctl(sock, SIOCSIFFLAGS, &ifr) < 0 )      // promisc mode
          Error_msg("ec_Inet_linux:%d ioctl(SIOCSIFFLAGS) | promisc on | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
-      exit_func(Inet_Restore_ifr);
+      atexit(Inet_Restore_ifr);
    }
 
 
@@ -449,14 +458,14 @@ void Inet_DisableForwarding(void)
       return;
    }
 
-   fscanf(fd, "%s", IpForward_status);
+   fscanf(fd, "%c", &IpForward_status);
    fclose(fd);
 
 #ifdef DEBUG
-   Debug_msg("Inet_DisableForwarding from %c", IpForward_status[0]);
+   Debug_msg("Inet_DisableForwarding from %c", IpForward_status);
 #endif
 
-   exit_func(Inet_RestoreForwarding);
+   atexit(Inet_RestoreForwarding);
 
    fd = fopen("/proc/sys/net/ipv4/ip_forward", "w");
    if (fd < 0 )
@@ -477,7 +486,7 @@ void Inet_RestoreForwarding(void)
 {
    FILE *fd;
 
-   if (!strstr(program_argv0, PROGRAM)) return;
+   if (strcmp(ECThread_getname(pthread_self()), PROGRAM)) return;
 
    fd = fopen("/proc/sys/net/ipv4/ip_forward", "w");
    if (fd < 0 )
@@ -489,10 +498,10 @@ void Inet_RestoreForwarding(void)
    }
 
 #ifdef DEBUG
-   Debug_msg("Inet_RestoreForwarding to %c", IpForward_status[0]);
+   Debug_msg("Inet_RestoreForwarding to %c", IpForward_status);
 #endif
 
-   fprintf(fd, "%c", IpForward_status[0] );
+   fprintf(fd, "%c", IpForward_status );
    fclose(fd);
 }
 
@@ -655,7 +664,7 @@ int Inet_SetARPEntry(unsigned long IP, char MAC[6])
 
     SavedIP=IP;
     memcpy(SavedMAC, MAC, 6);
-    exit_func(Inet_UnSetARPEntry);
+    atexit(Inet_UnSetARPEntry);
 
     return retval;
 }
@@ -746,7 +755,7 @@ void Inet_SetRoute(void)
 
     close(skfd);
 
-    exit_func(Inet_UnsetRoute);
+    atexit(Inet_UnsetRoute);
 }
 
 #endif
