@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_inet_solaris.c,v 1.3 2001/08/28 08:57:13 alor Exp $
+    $Id: ec_inet_solaris.c,v 1.5 2001/11/19 09:30:23 alor Exp $
 */
 
 
@@ -205,7 +205,7 @@ int Inet_FindIFace(char *iface)     // adapded from pcap_lookupdev
 }
 
 
-char Inet_CorrectIface(char *iface)
+int Inet_CorrectIface(char *iface)
 {
    int sock;
    struct ifreq ifr;
@@ -366,11 +366,11 @@ int Inet_OpenRawSock(char *iface)      // adapted from libpcap source code
     */
    cp = strpbrk(dname, "0123456789");
    if (cp == NULL)
-      Error_msg("ec_Inet_solaris:%d dlpi: %s missing unit number\n", __LINE__, iface);
+      Error_msg("ec_Inet_solaris:%d HAVE_DEV_DLPI: %s missing unit number\n", __LINE__, iface);
 
    ppa = strtol(cp, &eos, 10);
    if (*eos != '\0')
-      Error_msg("ec_Inet_solaris:%d dlpi: %s bad unit number\n", __LINE__, iface);
+      Error_msg("ec_Inet_solaris:%d HAVE_DEV_DLPI: %s bad unit number\n", __LINE__, iface);
 
    *cp = '\0';
 
@@ -394,7 +394,7 @@ int Inet_OpenRawSock(char *iface)      // adapted from libpcap source code
    /*
    ** Determine device and ppa
    */
-      cp = strpbrk(dname, "0123456789");
+   cp = strpbrk(dname, "0123456789");
    if (cp == NULL)
       Error_msg("ec_Inet_solaris:%d dlpi: %s missing unit number\n", __LINE__, iface);
 
