@@ -17,32 +17,18 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_buffer.c,v 1.1.1.1 2001/08/27 19:24:19 alor Exp $
+    $Id: ec_buffer.c,v 1.5 2001/12/08 16:38:43 alor Exp $
 */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
-#include <sys/ipc.h>
-#include <sys/shm.h>
-
-
 #include "include/ec_main.h"
 #include "include/ec_error.h"
 
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
-
-
 #define MAX_BUFFERS 10
 #define HEAD_LEN 8
 
 typedef struct {
     char *data;
     int len;
-    int key;
 }Buffer_Data;
 
 // global data
@@ -65,30 +51,24 @@ void Buffer_Flush(int ID);
 
 int Buffer_Create(int len)
 {
-    int i;
-
-#ifdef DEBUG
-   Debug_msg("Buffer_Create -- %d", len);
-#endif
+   int i;
 
    for (i=0; i<MAX_BUFFERS; i++)
    {
       if (!Buffer_List[i].data)
       {
-         Buffer_List[i].key = shmget(0,len+HEAD_LEN,IPC_CREAT | 0600);
-         if (Buffer_List[i].key == -1)
-            Error_msg("Can't attach %d bytes of shared memory",len);
-
-         Buffer_List[i].data = (char *)shmat(Buffer_List[i].key,0,0);
+         Buffer_List[i].data = (char *) calloc(len+HEAD_LEN, sizeof(char));
          Buffer_List[i].len = len;
-         shmctl(Buffer_List[i].key, IPC_RMID, NULL);
-
-         memset(Buffer_List[i].data,0,HEAD_LEN);
          break;
       }
    }
 
    if (i == MAX_BUFFERS) i=-1;
+
+#ifdef DEBUG
+   Debug_msg("Buffer_Create -- [%d] len %d", i, len);
+#endif
+
    return i;
 }
 
@@ -96,21 +76,22 @@ int Buffer_Create(int len)
 
 void Buffer_Flush(int ID)
 {
-    if (!Buffer_List[ID].data) return;
-    // Reset offsets
-    memset(Buffer_List[ID].data, 0, HEAD_LEN);
+   if (!Buffer_List[ID].data)
+      return;
+   // Reset offsets
+   memset(Buffer_List[ID].data, 0, HEAD_LEN);
 }
 
 
 
 int Buffer_Get(int ID, void *to_read, int size)
 {
-
    int reprise;
    unsigned long *R_Offset, W_Offset;
    char *data;
 
-   if (!Buffer_List[ID].data) return -1;
+   if (!Buffer_List[ID].data)
+      return -1;
 
    R_Offset = (unsigned long *)Buffer_List[ID].data;
    W_Offset = *(unsigned long *)(Buffer_List[ID].data+4);
@@ -139,7 +120,6 @@ int Buffer_Get(int ID, void *to_read, int size)
    *R_Offset=(*R_Offset+size)%Buffer_List[ID].len;
 
    return size;
-
 }
 
 
@@ -147,12 +127,14 @@ int Buffer_Get(int ID, void *to_read, int size)
 
 int Buffer_Put(int ID, void *to_write, int size)
 {
-   static int retry=0;
+   static int retry = 0;
    int reprise;
    unsigned long *R_Offset, *W_Offset;
    char *data;
 
-   if (!Buffer_List[ID].data) return -1;
+
+   if (!Buffer_List[ID].data)
+      return -1;
 
    R_Offset = (unsigned long *)Buffer_List[ID].data;
    W_Offset = (unsigned long *)(Buffer_List[ID].data+4);
@@ -162,7 +144,14 @@ int Buffer_Put(int ID, void *to_write, int size)
 
    while((*W_Offset<*R_Offset && *W_Offset+size>=*R_Offset) || reprise>=(long)*R_Offset)
    {
-      if (retry >= 2) return 0;
+      #ifdef DEBUG
+         Debug_msg("Buffer_Put -- %d BUFFER FULL !! buff len [%d] byte lost [%d]", ID, Buffer_List[ID].len, size);
+      #endif
+      if (retry >= 2)
+      {
+         retry = 0;
+         return 0;
+      }
       retry++;
       usleep(500);
    }
@@ -178,6 +167,7 @@ int Buffer_Put(int ID, void *to_write, int size)
    }
 
    *W_Offset = (*W_Offset+size)%Buffer_List[ID].len;
+
    return 0;
 
 }
