@@ -17,16 +17,13 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_debug.c,v 1.1.1.1 2001/08/27 19:23:59 alor Exp $
+    $Id: ec_debug.c,v 1.5 2001/12/08 11:22:20 alor Exp $
 */
 
 #include "include/ec_main.h"
 
 #ifdef DEBUG
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <stdarg.h>
 #ifdef HAVE_SYS_UTSNAME_H
    #include <sys/utsname.h>
@@ -46,6 +43,8 @@
    #define DEBUG_FILE "ettercap_debug.log"
 #endif
 
+#include "include/ec_thread.h"
+
 FILE *debug_dev;
 
 void Debug_Init(void);
@@ -60,8 +59,8 @@ void Debug_Init(void)
 
    if ((debug_dev = fopen (DEBUG_FILE, "a")) < 0)
    {
-      printf ("Couldn't open DEBUG FILE!\n");
-      exit (0);
+      fprintf(stderr, "Couldn't open DEBUG FILE!\n");
+      exit(0);
    }
    else
    {
@@ -80,10 +79,9 @@ void Debug_Init(void)
       #ifdef HAVE_NCURSES
          fprintf (debug_dev, "-> %s\n", curses_version());
       #endif
-      fprintf (debug_dev, "\n\n");
-      fprintf (debug_dev, "DEVICE OPENED FOR %s DEBUGGING\n\n", PROGRAM);
+      fprintf (debug_dev, "\n\nDEVICE OPENED FOR %s DEBUGGING\n\n", PROGRAM);
       fflush(debug_dev);
-      exit_func(Debug_Close);
+      atexit(Debug_Close);
    }
 }
 
@@ -91,7 +89,7 @@ void Debug_Init(void)
 
 void Debug_Close(void)
 {
-   fprintf (debug_dev, "\n\nDEBUGGING DEVICE FOR %s WAS CLOSED\n\n", program_argv0);
+   fprintf (debug_dev, "\n\nDEBUGGING DEVICE FOR %s WAS CLOSED\n\n", ECThread_getname(pthread_self()));
    fclose (debug_dev);
 }
 
@@ -103,10 +101,10 @@ void Debug_msg(char *message, ...)
    va_list ap;
    char debug_message[strlen(message)+2];
 
-   fprintf (debug_dev, "%s\t", program_argv0);
+   fprintf (debug_dev, "%-10s\t", ECThread_getname(pthread_self()));
 
-   strcpy(debug_message, message);              // for backward compatibility
-   strcat(debug_message, "\n");
+   strlcpy(debug_message, message, sizeof(debug_message)); // for backward compatibility
+   strlcat(debug_message, "\n", sizeof(debug_message));
 
    va_start(ap, message);
    vfprintf(debug_dev, debug_message, ap);
