@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_dissector_bgp.c,v 1.1.1.1 2001/08/27 19:26:00 alor Exp $
+    $Id: ec_dissector_bgp.c,v 1.6 2001/12/13 17:05:20 alor Exp $
 */
 
 /*
@@ -97,18 +97,10 @@
 
 #include "include/ec_main.h"
 
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "include/ec_dissector.h"
 #include "include/ec_inet_structures.h"
 #include "include/ec_error.h"
 
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
-
 // protos
 
 FUNC_DISSECTOR(Dissector_bgp);
@@ -128,10 +120,11 @@ FUNC_DISSECTOR(Dissector_bgp)
                           0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
    ONLY_CONNECTION;
 
+   if (data_to_ettercap->datalen == 0) return 0;      // No data...
+
    tcp = (TCP_header *) data;
 
    payload = (char *)((int)tcp + tcp->doff * 4);
-   if (data_to_ettercap->datalen == 0) return 0;      // No data...
 
    if ( payload[19] != 4 ) return 0;                  // not the right version (4)
 
@@ -141,7 +134,7 @@ FUNC_DISSECTOR(Dissector_bgp)
 
    if ( (param_length = payload[28]) == 0 ) return 0; // no optional parameter
 
-   parameters = payload+29;
+   parameters = payload + 29;
 
 #ifdef DEBUG
    Debug_msg("\tDissector_BGP");
@@ -155,8 +148,9 @@ FUNC_DISSECTOR(Dissector_bgp)
          #ifdef DEBUG
             Debug_msg("\tDissector_BGP version 4 AUTH");
          #endif
-         sprintf(data_to_ettercap->user, "AUTH TYPE 0x%02x\n", parameters[i+2]);
-         strncpy(data_to_ettercap->pass, parameters + i + 3, (len>30) ? 30 : len);
+         sprintf(data_to_ettercap->user, "\n");
+         snprintf(data_to_ettercap->info, sizeof(data_to_ettercap->info), "AUTH TYPE 0x%02x\n", parameters[i+2]);
+         strlcpy(data_to_ettercap->pass, parameters + i + 3, (len>28) ? 28 : (len+1));
          strcat(data_to_ettercap->pass, "\n");
       }
    }
