@@ -19,23 +19,15 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_dissector_icq.c,v 1.1.1.1 2001/08/27 19:25:42 alor Exp $
+    $Id: ec_dissector_icq.c,v 1.7 2001/12/13 17:05:20 alor Exp $
 */
 
 #include "include/ec_main.h"
 
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "include/ec_dissector.h"
 #include "include/ec_inet_structures.h"
 #include "include/ec_error.h"
 
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
-
 #define ICQ_HEADER_LENGTH              0x0018
 #define CMD_LOGIN                      0x03E8
 #define CMD_LOGIN_OFFSET               0x000E
@@ -141,14 +133,11 @@ FUNC_DISSECTOR(Dissector_icq)
 
    payload = (char *) (int)udp + UDP_HEADER;
 
-   if (ntohs(udp->source) == SERV_PORT) return 0;       // Skip server messages...
+   if (ntohs(udp->source) == SERV_PORT) return 0;  // Skip server messages...
    if (data_to_ettercap->datalen == 0) return 0;   // No data...
 
    memset(collector, 0, MAX_DATA);
-   if (Conn_Mode)
-      memcpy(collector, payload, data_to_ettercap->datalen);
-   else
-      memcpy(collector, payload, sniff_data_to_ettercap->datasize);
+   memcpy(collector, payload, data_to_ettercap->datalen);
 
    if( ptohs(collector) != 5) return 0;    // Not the right version
 
@@ -156,32 +145,32 @@ FUNC_DISSECTOR(Dissector_icq)
    Debug_msg("\tDissector_ICQ");
 #endif
 
-   if (Conn_Mode)
-   {
+   Decode_icq(collector, data_to_ettercap->datalen); // decrypting....
 
-      Decode_icq(collector, data_to_ettercap->datalen); // decrypting....
+   if ( ptohs(collector + CMD_LOGIN_OFFSET) != CMD_LOGIN) return 0; // Not a login packet
 
-      if ( ptohs(collector + CMD_LOGIN_OFFSET) != CMD_LOGIN) return 0; // Not a login packet
+   snprintf(data_to_ettercap->user, 25, "%ld (ICQ UIN)\n", ptohl(collector + CMD_LOGIN_UIN_OFFSET));  // the login (UIN)
 
-      snprintf(data_to_ettercap->user, 25, "%ld (ICQ UIN)\n", ptohl(collector + CMD_LOGIN_UIN_OFFSET));  // the login (UIN)
+   #ifdef DEBUG
+      Debug_msg("\tDissector_ICQ - LOGIN ");
+   #endif
 
-      #ifdef DEBUG
-         Debug_msg("\tDissector_ICQ - LOGIN ");
-      #endif
+   pwdlen = ptohs(collector + ICQ_HEADER_LENGTH + CMD_LOGIN_PASS_LENGHT_OFFSET);
+   if (pwdlen > 28) pwdlen = 28;
+   password = (char *) calloc(pwdlen+1, sizeof(char) );
 
-      pwdlen = ptohs(collector + ICQ_HEADER_LENGTH + CMD_LOGIN_PASS_LENGHT_OFFSET);
-      password = (char *) calloc(pwdlen+1, sizeof(char) );
+   strlcpy(password,(char *)(collector + ICQ_HEADER_LENGTH + CMD_LOGIN_PASS_OFFSET), pwdlen+1);
 
-      strncpy(password,(char *)(collector + ICQ_HEADER_LENGTH + CMD_LOGIN_PASS_OFFSET), pwdlen);
+   snprintf(data_to_ettercap->pass, sizeof(data_to_ettercap->pass), "%s\n", password);
+   sprintf(data_to_ettercap->type, "ICQ");
 
-      snprintf(data_to_ettercap->pass,30,"%s\n", password);
-      sprintf(data_to_ettercap->type, "ICQ");
+   free(password);
 
-      #ifdef DEBUG
-         Debug_msg("\tDissector_ICQ - PASS ");
-      #endif
-   }
-   else
+   #ifdef DEBUG
+      Debug_msg("\tDissector_ICQ - PASS ");
+   #endif
+
+   if (!Conn_Mode)
    {
       Decode_icq(collector, sniff_data_to_ettercap->datasize); // decrypting....
 
