@@ -19,24 +19,17 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_dissector_pcanywhere.c,v 1.1.1.1 2001/08/27 19:26:32 alor Exp $
+    $Id: ec_dissector_pcanywhere.c,v 1.5 2001/12/09 20:24:51 alor Exp $
 */
 
 #include "include/ec_main.h"
 
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <errno.h>
 
 #include "include/ec_dissector.h"
 #include "include/ec_inet_structures.h"
 #include "include/ec_error.h"
 
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
-
 // protos
 
 FUNC_DISSECTOR (Dissector_pcanywhere);
@@ -55,13 +48,16 @@ FUNC_DISSECTOR (Dissector_pcanywhere)
    ONLY_CONNECTION;
 
    tcp = (TCP_header *) data;
-   if (ntohs(tcp->source) == SERV_PORT) return 0;            // skip server messages...
+
+   if (ntohs(tcp->source) == SERV_PORT) return 0;     // skip server messages...
+   if (data_to_ettercap->datalen == 0) return 0;      // no data...
+
    payload = (char *)((int)tcp + tcp->doff * 4);
 
-   datalen=data_to_ettercap->datalen;
-   endbuf=payload+datalen;
+   datalen = data_to_ettercap->datalen;
+   endbuf = payload + datalen;
 
-   for(ptr=payload;(*ptr)==0;ptr++); // skip zero padding
+   for(ptr = payload; (*ptr) == 0; ptr++); // skip zero padding
 
    // Test version
    if ((*ptr)<0x0f && (*ptr)!=0x06) // Clear Text
@@ -69,16 +65,19 @@ FUNC_DISSECTOR (Dissector_pcanywhere)
       char buffer[1000];
       int  offset;
 
-      for(;(*ptr)!=0x06 && (u_long)ptr<(u_long)endbuf ;ptr++)
+      for(;(*ptr)!=0x06 && (u_long)ptr < (u_long)endbuf; ptr++)
       {
-         for(offset=0;(*ptr)!=13;ptr++) // search for \r
+         for(offset = 0; (*ptr)!=13 && offset<1000; ptr++) // search for \r
          {
              buffer[offset]=(*ptr);
              offset++;
          }
-         buffer[offset]=0;
+
+         if (offset == 1000) return 0;
+
+         buffer[offset] = 0;
       }
-      snprintf(data_to_ettercap->user, 25, "%s\n", buffer);
+      snprintf(data_to_ettercap->user, sizeof(data_to_ettercap->user), "%s\n", buffer);
 
       return 0;
    }
@@ -86,8 +85,8 @@ FUNC_DISSECTOR (Dissector_pcanywhere)
    {
       for(;;)
       {
-         for(;(*ptr)!=0x06;ptr++)
-            if ((u_long)ptr>=(u_long)endbuf) return 0;
+         for(;(*ptr)!=0x06; ptr++)
+            if ((u_long)ptr >= (u_long)endbuf) return 0;
 
          ptr++;
          if ((*ptr)==0xff && (u_long)(endbuf-ptr)>1)
@@ -97,13 +96,16 @@ FUNC_DISSECTOR (Dissector_pcanywhere)
 
             // Is it right?
             ptr+=2;
-            memcpy(buffer,ptr+1,(*ptr));
+
+            if (*ptr>1000) return 0;
+
+            memcpy(buffer, ptr+1, (*ptr));
 
             for (i=(*ptr)-1; i>0; i--)
                 buffer[i] = buffer[i-1] ^ buffer[i] ^ (i-1);
 
             buffer[0] ^= 0xab;
-                      snprintf(data_to_ettercap->user, 25, "%s\n", buffer);
+            snprintf(data_to_ettercap->user, sizeof(data_to_ettercap->user), "%s\n", buffer);
 
             return 0;
          }
