@@ -17,14 +17,11 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_dissector_telnet.c,v 1.1.1.1 2001/08/27 19:25:20 alor Exp $
+    $Id: ec_dissector_telnet.c,v 1.5 2001/12/09 20:24:51 alor Exp $
 */
 
 #include "include/ec_main.h"
 
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
 #ifdef HAVE_CTYPE_H
    #include <ctype.h>
 #endif
@@ -33,10 +30,6 @@
 #include "include/ec_inet_structures.h"
 #include "include/ec_error.h"
 
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
-
 // protos
 
 #ifndef HAVE_CTYPE_H
@@ -66,13 +59,13 @@ FUNC_DISSECTOR(Dissector_telnet)
 
    tcp = (TCP_header *) data;
 
-   if (ntohs(tcp->source) == SERV_PORT) return 0;            // skip server messages...
+   if (ntohs(tcp->source) == SERV_PORT) return 0;     // skip server messages...
    if (data_to_ettercap->datalen == 0) return 0;      // no data...
 
    payload = (char *)((int)tcp + tcp->doff * 4);
 
-   memset(collector, 0, 30);
-   datalen = (data_to_ettercap->datalen > 30) ? 30 : data_to_ettercap->datalen;
+   memset(collector, 0, sizeof(collector));
+   datalen = (data_to_ettercap->datalen > sizeof(collector)) ? sizeof(collector) : data_to_ettercap->datalen;
    strncpy(collector, payload, datalen);
 
    if (strcmp(collector, ""))
@@ -92,13 +85,13 @@ FUNC_DISSECTOR(Dissector_telnet)
       {
          if (end) strcat(collector, "\n");            // this is the terminator char for the data collection (ec_decodata.c)
 
-         strncpy(data_to_ettercap->user, collector, 29);
-         strncpy(data_to_ettercap->pass, collector, 29);
+         strlcpy(data_to_ettercap->user, collector, sizeof(data_to_ettercap->user)-1);
+         strlcpy(data_to_ettercap->pass, collector, sizeof(data_to_ettercap->pass)-1);
       }
       else if (end)
       {
-         strcpy(data_to_ettercap->user, "\n");
-         strcpy(data_to_ettercap->pass, "\n");
+         sprintf(data_to_ettercap->user, "\n");
+         sprintf(data_to_ettercap->pass, "\n");
       }
    }
 
