@@ -17,59 +17,49 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_dryad.c,v 1.3 2001/09/13 20:15:48 alor Exp $
+    $Id: ec_dryad.c,v 1.9 2001/12/09 20:24:51 alor Exp $
 */
 
-#include <stdio.h>
-#include <unistd.h>
+#include "include/ec_main.h"
+
 #include <fcntl.h>
-#include <stdlib.h>
-#include <string.h>
 #include <time.h>
 #include <sys/time.h>
 #include <math.h>
 #include <ctype.h>
 
-#include "include/ec_main.h"
 #include "include/ec_inet.h"
 #include "include/ec_inet_forge.h"
 #include "include/ec_inet_structures.h"
 #include "include/ec_buffer.h"
 #include "include/ec_error.h"
 #include "include/ec_dissector.h"
-
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
+#include "include/ec_thread.h"
+#include "include/ec_decodedata.h"
 
 // protos
 
-int Dryad_Run(void);
-void Dryad_Main(void);
+pthread_t Dryad_Run(void);
+void * Dryad_Main(void *);
 void Dryad_Get_Banner(PASSIVE_DATA *, CONNECTION *, u_char, u_char *, int);
 
 //----------------------------------------
 
 
-int Dryad_Run(void)
+pthread_t Dryad_Run(void)
 {
-   int Dryad_pid;
 
 #ifdef DEBUG
    Debug_msg("Dryad_Run");
 #endif
 
-   if (!(Dryad_pid = fork()))
-      Dryad_Main();
-   else
-      return Dryad_pid;
+   return ECThread_create("dryad", &Dryad_Main, NULL);
 
-   return 0;
 }
 
 
 
-void Dryad_Main(void)
+void * Dryad_Main(void *dummy)
 {
    char *buf;
    int sock, MTU;
@@ -85,8 +75,6 @@ void Dryad_Main(void)
    PASSIVE_DATA data_to_ettercap;
    CONNECTION state_machine_data;
 
-   strcpy(program_argv0, "dryad   ");
-
 #ifdef DEBUG
    Debug_msg("Dryad_Main");
 #endif
@@ -107,7 +95,7 @@ void Dryad_Main(void)
       short len=0;
       char goodpkt = 0;
 
-      if (getppid() == 1) exit(0);
+      pthread_testcancel();
 
       memset(buf, 0, MTU);
 
@@ -139,7 +127,7 @@ void Dryad_Main(void)
                strcpy(data_to_ettercap.ip, int_ntoa(arp->source_ip));      // STORE THE IP
 
                if ((*(u_long *)(arp->source_ip) & NetMask) != MyIP)    // Non Local IP
-                  strcpy(data_to_ettercap.type, "NL");   // possible at this level ??? misconfigured switches ???
+                  sprintf(data_to_ettercap.type, "NL");   // possible at this level ??? misconfigured switches ???
 
                goodpkt = 1;
             }
@@ -158,7 +146,7 @@ void Dryad_Main(void)
             strcpy(state_machine_data.dest_ip, int_ntoa(ip->dest_ip));
 
             if ((ip->source_ip & NetMask) != MyIP)    // Non Local IP
-               strcpy(data_to_ettercap.type, "NL");
+               sprintf(data_to_ettercap.type, "NL");
 
             #define UPPER_LIMIT(x,y) ((x<y) ? x : y)
             TTL = UPPER_LIMIT(255, pow(2, ceil(log(ip->ttl)/log(2))));   // round the TTL to the nearest power of 2 (ceiling)
@@ -180,7 +168,7 @@ void Dryad_Main(void)
                   case 11: // TTL-time-exceded
                   case 5:  // redirect
                   //case 3:   // Destination unreachable
-                           strcpy(data_to_ettercap.type, "RT");      // this type is issued by a router
+                           sprintf(data_to_ettercap.type, "RT");      // this type is issued by a router
                            break;
                }
 
@@ -267,11 +255,11 @@ void Dryad_Main(void)
                      }
                   }
 
-                  if (WS == -1) strcpy(WSS, "WS");
-                  else sprintf(WSS, "%02d", WS);
+                  if (WS == -1) sprintf(WSS, "WS");
+                  else snprintf(WSS, sizeof(WSS), "%02d", WS);
 
-                  if (MSS == -1) strcpy(_MSS, "_MSS");
-                  else sprintf(_MSS, "%04X", MSS);
+                  if (MSS == -1) sprintf(_MSS, "_MSS");
+                  else snprintf(_MSS, sizeof(_MSS), "%04X", MSS);
 
                   if (tcp->flags & TH_ACK)
                   {
@@ -279,8 +267,9 @@ void Dryad_Main(void)
                         data_to_ettercap.port = ntohs(tcp->source);     // STORE THE TCP OPEN PORT
                   }
 
-                  sprintf(data_to_ettercap.fingerprint, "%04X:%s:%02X:%s:%d:%d:%d:%d:%c",
-                                          WIN, _MSS, TTL, WSS , S, N, D, T, (tcp->flags & TH_ACK) ? 'A' : 'S');
+                  snprintf(data_to_ettercap.fingerprint, sizeof(data_to_ettercap.fingerprint),
+                           "%04X:%s:%02X:%s:%d:%d:%d:%d:%c",
+                           WIN, _MSS, TTL, WSS , S, N, D, T, (tcp->flags & TH_ACK) ? 'A' : 'S');
 
                } // if SYN
 
@@ -292,7 +281,7 @@ void Dryad_Main(void)
       else
          usleep(1);
 
-      if (goodpkt) Buffer_Put(pipe_with_dryad, &data_to_ettercap, sizeof(PASSIVE_DATA));
+      if (goodpkt) Decodedata_MakePassiveList(&data_to_ettercap);
    }
 
 }
@@ -304,7 +293,6 @@ void Dryad_Get_Banner(PASSIVE_DATA *data_to_ettercap, CONNECTION *state_machine_
    char info[20];
    int i;
 
-
    if ( (flags & TH_SYN) && (flags & TH_ACK) )  // SYN + ACK
    {
       sprintf(info, "%d", state_machine_data->source_port);
@@ -333,7 +321,8 @@ void Dryad_Get_Banner(PASSIVE_DATA *data_to_ettercap, CONNECTION *state_machine_
                for (i=0; i<datalen; i++)
                      if ( data[i] == '\r' ) data[i] = 0;
 
-               strncpy(data_to_ettercap->banner, ptr+8, 149);
+               strlcpy(data_to_ettercap->banner, ptr+8, sizeof(data_to_ettercap->banner));
+
                data_to_ettercap->port = state_machine_data->source_port;
 
                Dissector_StateMachine_SetStatus(state_machine_data, 0, NULL);
@@ -342,7 +331,6 @@ void Dryad_Get_Banner(PASSIVE_DATA *data_to_ettercap, CONNECTION *state_machine_
          else
          {
             u_char *ptr = data;
-
             for ( ; ptr < data + datalen ; )
             {
                if ( *ptr == 0xff )     // skip telnet options (if it is telnet obviously)
@@ -357,22 +345,29 @@ void Dryad_Get_Banner(PASSIVE_DATA *data_to_ettercap, CONNECTION *state_machine_
                   }
                }
                else if ( *ptr == '\r' || *ptr == '\n' || *ptr == 0) ptr++;
-               if ( isprint(*ptr) ) break;
+               else break;
             }
 
             if (strlen(ptr))
             {
                for (i=0 ; i < strlen(ptr); i++ )
+               {
+                  if (!isprint(ptr[i]))
+                  {
+                     ptr[i]=0;
+                     break;
+                  }
                   if (ptr[i] == '\n' || ptr[i] == '\r') ptr[i] = ' ';
+               }
+
+               strlcpy(data_to_ettercap->banner, ptr, sizeof(data_to_ettercap->banner));
 
-               strncpy(data_to_ettercap->banner, ptr, 149);
                data_to_ettercap->port = state_machine_data->source_port;
                Dissector_StateMachine_SetStatus(state_machine_data, 0, NULL);
             }
          }
       }
    }
-
 }
 
 /* EOF */
