@@ -17,18 +17,14 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_fingerprint.c,v 1.2 2001/09/06 20:07:42 alor Exp $
+    $Id: ec_fingerprint.c,v 1.7 2001/12/08 11:22:20 alor Exp $
 */
 
 #include "include/ec_main.h"
 
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <fcntl.h>
 #include <sys/time.h>
 #include <time.h>
-#include <unistd.h>
 
 #include "include/ec_inet_structures.h"
 #include "include/ec_inet.h"
@@ -36,10 +32,6 @@
 #include "include/ec_decodedata.h"
 #include "include/ec_error.h"
 
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
-
 #ifdef HAVE_SYS_UTSNAME_H
    #include <sys/utsname.h>
 #endif
@@ -287,13 +279,9 @@ char *Fingerprint_Make_Finger_print()
    char *condition;
    char *test;
    static char osname[500];
-   char database[sizeof(INSTALL_PATH)+22];
    static char res_list[2000];
 
-   strcpy(database, INSTALL_PATH);
-   strcat(database, "nmap-os-fingerprints");
-
-   file = fopen(database, "r");
+   file = fopen(DATA_PATH "/nmap-os-fingerprints", "r");
    if (!file)
       file = fopen("./share/nmap-os-fingerprints","r");
       if (!file)
@@ -316,7 +304,7 @@ char *Fingerprint_Make_Finger_print()
       {
          char *deadend;
 
-         sprintf(num_test,"T%1d",i);
+         snprintf(num_test, sizeof(num_test), "T%1d", i);
          while(fgets(dummy,500,file) && !strstr(dummy,num_test));
 
          test=dummy+3;
@@ -336,7 +324,7 @@ char *Fingerprint_Make_Finger_print()
       }
       if (!no_match)
       {
-         strcat(res_list,osname+sizeof("Fingerprint"));
+         strlcat(res_list, osname+sizeof("Fingerprint"), sizeof(res_list));
       }
    }
    fclose(file);
@@ -411,9 +399,9 @@ char * Fingerprint_OS(char *IP)
 
          res_name = (char *)malloc(50);
          uname(&buf);
-         strcpy(res_name, buf.sysname);
-         strcat(res_name, " ");
-         strcat(res_name, buf.release);
+         strlcpy(res_name, buf.sysname, 50);
+         strlcat(res_name, " ", 50);
+         strlcat(res_name, buf.release, 50);
          return res_name;
       #else
          return "Yourself ;) try `uname -sr`";
@@ -470,7 +458,7 @@ char * Fingerprint_MAC(char *MAC)
 
       mac_list = mac_index;
 
-      fto = fopen(INSTALL_PATH "mac-fingerprints", "r");
+      fto = fopen(DATA_PATH "/mac-fingerprints", "r");
       if (!fto)
          fto = fopen("./share/mac-fingerprints","r");
          if (!fto)
@@ -487,8 +475,8 @@ char * Fingerprint_MAC(char *MAC)
          if ( (mac_index->next = ( struct mac_database *) calloc (1, sizeof(mac_database))) == NULL)
             ERROR_MSG("calloc()");
 
-         strncpy(mac_index->mac, line, 8);
-         strncpy(mac_index->vendor, line+10, 60);
+         strlcpy(mac_index->mac, line, sizeof(mac_index->mac));
+         strlcpy(mac_index->vendor, line+10, 60);
 
          mac_index = (mac_database *) mac_index->next;
       }
