@@ -17,18 +17,15 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_illithid.c,v 1.5 2001/09/11 21:03:06 alor Exp $
+    $Id: ec_illithid.c,v 1.15 2001/12/13 18:58:24 alor Exp $
 */
 
-#include <stdio.h>
-#include <stdlib.h>
+#include "include/ec_main.h"
+
 #include <sys/time.h>
-#include <string.h>
-#include <unistd.h>
 #include <fcntl.h>
 #include <signal.h>
 
-#include "include/ec_main.h"
 #include "include/ec_inet.h"
 #include "include/ec_inet_forge.h"
 #include "include/ec_inet_structures.h"
@@ -37,10 +34,9 @@
 #include "include/ec_dissector.h"
 #include "include/ec_filterdrop.h"
 #include "include/ec_parser.h"
-
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
+#include "include/ec_thread.h"
+#include "include/ec_decodedata.h"
+#include "include/ec_plugins.h"
 
 #define HASHES 256
 #define PCK_STOD 1
@@ -74,98 +70,150 @@ typedef struct {
 } Ielem;
 
 
+typedef struct {
+    short mode;
+    char iface[10];
+    char IP1p[16];
+    char IP2p[16];
+    char MAC1p[20];
+    char MAC2p[20];
+} illithid_param;
+
+
 // global data
 int sock, Connection_Mode=1;
 unsigned char MyMAC[6];
 static Ielem* hash_table[HASHES];
 int illithid_gwip;
+int illithid_buffer = -1;
+pthread_t decoder_pid;
+
+pthread_mutex_t decoder_mtx = PTHREAD_MUTEX_INITIALIZER;
+pthread_cond_t decoder_cond = PTHREAD_COND_INITIALIZER;
 
 // protos...
 
-int Illithid_ARPBased_GetConnections(char *iface, char *IP1p, char *IP2p, char *MAC1, char *MAC2);
-int Illithid_PublicARP_GetConnections(char *iface, char *IP1p, char *IP2p, char *MAC1, char *MAC2);
-int Illithid_IPBased_GetConnections(char *iface, char *IP1p, char *IP2p);
-int Illithid_MACBased_GetConnections(char *iface, char *MAC1p, char *MAC2p);
-int Illithid_ToBeSniffed(char *source, int psource, char *dest, int pdest, SNIFFED_DATA *data);
-int Illithid_ToBeSniffed_ip(char *source, char *dest, CONNECTION *data);
+pthread_t Illithid_ARPBased_GetConnections(char *iface, char *IP1p, char *IP2p, char *MAC1, char *MAC2);
+pthread_t Illithid_PublicARP_GetConnections(char *iface, char *IP1p, char *IP2p, char *MAC1, char *MAC2);
+pthread_t Illithid_IPBased_GetConnections(char *iface, char *IP1p, char *IP2p);
+pthread_t Illithid_MACBased_GetConnections(char *iface, char *MAC1p, char *MAC2p);
+
+int Illithid_ToBeSniffed(SNIFFED_DATA *data);
+int Illithid_ToBeSniffed_ip(u_long source, u_long dest, CONNECTION *data);
 int Illithid_ToBeSniffed_mac(char *source, char *dest, CONNECTION *data);
-void Illithid_GetConnections(short mode, char *iface, char *IP1p, char *IP2p, char *MAC1p, char *MAC2p);
-void Illithid_Reset_Conn(void);
+
+void * Illithid_GetConnections(void *);
+void Illithid_Reset_Conn(void *);
 Ielem **Illithid_Find_Session(unsigned int IP1, unsigned int IP2, unsigned short Port1, unsigned short Port2, char proto, Ielem *outs);
 void Illithid_Set_Session(Ielem **session, Ielem *outs, short mode);
+void * Illithid_Decoder(void *dummy);
+void Illithid_Decoder_Put(CONNECTION *data);
 
 //---------------------------
 
-
-int Illithid_ARPBased_GetConnections(char *iface, char *IP1p, char *IP2p, char *MAC1p, char *MAC2p)
+void * Illithid_Decoder(void *dummy)      // decoder thread
 {
-   int Illithid_pid;
+   CONNECTION data_to_ettercap;
+   Buffer_Flush(illithid_buffer);
+   loop
+   {
+      pthread_testcancel();
+
+      if (Buffer_Get(illithid_buffer, &data_to_ettercap, sizeof(CONNECTION)) > 0)
+         Decodedata_MakeConnectionList(&data_to_ettercap);
+      else
+      {
+         pthread_cond_wait(&decoder_cond, &decoder_mtx);
+      }
+   }
+}
+
+
+void Illithid_Decoder_Put(CONNECTION *data)
+{
+   Buffer_Put(illithid_buffer, data, sizeof(CONNECTION));
+   pthread_cond_signal(&decoder_cond);
+}
+
+
+pthread_t Illithid_ARPBased_GetConnections(char *iface, char *IP1p, char *IP2p, char *MAC1p, char *MAC2p)
+{
+   static illithid_param param;
 
 #ifdef DEBUG
    Debug_msg("Illithid_ARPBased_GetConnections -- [%s] [%s] [%s] [%s]", IP1p, IP2p, MAC1p, MAC2p);
 #endif
 
-   if (!(Illithid_pid = fork()))
-      Illithid_GetConnections(ARPBASED, iface, IP1p, IP2p, MAC1p, MAC2p);
-   else
-      return Illithid_pid;
+   param.mode = ARPBASED;
+   strlcpy(param.iface, iface, sizeof(param.iface));
+   strlcpy(param.IP1p, IP1p, sizeof(param.IP1p));
+   strlcpy(param.IP2p, IP2p, sizeof(param.IP2p));
+   strlcpy(param.MAC1p, MAC1p, sizeof(param.MAC1p));
+   strlcpy(param.MAC2p, MAC2p, sizeof(param.MAC2p));
 
-   return 0;
+   return ECThread_create("illithid", &Illithid_GetConnections, &param);
 }
 
-int Illithid_PublicARP_GetConnections(char *iface, char *IP1p, char *IP2p, char *MAC1p, char *MAC2p)
+pthread_t Illithid_PublicARP_GetConnections(char *iface, char *IP1p, char *IP2p, char *MAC1p, char *MAC2p)
 {
-   int Illithid_pid;
+   static illithid_param param;
 
 #ifdef DEBUG
    Debug_msg("Illithid_PublicARP_GetConnections -- [%s] [%s] [%s] [%s]", IP1p, IP2p, MAC1p, MAC2p);
 #endif
 
-   if (!(Illithid_pid = fork()))
-      Illithid_GetConnections(PUBLICARP, iface, IP1p, IP2p, MAC1p, MAC2p);
-   else
-      return Illithid_pid;
+   param.mode = PUBLICARP;
+   strlcpy(param.iface, iface, sizeof(param.iface));
+   strlcpy(param.IP1p, IP1p, sizeof(param.IP1p));
+   strlcpy(param.IP2p, IP2p, sizeof(param.IP2p));
+   strlcpy(param.MAC1p, MAC1p, sizeof(param.MAC1p));
+   strlcpy(param.MAC2p, MAC2p, sizeof(param.MAC2p));
 
-   return 0;
+   return ECThread_create("illithid", &Illithid_GetConnections, &param);
 }
 
 
 
-int Illithid_IPBased_GetConnections(char *iface, char *IP1p, char *IP2p)
+pthread_t Illithid_IPBased_GetConnections(char *iface, char *IP1p, char *IP2p)
 {
-      int Illithid_pid;
+   static illithid_param param;
 
 #ifdef DEBUG
    Debug_msg("Illithid_IPBased_GetConnections -- [%s] [%s]", IP1p, IP2p);
 #endif
 
-   if (!(Illithid_pid = fork()))
-           Illithid_GetConnections(IPBASED, iface, IP1p, IP2p, "", "");
-   else
-      return Illithid_pid;
+   param.mode = IPBASED;
+   strlcpy(param.iface, iface, sizeof(param.iface));
+   strlcpy(param.IP1p, IP1p, sizeof(param.IP1p));
+   strlcpy(param.IP2p, IP2p, sizeof(param.IP2p));
+   strcpy(param.MAC1p, "");
+   strcpy(param.MAC2p, "");
+
+   return ECThread_create("illithid", &Illithid_GetConnections, &param);
 
-   return 0;
 }
 
 
-int Illithid_MACBased_GetConnections(char *iface, char *MAC1p, char *MAC2p)
+pthread_t Illithid_MACBased_GetConnections(char *iface, char *MAC1p, char *MAC2p)
 {
-      int Illithid_pid;
+   static illithid_param param;
 
 #ifdef DEBUG
    Debug_msg("Illithid_MACBased_GetConnections -- [%s] [%s]", MAC1p, MAC2p);
 #endif
 
-   if (!(Illithid_pid = fork()))
-      Illithid_GetConnections(MACBASED, iface, "", "", MAC1p, MAC2p);
-   else
-      return Illithid_pid;
+   param.mode = MACBASED;
+   strlcpy(param.iface, iface, sizeof(param.iface));
+   strcpy(param.IP1p, "");
+   strcpy(param.IP2p, "");
+   strlcpy(param.MAC1p, MAC1p, sizeof(param.MAC1p));
+   strlcpy(param.MAC2p, MAC2p, sizeof(param.MAC2p));
 
-   return 0;
+   return ECThread_create("illithid", &Illithid_GetConnections, &param);
 }
 
 
-void Illithid_GetConnections(short mode, char *iface, char *IP1p, char *IP2p, char *MAC1p, char *MAC2p)
+void * Illithid_GetConnections(void *param)
 {
    int len, MTU, delta=0, datalen, nchars, IP1=0, IP2=0;
    short pkttype, forward_mode;
@@ -184,55 +232,58 @@ void Illithid_GetConnections(short mode, char *iface, char *IP1p, char *IP2p, ch
    unsigned char MAC1[6];
    unsigned char MAC2[6];
    unsigned char GWMAC[6];
-   char filter_status;
-
-   strcpy(program_argv0, "illithid");
 
    exit_func(Illithid_Reset_Conn);
 
-   if (inet_aton(IP1p,&addr))
+   if (illithid_buffer == -1) illithid_buffer = Buffer_Create(5.0e4);   // 50 Kb
+   decoder_pid = ECThread_create("decoder", &Illithid_Decoder, NULL);
+
+   if (inet_aton(((illithid_param *)param)->IP1p,&addr))
        IP1 = addr.s_addr;
-   if (inet_aton(IP2p,&addr))
+   if (inet_aton(((illithid_param *)param)->IP2p,&addr))
        IP2 = addr.s_addr;
 
-   Inet_GetMACfromString(MAC1p, MAC1);
-   Inet_GetMACfromString(MAC2p, MAC2);
+   Inet_GetMACfromString(((illithid_param *)param)->MAC1p, MAC1);
+   Inet_GetMACfromString(((illithid_param *)param)->MAC2p, MAC2);
 
    if (illithid_gwip) memcpy(GWMAC, Inet_MacFromIP(illithid_gwip), 6);  // get the gateway's mac for smart arp on a client
 
-   sock = Inet_OpenRawSock(iface);
-   Inet_GetIfaceInfo( iface, &MTU, MyMAC, &MyIP, &NetMask);
+   sock = Inet_OpenRawSock(((illithid_param *)param)->iface);
+   Inet_GetIfaceInfo( ((illithid_param *)param)->iface, &MTU, MyMAC, &MyIP, &NetMask);
 
    fcntl(sock,F_SETFL,O_NONBLOCK);
    fcntl(pipe_inject[0], F_SETFL, O_NONBLOCK);
-   fcntl(pipe_filter[0], F_SETFL, O_NONBLOCK);
    fcntl(pipe_kill[0], F_SETFL, O_NONBLOCK);
 
-   forward_mode = mode;
-   if (mode == ARPBASED && (!strcmp(IP1p,"") || !strcmp(IP2p,"")) )
+   forward_mode = ((illithid_param *)param)->mode;
+   if (((illithid_param *)param)->mode == ARPBASED && ( IP1 == 0 || IP2 == 0 ) )
        forward_mode = PUBLICARP;
 
-   if (mode > PUBLICARP)
-      Inet_SetPromisc(iface);
+   if (((illithid_param *)param)->mode > PUBLICARP)
+      Inet_SetPromisc(((illithid_param *)param)->iface);
    else
       IP_Test=MyIP&NetMask;
 
-   buffer = Inet_Forge_packet(MTU);
+   buffer = Inet_Forge_packet(MTU*2);
    ins_pck = Inet_Forge_packet(MTU);
 
    loop
    {
-      if (getppid() == 1) exit(0);
+      pthread_testcancel();
       delta = 0;
       memset(buffer, 0, MTU);    // REQUIRED BY DISSECTORS !!!
 
       len = Inet_GetRawPacket(sock, buffer, MTU, &pkttype);
 
-      if (len > 0 && ( mode > PUBLICARP || pkttype == PACKET_HOST))
+      if (len > 0 && ( ((illithid_param *)param)->mode > PUBLICARP || pkttype == PACKET_HOST))
       {
          char PckDir = PCK_DROP;
          int dontforward = 0;
 
+#ifdef PERMIT_PLUGINS
+         Plugin_HookPoint(PCK_RECEIVED_RAW, buffer);     // HOOK POINT: PCK_RECEIVED_RAW
+#endif
+
          memset(&data_to_ettercap, 0, sizeof(CONNECTION));
          memset(&sniff_data_to_ettercap, 0, sizeof(SNIFFED_DATA));
          Act_Connection_Mode = Connection_Mode;
@@ -245,15 +296,24 @@ void Illithid_GetConnections(short mode, char *iface, char *IP1p, char *IP2p, ch
          if ( ntohs(eth->type) == ETH_P_IP )
          {
             ip = (IP_header *)(eth+1);
-            strcpy(data_to_ettercap.source_ip, int_ntoa(ip->source_ip) );
-            strcpy(data_to_ettercap.dest_ip, int_ntoa(ip->dest_ip) );
-            strcpy(sniff_data_to_ettercap.source_ip, int_ntoa(ip->source_ip) );
-            strcpy(sniff_data_to_ettercap.dest_ip, int_ntoa(ip->dest_ip) );
+
+            data_to_ettercap.fast_source_ip = ntohl(ip->source_ip);
+            data_to_ettercap.fast_dest_ip = ntohl(ip->dest_ip);
+            strlcpy(data_to_ettercap.source_ip, int_ntoa(ip->source_ip), sizeof(data_to_ettercap.source_ip));
+            strlcpy(data_to_ettercap.dest_ip, int_ntoa(ip->dest_ip), sizeof(data_to_ettercap.dest_ip));
+
+            if (!Act_Connection_Mode)
+            {
+               sniff_data_to_ettercap.fast_source_ip = ntohl(ip->source_ip);
+               sniff_data_to_ettercap.fast_dest_ip = ntohl(ip->dest_ip);
+               strlcpy(sniff_data_to_ettercap.source_ip, int_ntoa(ip->source_ip), sizeof(sniff_data_to_ettercap.source_ip));
+               strlcpy(sniff_data_to_ettercap.dest_ip, int_ntoa(ip->dest_ip), sizeof(sniff_data_to_ettercap.dest_ip));
+            }
 
             if (forward_mode == MACBASED)
                PckDir = Illithid_ToBeSniffed_mac(MAC1, MAC2, &data_to_ettercap);
             else if ( forward_mode == IPBASED)
-               PckDir = Illithid_ToBeSniffed_ip(IP1p, IP2p, &data_to_ettercap);
+               PckDir = Illithid_ToBeSniffed_ip(ntohl(IP1), ntohl(IP2), &data_to_ettercap);
             else if ( forward_mode == ARPBASED && (ip->dest_ip==IP1 || ip->source_ip==IP2) && (((ip->dest_ip&NetMask)!=IP_Test || (ip->source_ip&NetMask)!=IP_Test)
                  || (ip->dest_ip==IP1 && ip->source_ip==IP2)))
             {
@@ -270,19 +330,19 @@ void Illithid_GetConnections(short mode, char *iface, char *IP1p, char *IP2p, ch
             }
             else if (forward_mode == PUBLICARP)
             {
-               if (!strcmp(data_to_ettercap.dest_ip,IP1p))
+               if (data_to_ettercap.fast_dest_ip == ntohl(IP1))
                {
                   memcpy(eth->dest_mac, MAC1, 6);
                   PckDir = PCK_DTOS;
                }
-               else if (!strcmp(data_to_ettercap.dest_ip,IP2p))
+               else if (data_to_ettercap.fast_dest_ip == ntohl(IP2))
                {
                   memcpy(eth->dest_mac, MAC2, 6);
                   PckDir = PCK_STOD;
                }
                else if ((ip->dest_ip&NetMask)!=IP_Test)
                {
-                  if (!strcmp(IP1p,""))
+                  if ( IP1 == 0)  // 0 its ANY
                   {
                      if (memcmp(eth->source_mac, MAC2, 6))
                      {
@@ -313,7 +373,7 @@ void Illithid_GetConnections(short mode, char *iface, char *IP1p, char *IP2p, ch
                {
                   int i;
 
-                  if (!strcmp(IP1p,""))
+                  if ( IP1 == 0 )   // 0 its ANY
                      PckDir = PCK_DTOS;
                   else
                      PckDir = PCK_STOD;
@@ -346,35 +406,54 @@ void Illithid_GetConnections(short mode, char *iface, char *IP1p, char *IP2p, ch
 
                if (!( ntohs(ip->frag_and_flags) & IP_OFFMASK))
                {
+                  DISSECTION dissector_data;
+
                   tcp_data = (char *)((int)tcp + tcp->doff * 4);
                   tcp_datalen = (int)ip + ntohs(ip->t_len) - (int)tcp_data;
 
-                  if (Act_Connection_Mode)
+                  if (tcp_datalen < 0 || ntohs(ip->t_len) > len) goto send;
+
+                  data_to_ettercap.source_port = ntohs(tcp->source);
+                  data_to_ettercap.dest_port = ntohs(tcp->dest);
+                  data_to_ettercap.source_seq = ntohl(tcp->seq) + tcp_datalen;
+                  data_to_ettercap.dest_seq = ntohl(tcp->seq) + tcp_datalen;
+                  data_to_ettercap.flags = tcp->flags;
+                  data_to_ettercap.proto = 'T';
+                  data_to_ettercap.datalen = tcp_datalen;
+
+                  if (!Act_Connection_Mode)
                   {
-                      data_to_ettercap.source_port = ntohs(tcp->source);
-                      data_to_ettercap.dest_port = ntohs(tcp->dest);
-                      data_to_ettercap.source_seq = ntohl(tcp->seq) + tcp_datalen;
-                      data_to_ettercap.dest_seq = ntohl(tcp->seq) + tcp_datalen;
-                      data_to_ettercap.flags = tcp->flags;
-                      data_to_ettercap.proto = 'T';
-                      data_to_ettercap.datalen = tcp_datalen;
-                      dontforward = Dissector_Connections( mode, IPPROTO_TCP, (u_char *)tcp, &data_to_ettercap, Act_Connection_Mode);
-                      if (!dontforward) Buffer_Put(pipe_with_illithid, &data_to_ettercap, sizeof(CONNECTION));
+                     sniff_data_to_ettercap.source_port = ntohs(tcp->source);
+                     sniff_data_to_ettercap.dest_port = ntohs(tcp->dest);
+                     sniff_data_to_ettercap.seq = ntohl(tcp->seq);
+                     sniff_data_to_ettercap.ack_seq = ntohl(tcp->ack_seq);
+                     sniff_data_to_ettercap.flags = tcp->flags;
+                     sniff_data_to_ettercap.proto = 'T';
+                     tcp_datalen = (tcp_datalen > MAX_DATA) ? MAX_DATA : tcp_datalen;
+                     sniff_data_to_ettercap.datasize = tcp_datalen;
+                     memset(&sniff_data_to_ettercap.data, 0, sizeof(sniff_data_to_ettercap.data));
+                     memcpy(&sniff_data_to_ettercap.data, tcp_data, tcp_datalen);
+#ifdef PERMIT_PLUGINS
+                     Plugin_HookPoint(PCK_RECEIVED_STRUCT_FILLED, &sniff_data_to_ettercap);     // HOOK POINT: PCK_RECEIVED_STRUCT_FILLED
+#endif
                   }
-                  else
+
+                  dontforward = Dissector_Connections( ((illithid_param *)param)->mode, IPPROTO_TCP, (u_char *)tcp, &data_to_ettercap, &sniff_data_to_ettercap, Act_Connection_Mode);
+
+                  dissector_data.layer4 = (u_char *)tcp;
+                  dissector_data.connection = &data_to_ettercap;
+#ifdef PERMIT_PLUGINS
+                  Plugin_HookPoint(PCK_DISSECTOR, &dissector_data);     // HOOK POINT: PCK_DISSECTOR
+#endif
+                  if (!dontforward) Illithid_Decoder_Put(&data_to_ettercap);
+
+                  if (!Act_Connection_Mode)
                   {
-                      sniff_data_to_ettercap.source_port = ntohs(tcp->source);
-                      sniff_data_to_ettercap.dest_port = ntohs(tcp->dest);
-                      sniff_data_to_ettercap.seq = ntohl(tcp->seq);
-                      sniff_data_to_ettercap.ack_seq = ntohl(tcp->ack_seq);
-                      sniff_data_to_ettercap.flags = tcp->flags;
-                      sniff_data_to_ettercap.proto = 'T';
-                      tcp_datalen = (tcp_datalen > MAX_DATA) ? MAX_DATA : tcp_datalen;
-                      sniff_data_to_ettercap.datasize = tcp_datalen;
-                      memset(&sniff_data_to_ettercap.data, 0, sizeof(sniff_data_to_ettercap.data));
-                      memcpy(&sniff_data_to_ettercap.data, tcp_data, tcp_datalen);
-                      dontforward = Dissector_Connections( mode, IPPROTO_TCP, (u_char *)tcp, &sniff_data_to_ettercap, Act_Connection_Mode);
-                      if (!dontforward) Buffer_Put(pipe_with_illithid_data, &sniff_data_to_ettercap, sizeof(SNIFFED_DATA));
+#ifdef PERMIT_PLUGINS
+                     Plugin_HookPoint(PCK_DECODED, &sniff_data_to_ettercap);     // HOOK POINT: PCK_DECODED
+#endif
+                     if (!dontforward && Illithid_ToBeSniffed(&sniff_data_to_ettercap))
+                        Buffer_Put(pipe_with_illithid_data, &sniff_data_to_ettercap, sizeof(SNIFFED_DATA));
                   }
                }
             }
@@ -387,19 +466,19 @@ void Illithid_GetConnections(short mode, char *iface, char *IP1p, char *IP2p, ch
 
                if (!( ntohs(ip->frag_and_flags) & IP_OFFMASK))
                {
+                  DISSECTION dissector_data;
+
                   udp_data = (char *)((int)udp + UDP_HEADER);
                   udp_datalen = ntohs(udp->len) - UDP_HEADER;
 
-                  if (Act_Connection_Mode)
-                  {
-                     data_to_ettercap.source_port = ntohs(udp->source);
-                     data_to_ettercap.dest_port = ntohs(udp->dest);
-                     data_to_ettercap.proto = 'U';
-                     data_to_ettercap.datalen = udp_datalen;
-                     dontforward = Dissector_Connections( mode, IPPROTO_UDP, (u_char *)udp, &data_to_ettercap, Act_Connection_Mode);
-                     if (!dontforward) Buffer_Put(pipe_with_illithid, &data_to_ettercap, sizeof(CONNECTION));
-                  }
-                  else
+                  if (udp_datalen < 0 || udp_datalen > len) goto send;
+
+                  data_to_ettercap.source_port = ntohs(udp->source);
+                  data_to_ettercap.dest_port = ntohs(udp->dest);
+                  data_to_ettercap.proto = 'U';
+                  data_to_ettercap.datalen = udp_datalen;
+
+                  if(!Act_Connection_Mode)
                   {
                      sniff_data_to_ettercap.source_port = ntohs(udp->source);
                      sniff_data_to_ettercap.dest_port = ntohs(udp->dest);
@@ -408,8 +487,27 @@ void Illithid_GetConnections(short mode, char *iface, char *IP1p, char *IP2p, ch
                      sniff_data_to_ettercap.datasize = udp_datalen;
                      memset(&sniff_data_to_ettercap.data, 0, sizeof(sniff_data_to_ettercap.data));
                      memcpy(&sniff_data_to_ettercap.data, udp_data, udp_datalen);
-                     dontforward = Dissector_Connections( mode, IPPROTO_UDP, (u_char *)udp, &sniff_data_to_ettercap, Act_Connection_Mode);
-                     if (!dontforward) Buffer_Put(pipe_with_illithid_data, &sniff_data_to_ettercap, sizeof(SNIFFED_DATA));
+#ifdef PERMIT_PLUGINS
+                     Plugin_HookPoint(PCK_RECEIVED_STRUCT_FILLED, &sniff_data_to_ettercap);     // HOOK POINT: PCK_RECEIVED_STRUCT_FILLED
+#endif
+                  }
+
+                  dontforward = Dissector_Connections( ((illithid_param *)param)->mode, IPPROTO_UDP, (u_char *)udp, &data_to_ettercap, &sniff_data_to_ettercap, Act_Connection_Mode);
+
+                  dissector_data.layer4 = (u_char *)udp;
+                  dissector_data.connection = &data_to_ettercap;
+#ifdef PERMIT_PLUGINS
+                  Plugin_HookPoint(PCK_DISSECTOR, &dissector_data);     // HOOK POINT: PCK_DISSECTOR
+#endif
+                  if (!dontforward) Illithid_Decoder_Put(&data_to_ettercap);
+
+                  if (!Act_Connection_Mode)
+                  {
+#ifdef PERMIT_PLUGINS
+                     Plugin_HookPoint(PCK_DECODED, &sniff_data_to_ettercap);     // HOOK POINT: PCK_DECODED
+#endif
+                     if (!dontforward && Illithid_ToBeSniffed(&sniff_data_to_ettercap))
+                        Buffer_Put(pipe_with_illithid_data, &sniff_data_to_ettercap, sizeof(SNIFFED_DATA));
                   }
                }
             }
@@ -482,7 +580,7 @@ void Illithid_GetConnections(short mode, char *iface, char *IP1p, char *IP2p, ch
                   session.elem1.IPID = ntohs(ip->ident);
                   session.elem1.flags |= tcp->flags;
 
-                  if (mode==ARPBASED)
+                  if (((illithid_param *)param)->mode==ARPBASED)
                   {
                      tcp->seq = htonl(session.elem1.Seq+session.elem1.ByteSent);
                      tcp->ack_seq = htonl(session.elem1.Ack-session.elem2.ByteSent);
@@ -520,33 +618,32 @@ void Illithid_GetConnections(short mode, char *iface, char *IP1p, char *IP2p, ch
                         tcp->checksum = Inet_Forge_Checksum((unsigned short *)tcp, IPPROTO_TCP, ntohs(ip->t_len)-ip->h_len*4, ip->source_ip, ip->dest_ip);
                      }
                   } // endif ARPBASED
-                  Illithid_Set_Session(saved_session, &session, mode);
-               } //endif TCP
+
+                  if (((illithid_param *)param)->mode <= PUBLICARP && PckDir && !dontforward)
+                  {
+#ifdef PERMIT_PLUGINS
+                     Plugin_HookPoint(PCK_PRE_FORWARD, buffer);      // HOOK POINT: PCK_PRE_FORWARD
+#endif
+                     session.elem1.PacketSent += Inet_SendLargeTCPPacket(sock, buffer, len, MTU);
+                  }
+                  Illithid_Set_Session(saved_session, &session, ((illithid_param *)param)->mode);
+                  goto nosend;
+               } //endif TCP filtering
             } //endif !dontforward
 
-send:       if (mode <= PUBLICARP && PckDir && !dontforward) Inet_SendRawPacket(sock, buffer, len);
-
+send:       if (((illithid_param *)param)->mode <= PUBLICARP && PckDir && !dontforward)
+            {
+#ifdef PERMIT_PLUGINS
+               Plugin_HookPoint(PCK_PRE_FORWARD, buffer);      // HOOK POINT: PCK_PRE_FORWARD
+#endif
+               Inet_SendRawPacket(sock, buffer, len);
+            }
+nosend:     while(0);      // dirty trick to avoid warning....
          } // endif ETH_P_IP
       }  // endif len > 0
       else
          usleep(1000);
 
-// ==========================================================================
-      nchars = read(pipe_filter[0], &filter_status, 1);
-      if (nchars == 1)
-      {
-         if (filter_status == 'S')
-            filter_on_source = (filter_on_source) ? 0 : 1;
-         if (filter_status == 'D')
-            filter_on_dest = (filter_on_dest) ? 0 : 1;
-
-         #ifdef DEBUG
-            Debug_msg("Illithid_GetConnections -- FILTERS source %d dest %d", filter_on_source, filter_on_dest);
-         #endif
-
-         if (filter_on_source || filter_on_dest)
-            Parser_LoadFilters("");
-      }
 // ==========================================================================
       {
          KILL_DATA to_kill;
@@ -601,7 +698,7 @@ send:       if (mode <= PUBLICARP && PckDir && !dontforward) Inet_SendRawPacket(
 
 // ==========================================================================
 
-      if (mode==ARPBASED)
+      if (((illithid_param *)param)->mode==ARPBASED)
       {
          INJECTED_DATA inj;
 
@@ -657,58 +754,63 @@ send:       if (mode <= PUBLICARP && PckDir && !dontforward) Inet_SendRawPacket(
             session.elem1.ByteSent += inj.datalen;
             session.elem1.PacketSent++;
 
-            Illithid_Set_Session(saved_session, &session, mode);
+            Illithid_Set_Session(saved_session, &session, ((illithid_param *)param)->mode);
          }
       }
 //==================================================================
 
    } // end loop
+
+   exit_func_end();
+
 }
 
 
-int Illithid_ToBeSniffed(char *source, int psource, char *dest, int pdest, SNIFFED_DATA *data)
+int Illithid_ToBeSniffed(SNIFFED_DATA *data)
 {
    char s=0, ps=0, d=0, pd=0;
 
-   if (psource == 0) ps = 1;
-   if (pdest == 0) pd = 1;
-   if (!strcmp(source, "") || !strcmp(source, "0.0.0.0") ) s = 1;
-   if (!strcmp(dest, "") || !strcmp(dest, "0.0.0.0")) d = 1;
+   if (current_illithid_data.proto != data->proto) return 0;
 
-   if (s || !strcmp(source, data->source_ip))
-      if (ps || psource == data->source_port)
+   if (current_illithid_data.source_port == 0) ps = 1;
+   if (current_illithid_data.dest_port == 0) pd = 1;
+   if (current_illithid_data.source_ip == 0) s = 1;
+   if (current_illithid_data.dest_ip == 0) d = 1;
+
+   if (s || current_illithid_data.source_ip == data->fast_source_ip)
+      if (ps || current_illithid_data.source_port == data->source_port)
       {  s = 1;   ps = 1;  }
 
-   if (s || !strcmp(source, data->dest_ip))
-      if (ps || psource == data->dest_port)
+   if (s || current_illithid_data.source_ip == data->fast_dest_ip)
+      if (ps || current_illithid_data.source_port == data->dest_port)
       {  s = 1;   ps = 1;  }
 
-   if (d || !strcmp(dest, data->source_ip))
-      if (pd || pdest == data->source_port)
+   if (d || current_illithid_data.dest_ip == data->fast_source_ip)
+      if (pd || current_illithid_data.dest_port == data->source_port)
       {  d = 1;   pd = 1;  }
 
-   if (d || !strcmp(dest, data->dest_ip))
-      if (pd || pdest == data->dest_port)
+   if (d || current_illithid_data.dest_ip == data->fast_dest_ip)
+      if (pd || current_illithid_data.dest_port == data->dest_port)
       {  d = 1;   pd = 1;  }
 
-   return ( s && ps && d && pd );
+   return ( Options.reverse ^ (s && ps && d && pd) );
 }
 
 
-int Illithid_ToBeSniffed_ip(char *source, char *dest, CONNECTION *data)
+int Illithid_ToBeSniffed_ip(u_long source, u_long dest, CONNECTION *data)
 {
    char s=0, d=0;
 
-   if (!strcmp(source, "")) s = 1;
-   if (!strcmp(dest, "")) d = 1;
+   if (source == 0) s = 1;
+   if (dest == 0) d = 1;
 
    if (s && d) return(PCK_ANY);
 
-   if ( (s || !strcmp(source, data->source_ip)) &&
-        (d || !strcmp(dest, data->dest_ip)) ) return (PCK_STOD);
+   if ( (s || Options.reverse ^(source == data->fast_source_ip) ) &&
+        (d || Options.reverse ^(dest == data->fast_dest_ip) ) ) return (PCK_STOD);
 
-   if ( (s || !strcmp(source, data->dest_ip)) &&
-        (d || !strcmp(dest, data->source_ip)) ) return (PCK_DTOS);
+   if ( (s || Options.reverse ^(source == data->fast_dest_ip) ) &&
+        (d || Options.reverse ^(dest == data->fast_source_ip) ) ) return (PCK_DTOS);
 
    return (0);
 
@@ -789,11 +891,14 @@ Ielem **Illithid_Find_Session(unsigned int IP1, unsigned int IP2, unsigned short
       outs->elem2.IP   = IP2;
       outs->elem2.Port = Port2;
       outs->elem2.IPID = outs->elem1.IPID;
-//FIXME
-      if (!Options.normal) // 'Cause not injecting is stored on changes
-      {                    // by Illithid_Set_Session
+
+      if (!Options.normal)
+      {
          Ielem *new_elem;
          new_elem = malloc(sizeof(Ielem));
+
+         if (!new_elem) return (to_find);
+
          new_elem->proto = outs->proto;
          memcpy(&(new_elem->elem1), &(outs->elem1), sizeof(side_elem));
          memcpy(&(new_elem->elem2), &(outs->elem2), sizeof(side_elem));
@@ -852,8 +957,11 @@ void Illithid_Set_Session(Ielem **session, Ielem *outs, short mode)
       Ielem *new_elem;
       unsigned char hash;
 
-      hash = (outs->elem1.IP+outs->elem2.IP+outs->elem1.Port+outs->elem2.Port)%HASHES; // Very simple hash
+      hash = (htonl(outs->elem1.IP)+htonl(outs->elem2.IP)+htons(outs->elem1.Port)+htons(outs->elem2.Port))%HASHES; // Very simple hash
       new_elem = malloc(sizeof(Ielem));
+
+      if (!new_elem) return;
+
       new_elem->proto = outs->proto;
       outs->elem1.flags &= TH_FIN;
       outs->elem2.flags &= TH_FIN;
@@ -865,7 +973,7 @@ void Illithid_Set_Session(Ielem **session, Ielem *outs, short mode)
 }
 
 
-void Illithid_Reset_Conn(void)
+void Illithid_Reset_Conn(void *dummy)
 {
    int i,j;
    Ielem *session, *next;
@@ -923,8 +1031,13 @@ void Illithid_Reset_Conn(void)
          free(session);
          session = next;
       }
+      hash_table[j] = 0;
    }
 
+   ECThread_destroy(decoder_pid);
+
+   Buffer_Flush(illithid_buffer);
+
 #ifdef DEBUG
    Debug_msg("Illithid -- ShutDowned gracefully...");
 #endif
