@@ -17,14 +17,11 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_inet.c,v 1.6 2001/09/10 16:41:33 alor Exp $
+    $Id: ec_inet.c,v 1.15 2001/12/09 20:24:51 alor Exp $
 */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
-#include <strings.h>
+#include "include/ec_main.h"
+
 #include <sys/time.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
@@ -35,15 +32,12 @@
    #define WAIT_ANY       (-1)    /* Any process.  */
 #endif
 
-#include "include/ec_main.h"
 #include "include/ec_error.h"
 #include "include/ec_inet_structures.h"
 #include "include/ec_inet_forge.h"
 #include "include/ec_buffer.h"
-
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
+#include "include/ec_thread.h"
+#include "include/ec_parser.h"
 
 static char ETH_BROADCAST[6] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
 static char ARP_BROADCAST[6] = {0x0,0x0,0x0,0x0,0x0,0x0};
@@ -56,7 +50,7 @@ typedef struct {
 
 struct ifreq old_ifr;         // old iface flags
 #ifdef LINUX
-   char IpForward_status[2];  // old ipforward status
+   char IpForward_status;     // old ipforward status
 #else
    int IpForward_status;      // old ipforward status
 #endif
@@ -79,11 +73,14 @@ int Inet_Fake_Host(void);
 int Inet_CheckSwitch(void);
 int Inet_OpenSocket(char *host, short port);
 int Inet_Http_Send(int s, char *payload);
-int Inet_Http_Receive(int s, char *payload);
+int Inet_Http_Receive(int s, char *payload, size_t size);
+int Inet_SendLargeTCPPacket(int sock, char *buffer, int len, int MTU);
+int Inet_Load_Host_List(void);
+char * Inet_Save_Host_List(void);
 
 // Following are architecture dependent !! implementations are in ./src/`uname`/ec_inet_`uname`.c
 int Inet_FindIFace(char *iface);
-char Inet_CorrectIface(char *iface);
+int Inet_CorrectIface(char *iface);
 int Inet_GetIfaceInfo(char *iface, int *MTU, char *MyMAC, u_long *IP, u_long *NetMask);
 int Inet_SetPromisc(char *iface);
 int Inet_OpenRawSock(char *iface);
@@ -105,7 +102,7 @@ char *Inet_MacFromIP(unsigned long ip);
 // Architecture dependent implemetation of Inet functions...
 
 #ifdef LINUX
-   #include "OS/linux/ec_inet_linux.c"
+	#include "OS/linux/ec_inet_linux.c"
 #endif
 #if defined (FREEBSD) || defined (OPENBSD) || defined (NETBSD)
    #include "OS/BSD/ec_inet_BSD.c"
@@ -148,9 +145,10 @@ char * Inet_NameToIp(char *name) // returns ip from hostname
    static char ip[16];
 
    host = gethostbyname(name);
+	memset(ip, 0, sizeof(ip));
 
    if (host != NULL)
-      strncpy(ip, inet_ntoa( *(struct in_addr *) *host->h_addr_list), 16);
+      strlcpy(ip, inet_ntoa( *(struct in_addr *) *host->h_addr_list), 16);
    else
       Error_msg("Invalid host address %s !!", name);
 
@@ -171,7 +169,7 @@ char * Inet_GetMyInfo(char tipo)
    switch (tipo)
    {
          case 0:  // requesting IP
-                  sprintf(IP, "%s", int_ntoa(IP_Add));
+                  snprintf(IP, sizeof(IP), "%s", int_ntoa(IP_Add));
                   #ifdef DEBUG
                      Debug_msg("Inet_GetMyInfo IP -- [%s]", IP);
                   #endif
@@ -187,7 +185,7 @@ char * Inet_GetMyInfo(char tipo)
                   break;
 
          case 2:  // requesting netmask
-                  sprintf(MySubnet, "%s", int_ntoa(NetMask));
+                  snprintf(MySubnet, sizeof(MySubnet), "%s", int_ntoa(NetMask));
                   #ifdef DEBUG
                      Debug_msg("Inet_GetMyInfo NetMask -- [%s]", MySubnet);
                   #endif
@@ -227,6 +225,9 @@ int Inet_HostInLAN()
    host_list *current;
    int host_alive = 0, k = 0;
 
+   if (Options.hostsfromfile)
+       return Inet_Load_Host_List();
+
 #ifdef DEBUG
    Debug_msg("Inet_HostInLAN\tIface: %s", Options.netiface);
 #endif
@@ -249,7 +250,7 @@ int Inet_HostInLAN()
    current = list;
    for( ; current; current = (host_list *)current->next)
    {
-      sprintf(Host_In_LAN[k].ip,"%s",int_ntoa(current->IP_Add));
+      snprintf(Host_In_LAN[k].ip, sizeof(Host_In_LAN[k].ip), "%s", int_ntoa(current->IP_Add));
       snprintf(Host_In_LAN[k].name, 128, "%s", Inet_HostName(Host_In_LAN[k].ip));
       Inet_PutMACinString(Host_In_LAN[k].mac, current->MAC_Add);
       k++;
@@ -262,6 +263,94 @@ int Inet_HostInLAN()
 }
 
 
+char * Inet_Save_Host_List(void)
+{
+   FILE *hfile;
+   u_long N_IP, N_MASK, index;
+   static char hfile_name[50];
+   char temp[50];
+   struct in_addr ip_addr;
+   struct in_addr mask_addr;
+
+   Inet_GetIfaceInfo(Options.netiface, NULL, NULL, &N_IP, &N_MASK);
+
+   N_IP = N_IP & N_MASK;
+   ip_addr.s_addr = N_IP;
+   mask_addr.s_addr = N_MASK;
+   snprintf(temp, 49, "%s", inet_ntoa(mask_addr));
+   snprintf(hfile_name, 49, "%s_%s.ehl", inet_ntoa(ip_addr), temp);
+
+#ifdef DEBUG
+   Debug_msg("Inet_Save_Host_List saving file: %s", hfile_name);
+#endif
+
+   hfile = fopen(hfile_name, "w");
+   if (hfile == NULL)
+      ERROR_MSG("fopen()");
+
+   for (index = 1; index<number_of_hosts_in_lan; index++)
+      fprintf(hfile,"%s %s %s\n",Host_In_LAN[index].ip, Host_In_LAN[index].mac, Parser_StrSpacetoUnder(Host_In_LAN[index].name));
+
+   fclose(hfile);
+
+   return hfile_name;
+}
+
+
+
+int Inet_Load_Host_List(void)
+{
+   FILE *hfile;
+   char C_IP[50], C_MAC[50], C_NAME[128];
+   char N_MAC[6];
+   u_long N_IP;
+   int  host_alive,index;
+
+#ifdef DEBUG
+   Debug_msg("Inet_Load_Host_List from file: %s", Options.hostfile);
+#endif
+
+   hfile = fopen(Options.hostfile,"r");
+   if (hfile == NULL)
+      ERROR_MSG("fopen()");
+
+   fprintf(stdout, "Loading Host List from file %s\n", Options.hostfile);
+
+   for (host_alive = 1; !feof(hfile); host_alive++)
+      fscanf(hfile,"%49s %49s %127s\n", C_IP, C_MAC, C_NAME);
+   fseek(hfile, 0, SEEK_SET);
+
+#ifdef DEBUG
+   Debug_msg("Inet_Load_Host_List -- %d hosts in the lan", host_alive);
+#endif
+
+   if (Host_In_LAN) free(Host_In_LAN);
+
+   Host_In_LAN = (HOST *)calloc(host_alive,sizeof(HOST));
+   if (Host_In_LAN == NULL)
+      ERROR_MSG("calloc()");
+
+   Inet_GetIfaceInfo(Options.netiface, NULL, N_MAC, &N_IP, NULL);
+
+   snprintf(Host_In_LAN[0].ip, sizeof(Host_In_LAN[0].ip), "%s", int_ntoa(N_IP));
+   Inet_PutMACinString(Host_In_LAN[0].mac, N_MAC);
+   snprintf(Host_In_LAN[0].name, 128, "%s", Inet_HostName(Host_In_LAN[0].ip));
+
+   for (index=1; index<host_alive; index++)
+   {
+      fscanf(hfile,"%49s %49s %127s\n", C_IP, C_MAC, C_NAME);
+      snprintf(Host_In_LAN[index].ip, sizeof(Host_In_LAN[index].ip), "%s", C_IP);
+      snprintf(Host_In_LAN[index].mac, sizeof(Host_In_LAN[index].mac), "%s", C_MAC);
+      snprintf(Host_In_LAN[index].name, 128, "%s", C_NAME);
+   }
+
+   fclose(hfile);
+
+   Options.hostsfromfile = 0;
+
+   return host_alive;
+}
+
 
 
 void Inet_Free_list(host_list *head)
@@ -397,7 +486,7 @@ host_list *Inet_Host_in_LAN_list(char *interface)
          }
 
          Inet_Forge_packet_destroy( buf );
-         usleep(500000);   // 0.5 second to wait for slower replies
+         usleep(500000);   // 0.5 second to wait for slower replies (ettercap is syncronized on this process)
          exit(0);          // the forked process must exit after arp storm
       }
    }
@@ -680,7 +769,10 @@ int Inet_CheckSwitch()
    Debug_msg("Inet_CheckSwitch" );
 #endif
 
-   if (number_of_hosts_in_lan>1)
+
+   if (Options.link) return 0;
+
+   if (number_of_hosts_in_lan==2)
    {
        int fakeip,destip,sock,MTU,i;
        char MyMAC[6],DestMAC[6];
@@ -737,9 +829,61 @@ int Inet_CheckSwitch()
 
        free(buf);
        Inet_CloseRawSock(sock);
+       Inet_Restore_ifr();
+   }
+   else
+   if (number_of_hosts_in_lan>2)
+   {
+       int  sourceip,destip,sock,MTU;
+       char MyMAC[6],DestMAC[6],SourceMAC[6];
+       char *buf;
+       TIME_DECLARE;
+
+       sourceip=inet_addr(Host_In_LAN[2].ip);
+       destip=inet_addr(Host_In_LAN[1].ip);
+       Inet_GetMACfromString(Host_In_LAN[2].mac, SourceMAC);
+       Inet_GetMACfromString(Host_In_LAN[1].mac, DestMAC);
+       sock = Inet_OpenRawSock(Options.netiface);
+       fcntl(sock, F_SETFL, O_NONBLOCK);
+       Inet_GetIfaceInfo(Options.netiface, &MTU, MyMAC, NULL, NULL);
+       Inet_SetPromisc(Options.netiface);
+
+       buf = Inet_Forge_packet(MTU);
+       Inet_Forge_ethernet( buf, MyMAC, DestMAC, ETH_P_IP );
+       Inet_Forge_ip( buf + ETH_HEADER,
+                      sourceip, destip,
+                      sizeof(ICMP_header),
+                      0xe77e, 0,
+                      IPPROTO_ICMP );
+       Inet_Forge_icmp( buf + ETH_HEADER + IP_HEADER, ICMP_ECHO, 0, NULL, 0 );
+       Inet_SendRawPacket(sock, buf, ETH_HEADER + IP_HEADER + ICMP_HEADER);
+
+       TIME_START;
+       do
+       {
+           short type; int len;
+
+           len=Inet_GetRawPacket(sock,buf,MTU,&type);
+           TIME_FINISH;
+
+            if (len>0)
+            {
+               ETH_header *eth;
+               eth=(ETH_header *)buf;
+               if (!memcmp(SourceMAC,eth->dest_mac,6))
+               {
+                  link_type=1;
+                  break;
+               }
+            }
+         }while(TIME_ELAPSED<1);
+
+       free(buf);
+       Inet_CloseRawSock(sock);
+       Inet_Restore_ifr();
    }
    else return 0;
-   Inet_Restore_ifr();
+
 
 #ifdef DEBUG
    Debug_msg("Inet_CheckSwitch -- type %d", link_type );
@@ -782,7 +926,7 @@ void Inet_PutMACinString(char *mac_string, unsigned char *MAC)
    for (i=0; i<6; i++)
       MAC_Add[i]=(unsigned int)MAC[i];
 
-   sprintf(mac_string,"%02X:%02X:%02X:%02X:%02X:%02X",
+   sprintf(mac_string, "%02X:%02X:%02X:%02X:%02X:%02X",
       (unsigned int)MAC_Add[0],(unsigned int)MAC_Add[1],(unsigned int)MAC_Add[2],
       (unsigned int)MAC_Add[3],(unsigned int)MAC_Add[4],(unsigned int)MAC_Add[5]);
 
@@ -830,13 +974,54 @@ int Inet_Http_Send(int s, char *payload)
 
 
 
-int Inet_Http_Receive(int s, char *payload)
+int Inet_Http_Receive(int s, char *payload, size_t size)
 {
    int i=0;
 
-   while( read(s, payload + i++, 1) );
+   while( i < size && read(s, payload + i++, 1) );
 
    return i;
 }
 
+
+int Inet_SendLargeTCPPacket(int sock, char *buffer, int len, int MTU)
+{
+   IP_header *ip;
+   TCP_header *tcp;
+   char *data;
+   int datalen, totdatalen;
+
+   if (len <= MTU+ETH_HEADER)
+   {
+      Inet_SendRawPacket(sock, buffer, len);
+      return(0);
+   }
+
+   //Only for splitted TCP packets
+   ip = (IP_header *) (buffer + ETH_HEADER);
+   tcp = (TCP_header *) ((int)ip + ip->h_len * 4);
+   data = (char *)((int)tcp + tcp->doff * 4);
+   datalen = (int)ip + MTU - (int)data;
+   totdatalen = (int)ip + ntohs(ip->t_len) - (int)data;
+   ip->t_len = htons(MTU);
+   ip->checksum = 0;
+   ip->checksum = Inet_Forge_ChecksumIP( (unsigned short *) ip, sizeof(IP_header));
+   tcp->checksum = 0;
+   tcp->checksum = Inet_Forge_Checksum((unsigned short *)tcp, IPPROTO_TCP, ntohs(ip->t_len)-ip->h_len*4, ip->source_ip, ip->dest_ip);
+   Inet_SendRawPacket(sock, buffer, MTU+ETH_HEADER);
+
+   ip->t_len = htons(len - datalen - ETH_HEADER);
+
+   ip->ident = htons ( ntohs(ip->ident) + 1 );
+   ip->checksum = 0;
+   ip->checksum = Inet_Forge_ChecksumIP( (unsigned short *) ip, sizeof(IP_header));
+   memcpy(data , data+datalen, totdatalen-datalen);
+   tcp->seq = htonl ( ntohl(tcp->seq) + datalen );
+   tcp->checksum = 0;
+   tcp->checksum = Inet_Forge_Checksum((unsigned short *)tcp, IPPROTO_TCP, ntohs(ip->t_len)-ip->h_len*4, ip->source_ip, ip->dest_ip);
+   Inet_SendRawPacket(sock, buffer, ntohs(ip->t_len)+ETH_HEADER);
+
+   return(1);
+}
+
 /* EOF */
