@@ -17,24 +17,20 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_interface.c,v 1.4 2001/09/06 20:07:42 alor Exp $
+    $Id: ec_interface.c,v 1.10 2001/12/09 20:24:51 alor Exp $
 */
 
 #include "include/ec_main.h"
 
 #ifdef HAVE_NCURSES  // don't compile if ncurses interface is not supported
 
-#include <string.h>
 #ifdef HAVE_NCURSES_H
    #include <ncurses.h>
 #else
    #include <curses.h>
 #endif
-#include <unistd.h>
-#include <stdio.h>
-#include <stdlib.h>
+
 #include <stdarg.h>
-#include <signal.h>
 
 #ifdef HAVE_TERMIOS_H
    #include <termios.h>
@@ -55,11 +51,6 @@
    #include "include/ec_plugins.h"
 #endif
 
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
-
-
 #define DOWN   +1             // movements for the pointer
 #define UP     -1
 #define RIGHT  +1
@@ -217,9 +208,9 @@ void Interface_Redraw(void)
    wbkgdset(b_top_window, COLOR_PAIR(TITLE_COLOR)); wattron(b_top_window, A_BOLD);
    wbkgdset(b_bottom_window, COLOR_PAIR(TITLE_COLOR)); wattron(b_bottom_window, A_BOLD);
 
-   mvwprintw(b_main_window, 0, CENTER(W_MAINX1, W_MAINX2, strlen(title_main_window)), title_main_window);
-   mvwprintw(b_bottom_window, 0, CENTER(W_BOTTOMX1, W_BOTTOMX2, strlen(title_bottom_window)), title_bottom_window);
-   mvwprintw(b_top_window, 0, CENTER(W_TOPX1, W_TOPX2, strlen(title_top_window)), title_top_window);
+   mvwprintw(b_main_window, 0, CENTER(W_MAINX1, W_MAINX2, strlen(title_main_window)), "%s", title_main_window);
+   mvwprintw(b_bottom_window, 0, CENTER(W_BOTTOMX1, W_BOTTOMX2, strlen(title_bottom_window)), "%s", title_bottom_window);
+   mvwprintw(b_top_window, 0, CENTER(W_TOPX1, W_TOPX2, strlen(title_top_window)), "%s", title_top_window);
 
    wbkgdset(b_main_window, COLOR_PAIR(BOTTOM_COLOR)); wattroff(b_main_window, A_BOLD);
    wbkgdset(b_top_window, COLOR_PAIR(BOTTOM_COLOR)); wattroff(b_top_window, A_BOLD);
@@ -581,7 +572,7 @@ char Interface_PopUp(char *question, ...)
    wbkgdset(question_window, COLOR_PAIR(TITLE_COLOR));
    wattron(question_window, A_BOLD);
    box(question_window,ACS_VLINE,ACS_HLINE);
-   mvwprintw(question_window,  2, 2, message);
+   mvwprintw(question_window,  2, 2, "%s", message);
    wnoutrefresh(question_window);
    doupdate();
    answer = wgetch(question_window);
@@ -614,7 +605,7 @@ void Interface_RefreshList(void)
    wbkgdset(message_window, COLOR_PAIR(TITLE_COLOR));
    wattron(message_window, A_BOLD);
    box(message_window,ACS_VLINE,ACS_HLINE);
-   mvwprintw(message_window,  2, 2, mess);
+   mvwprintw(message_window,  2, 2, "%s", mess);
    wnoutrefresh(message_window);
    doupdate();
 
@@ -659,7 +650,7 @@ void Interface_HelpWindow(char *help[])
 
    for (counter = help; *counter; counter++)
    {
-      mvwprintw(help_window, i++, 2, *counter);
+      mvwprintw(help_window, i++, 2, "%s", *counter);
    }
 
    wnoutrefresh(help_window);
@@ -863,7 +854,7 @@ void Interface_CheckForPoisoner(void)
    wbkgdset(message_window, COLOR_PAIR(TITLE_COLOR));
    wattron(message_window, A_BOLD);
    box(message_window,ACS_VLINE,ACS_HLINE);
-   mvwprintw(message_window,  2, 2, mess);
+   mvwprintw(message_window,  2, 2, "%s", mess);
    wnoutrefresh(message_window);
    doupdate();
 
@@ -919,7 +910,7 @@ void Interface_FingerPrint(void)
    wbkgdset(message_window, COLOR_PAIR(TITLE_COLOR));
    wattron(message_window, A_BOLD);
    box(message_window,ACS_VLINE,ACS_HLINE);
-   mvwprintw(message_window,  2, 2, mess);
+   mvwprintw(message_window,  2, 2, "%s", mess);
    wnoutrefresh(message_window);
    doupdate();
 
@@ -961,13 +952,13 @@ void Interface_FingerPrint(void)
    wbkgdset(finger_win, COLOR_PAIR(BOTTOM_COLOR));
 
    if (strcmp(long_os_fingers, ""))
-      strncpy(temp, strtok(long_os_fingers, "\n"), W_MAINX2-4-18);
+      strlcpy(temp, strtok(long_os_fingers, "\n"), W_MAINX2-4-18);
 
    mvwprintw(finger_win, 2, 20, "%s", temp);
    for(i=0; i<j-1; i++)
    {
       memset(temp, 0, W_MAINX2);
-      strncpy(temp, strtok(NULL, "\n"), W_MAINX2-4-18);
+      strlcpy(temp, strtok(NULL, "\n"), W_MAINX2-4-18);
       mvwprintw(finger_win, 3+i, 20, "%s", temp);
       #ifdef DEBUG
          Debug_msg("Interface_FingerPrint -- [%s]", temp );
@@ -1047,6 +1038,9 @@ void Interface_Run(void)
       Interface_InitList();
    }
 
+   if (Options.hoststofile)
+      Interface_PopUp("Host list dumped into file: %s", Inet_Save_Host_List());
+
    loop
    {
       KeyPress = wgetch(main_window);
@@ -1153,7 +1147,7 @@ void Interface_Run(void)
                         "[return]  - select the IP",
                         "[space]   - deselect the IPs",
                         "[tab]     - switch between source and dest",
-                        "[aA]      - ARP posoning based sniffing ",
+                        "[aA]      - ARP poisoning based sniffing ",
                         "             . for sniffing on switched LAN",
                         "             . for man-in-the-middle technique",
                         "[sS]      - IP based sniffing",
@@ -1167,6 +1161,7 @@ void Interface_Run(void)
                         "[oO]      - passive host identification",
                         "[cC]      - check for other poisoner...",
                         "[rR]      - refresh the list",
+                        "[kK]      - save host list to a file",
                         "[hH]      - this help screen",
                         NULL};
                      Interface_HelpWindow(help);
@@ -1181,6 +1176,11 @@ void Interface_Run(void)
                   Interface_InitList();
                   break;
 
+         case 'K':
+         case 'k':
+                  Interface_PopUp("Host list dumped to file: %s", Inet_Save_Host_List());
+                  break;
+
          case KEY_CTRL_L:  // CTRL+L refresh the screen
                   Interface_Redraw();
                   break;
@@ -1241,16 +1241,14 @@ void Interface_WExit(char *buffer)
 {
 
 #ifdef DEBUG
-   Debug_msg("Interface_WExit");
+   Debug_msg("Interface_WExit -- [%s]", buffer);
 #endif
 
    if (ScreenMode) Interface_CloseScreen();
 
-   printf("\033[01m\033[1m%s %s brought from the dark side of the net by ALoR and NaGA...\033[0m\n\n", PROGRAM, VERSION);
-   printf("\033[01m\033[1mmay the packets be with you...\033[0m\n\n");
-   printf("\n");
-   printf(buffer);
-   printf("\n\n");
+   fprintf(stdout, "\033[01m\033[1m%s %s brought from the dark side of the net by ALoR and NaGA...\033[0m\n\n", PROGRAM, VERSION);
+   fprintf(stdout, "\033[01m\033[1mmay the packets be with you...\033[0m\n\n");
+   fprintf(stdout, "\n%s\n\n", buffer);
 
    exit(0);
 }
