@@ -17,23 +17,22 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_interface_passive.c,v 1.6 2001/09/13 20:15:48 alor Exp $
+    $Id: ec_interface_passive.c,v 1.11 2001/12/08 11:22:20 alor Exp $
 */
 
 #include "include/ec_main.h"
 
 #ifdef HAVE_NCURSES  // don't compile if ncurses interface is not supported
 
-#include <string.h>
 #ifdef HAVE_NCURSES_H
    #include <ncurses.h>
 #else
    #include <curses.h>
 #endif
-#include <stdlib.h>
-#include <unistd.h>
-#include <signal.h>
+
 #include <sys/time.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 #include "include/ec_interface.h"
 #include "include/ec_decodedata.h"
@@ -45,10 +44,8 @@
 #include "include/ec_error.h"
 #include "include/ec_logtofile.h"
 #include "include/ec_fingerprint.h"
+#include "include/ec_thread.h"
 
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
 
 #define BOTTOM_COLOR  1        // color schemes
 #define TITLE_COLOR   2
@@ -65,6 +62,14 @@
 #define KEY_RETURN   10       // they aren't defined in ncurses.h :(
 #define KEY_CTRL_L   12
 
+#ifndef CTRL
+#define CTRL(x)         ((x) & 0x1f)
+#endif
+
+#define KEY_QUIT            CTRL('q')
+#define KEY_ESCAPE          CTRL('[')
+
+
 // protos...
 void Interface_Passive_Run(void);
 void Interface_Passive_PointItem(char direction);
@@ -285,7 +290,7 @@ void Interface_Passive_MakeReport(char mode)
    wbkgdset(message_window, COLOR_PAIR(TITLE_COLOR));
    wattron(message_window, A_BOLD);
    box(message_window,ACS_VLINE,ACS_HLINE);
-   mvwprintw(message_window,  2, 2, mess);
+   mvwprintw(message_window,  2, 2, "%s", mess);
    wnoutrefresh(message_window);
    doupdate();
 
@@ -329,8 +334,8 @@ void Interface_Passive_ShowDetails(int i)
 
    dimY = (dimY > W_BOTTOMY2-2) ? W_BOTTOMY2-2 : dimY;   // limit the window height
 
-   d_win = newwin(dimY+2, dimX+2, W_BOTTOMY2/2 - dimY/2, W_MAINX2/2 - dimX/2);
-   detail_window = newwin(dimY, dimX, W_BOTTOMY2/2 - dimY/2 +1, W_MAINX2/2 - dimX/2 +1);
+   d_win = newwin(dimY+2, dimX+2, W_BOTTOMY2/2 - dimY/2 - 1, W_MAINX2/2 - dimX/2);
+   detail_window = newwin(dimY, dimX, W_BOTTOMY2/2 - dimY/2, W_MAINX2/2 - dimX/2 +1);
    wbkgdset(d_win, COLOR_PAIR(HELP_COLOR));
    wattron(d_win, A_BOLD);
    box(d_win,ACS_VLINE,ACS_HLINE);
@@ -451,7 +456,7 @@ void Interface_Passive_Showname(void)
       wbkgdset(message_window, COLOR_PAIR(TITLE_COLOR));
       wattron(message_window, A_BOLD);
       box(message_window,ACS_VLINE,ACS_HLINE);
-      mvwprintw(message_window,  2, 2, mess);
+      mvwprintw(message_window,  2, 2, "%s", mess);
       wnoutrefresh(message_window);
       doupdate();
 
@@ -459,7 +464,7 @@ void Interface_Passive_Showname(void)
       for(i=0; i<number_of_passive_hosts; i++)
       {
          if ( !strcmp(Passive_Host[i].name, "") )
-            strncpy(Passive_Host[i].name, Inet_HostName(Passive_Host[i].ip), 17 );
+            strlcpy(Passive_Host[i].name, Inet_HostName(Passive_Host[i].ip), sizeof(Passive_Host[i].name));
       }
 
       delwin(message_window);
@@ -492,7 +497,7 @@ void Interface_Passive_Submit(char *ip, char *fingerprint)
    wbkgdset(f_win, COLOR_PAIR(HELP_COLOR));
    wattron(f_win, A_BOLD);
    box(f_win,ACS_VLINE,ACS_HLINE);
-   mvwprintw(f_win,  0, 2, "Fingerprint submitter:", MAX_INJECT);
+   mvwprintw(f_win,  0, 2, "Fingerprint submitter:    (press F10 to exit) ", MAX_INJECT);
    wbkgdset(w_submit, COLOR_PAIR(BOTTOM_COLOR));
    wmove(w_submit, 0, 0);
    scrollok(w_submit, TRUE);
@@ -505,7 +510,7 @@ void Interface_Passive_Submit(char *ip, char *fingerprint)
       wnoutrefresh(w_submit);
       doupdate();
 
-   memset(OS, 0, 60);
+   memset(OS, 0, sizeof(OS));
 
    loop
    {
@@ -523,6 +528,14 @@ void Interface_Passive_Submit(char *ip, char *fingerprint)
          OS[p_text] = 0;
          if ( p_text > 0 ) p_text--;
       }
+      else if ( c == KEY_F(10) || c == KEY_ESCAPE || c == KEY_QUIT )  // exit
+      {
+         curs_set(FALSE);
+         delwin(f_win);
+         delwin(w_submit);
+         doupdate();
+         return;
+      }
       else
       {
          pechochar(w_submit, c);
@@ -549,7 +562,7 @@ void Interface_Passive_Submit(char *ip, char *fingerprint)
    for (i=0; i<60; i++)
       if (OS[i] == ' ') OS[i] = '+';
 
-   sprintf(data, "&finger=%s&os=%s", fingerprint, OS);
+   snprintf(data, sizeof(data), "&finger=%s&os=%s", fingerprint, OS);
 
    werase(w_submit);
    wprintw(w_submit, "Connecting to http://%s...\n", host);
@@ -565,10 +578,10 @@ void Interface_Passive_Submit(char *ip, char *fingerprint)
       wnoutrefresh(w_submit);
       doupdate();
 
-   sprintf(getmsg, "GET %s%s HTTP/1.0\r\n"
-                   "Host: %s\r\n"
-                   "User-Agent: %s (%s).\r\n"
-                   "\r\n", page, data, host, PROGRAM, VERSION );
+   snprintf(getmsg, sizeof(getmsg), "GET %s%s HTTP/1.0\r\n"
+                                    "Host: %s\r\n"
+                                    "User-Agent: %s (%s).\r\n"
+                                    "\r\n", page, data, host, PROGRAM, VERSION );
 
 #ifdef DEBUG
    Debug_msg("Interface_Passive_Submit -- %s", getmsg);
@@ -588,28 +601,29 @@ void Interface_Passive_Submit(char *ip, char *fingerprint)
    f_os = fopen( "./" OS_FILE, "r");
    if (f_os == NULL)
    {
-      f_os = fopen( INSTALL_PATH OS_FILE, "r");
+      f_os = fopen( DATA_PATH "/" OS_FILE, "r");
       if (f_os == NULL)
-         Error_msg("\nCan't find " OS_FILE " in ./ or " INSTALL_PATH);
+         Error_msg("\nCan't find " OS_FILE " in ./ or " DATA_PATH);
    }
 
    f_temp = tmpfile();
+   fchmod(fileno(f_temp), 0700);
 
    while(fgets (data, 100, f_os))
    {
       if ( strchr(data, '#') || !strlen(data))
       {
-         fprintf(f_temp, data);
+         fprintf(f_temp, "%s", data);
          continue;
       }
 
       if (strcmp(data, fingerprint) < 0)
-         fprintf(f_temp, data);
+         fprintf(f_temp, "%s", data);
       else
       {
-         if (strlen(OS)) fprintf(f_temp, "%s:%s\n", fingerprint, OS);
-         memset(OS, 0, 60);
-         fprintf(f_temp, data);
+         if (strlen(OS)) fprintf(f_temp, "%s:%.60s\n", fingerprint, OS);
+         memset(OS, 0, sizeof(OS));
+         fprintf(f_temp, "%s", data);
       }
    }
    rewind(f_temp);
@@ -617,11 +631,11 @@ void Interface_Passive_Submit(char *ip, char *fingerprint)
 
    f_os = fopen( "./" OS_FILE, "w");
    if (f_os == NULL)
-      f_os = fopen( INSTALL_PATH OS_FILE, "w");
+      f_os = fopen( DATA_PATH "/" OS_FILE, "w");
 
    while(fgets (data, 100, f_temp))
    {
-      fprintf(f_os, data);
+      fprintf(f_os, "%s", data);
    }
 
    fclose(f_temp);
@@ -646,18 +660,14 @@ void Interface_Passive_Submit(char *ip, char *fingerprint)
 void Interface_Passive_Run(void)
 {
    int KeyPress;
-   int Dryad_pid = 0;
+   pthread_t Dryad_pid;
    fd_set msk_fd;
    struct timeval TimeOut;
-   PASSIVE_DATA data_from_dryad;
-   CONNECTION data_from_illithid;
 
 #ifdef DEBUG
    Debug_msg("Interface_Passive_Run");
 #endif
 
-   if (pipe_with_dryad == -1) pipe_with_dryad = Buffer_Create(2.0e5);         // data from passive scanner
-
    init_pair(W_MAIN_COLOR, COLOR_WHITE, COLOR_BLUE);
    init_pair(W_POINT_COLOR, COLOR_WHITE, COLOR_CYAN);
    init_pair(N_MAIN_COLOR, COLOR_BLACK, COLOR_BLUE);
@@ -696,34 +706,18 @@ void Interface_Passive_Run(void)
    number_of_passive_hosts = 0;
    Interface_Passive_InitList();
 
-   Buffer_Flush(pipe_with_dryad);
-
    Dryad_pid = Dryad_Run();
 
-
    memset(&TimeOut, 0, sizeof(TimeOut));  //  timeout = 0
    FD_ZERO(&msk_fd);
 
    loop
    {
-      int datalen, datalen_illithid;
       FD_SET(0, &msk_fd);
 
       select(FOPEN_MAX, &msk_fd, (fd_set *) 0, (fd_set *) 0, &TimeOut);
 
-      datalen_illithid = Buffer_Get(pipe_with_illithid, &data_from_illithid, sizeof(CONNECTION));
-      datalen = Buffer_Get(pipe_with_dryad, &data_from_dryad, sizeof(PASSIVE_DATA));
-
-      if (datalen_illithid > 0)
-         number_of_connections = Decodedata_MakeConnectionList(data_from_illithid);
-
-      if (datalen > 0)
-      {
-         number_of_passive_hosts = Decodedata_MakePassiveList(data_from_dryad);
-         Interface_Passive_InitList();
-      }
-      else  // workaround for the 100% CPU usage...
-         usleep(5);
+      Interface_Passive_InitList();
 
       if (FD_ISSET(0, &msk_fd))
       {
@@ -806,7 +800,7 @@ void Interface_Passive_Run(void)
             case 'Q':
             case 'q':
             case KEY_F(10):
-                     kill(Dryad_pid, SIGTERM);
+                     ECThread_destroy(Dryad_pid);
                      werase(bottom_window);
                      werase(top_window);
                      wnoutrefresh(bottom_window);
@@ -816,6 +810,7 @@ void Interface_Passive_Run(void)
                      number_of_passive_hosts = -1;
                      Host_Pointer = 0;
                      showname = 0;
+                     Options.passive = 0;
                      #ifdef DEBUG
                         Debug_msg("Interface_Passive_END");
                      #endif
@@ -823,7 +818,8 @@ void Interface_Passive_Run(void)
                      break;
          }
       }
-
+      else
+         usleep(1);
    }
 
 }
