@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_interface_plugins.c,v 1.1.1.1 2001/08/27 19:25:06 alor Exp $
+    $Id: ec_interface_plugins.c,v 1.5 2001/12/08 11:22:20 alor Exp $
 */
 
 #include "include/ec_main.h"
@@ -25,25 +25,24 @@
 #ifdef PERMIT_PLUGINS
 #ifdef HAVE_NCURSES  // don't compile if ncurses interface is not supported
 
-#include <string.h>
 #ifdef HAVE_NCURSES_H
    #include <ncurses.h>
 #else
    #include <curses.h>
 #endif
-#include <unistd.h>
-#include <stdlib.h>
+
+
+#include <sys/time.h>
+#include <ctype.h>
 
 #include "include/ec_interface.h"
 #include "include/ec_plugins.h"
+#include "include/ec_buffer.h"
 
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
-
-#define NORM_COLOR   6
+#define BOTTOM_COLOR 1
 #define TITLE_COLOR  2
 #define HELP_COLOR   6
+#define NORM_COLOR   6
 #define REV_COLOR    7
 
 #define KEY_RETURN   10
@@ -62,13 +61,22 @@ void Interface_Plugins_InitList(void);
 void Interface_Plugins_SelectItem(void);
 void Interface_Plugins_Scroll(short direction);
 void Interface_Plugins_DrawScroller(void);
+void Interface_Plugins_Output(void);
+void Interface_Plugins_Output_Scroll(short direction);
+void Interface_Plugins_Output_DrawScroller(void);
+void Interface_Plugins_PluginOutput(void);
+void Interface_Plugins_HidePluginOutput(void);
 
 // global variables
 
 WINDOW *plugin_window, *b_plugin_window;
-extern WINDOW *main_window;
+extern WINDOW *main_window, *top_window, *b_top_window;
+WINDOW *output_window, *o_win;
 
 extern int W_MAINX1, W_MAINY1, W_MAINX2, W_MAINY2, W_SELECTPLUG;
+extern int W_BOTTOMY1, W_BOTTOMY2;
+
+extern struct plug_array *Plugins_Array;  // ec_plugins.c
 
 short number_of_plugins;
 
@@ -79,7 +87,7 @@ short P_Sel_Number;
 
 #define LMARGIN 0
 
-short scroll_yp;
+short scroll_yp, scroll_out_yp;
 
 // -------------------
 
@@ -123,7 +131,7 @@ char Interface_Plugins_PopUp(char *question)
    wbkgdset(question_window, COLOR_PAIR(TITLE_COLOR));
    wattron(question_window, A_BOLD);
    box(question_window,ACS_VLINE,ACS_HLINE);
-   mvwprintw(question_window,  2, 2, question);
+   mvwprintw(question_window,  2, 2, "%s", question);
    wnoutrefresh(question_window);
    doupdate();
    answer = wgetch(question_window);
@@ -158,7 +166,7 @@ void Interface_Plugins_InitList(void)
    if (number_of_plugins == 0)      // no plugin... no action... ;)
    {
       wmove(plugin_window, scroll_yp, LMARGIN );
-      wprintw(plugin_window, "NO plugin available in %s or in ./ !!\n", INSTALL_PATH);
+      wprintw(plugin_window, "NO plugin available in %s or in ./ !!\n", PLUGIN_PATH);
       pnoutrefresh(plugin_window, scroll_yp, 0, W_MAINY1 + 3, 3, W_MAINY2 - 3 , W_MAINX2 - 2);
       doupdate();
       return;
@@ -169,9 +177,15 @@ void Interface_Plugins_InitList(void)
    for(j=Plug_Base_Pointer; j<Plug_Top_Pointer; j++)     // prints connections within the plugin_window height
    {
       wmove(plugin_window, scroll_yp + j - Plug_Base_Pointer, LMARGIN );
-      wprintw(plugin_window, "%3d) %8s   v %.1f -- %s", j+1, Plugin_Getname(Plugins_list[j].name),
-                                                               Plugins_list[j].version,
-                                                               Plugins_list[j].description);
+
+      if (Plugins_Array[j].status == 'A')
+         wbkgdset(plugin_window, COLOR_PAIR(BOTTOM_COLOR));
+
+      wprintw(plugin_window, "%3d) %-11s  %.1f %c -- %s", j+1, Plugins_Array[j].name,
+                                                              Plugins_Array[j].version,
+                                                              Plugins_Array[j].status,
+                                                              Plugins_Array[j].description);
+      wbkgdset(plugin_window, COLOR_PAIR(NORM_COLOR));
    }
 
    if (has_colors())
@@ -181,10 +195,11 @@ void Interface_Plugins_InitList(void)
 
    wmove(plugin_window, scroll_yp + Plug_Pointer - Plug_Base_Pointer, LMARGIN );
    whline(plugin_window, ' ', W_SELECTPLUG);
-   wprintw(plugin_window, "%3d) %8s   v %.1f -- %s", Plug_Pointer+1,
-                                                       Plugin_Getname(Plugins_list[Plug_Pointer].name),
-                                                       Plugins_list[Plug_Pointer].version,
-                                                       Plugins_list[Plug_Pointer].description);
+   wprintw(plugin_window, "%3d) %-11s  %.1f %c -- %s", Plug_Pointer+1,
+                                                      Plugins_Array[Plug_Pointer].name,
+                                                      Plugins_Array[Plug_Pointer].version,
+                                                      Plugins_Array[Plug_Pointer].status,
+                                                      Plugins_Array[Plug_Pointer].description);
 
    pnoutrefresh(plugin_window, scroll_yp, 0, W_MAINY1 + 3, 3, W_MAINY2 - 3 , W_MAINX2 - 2);
    doupdate();
@@ -234,10 +249,16 @@ void Interface_Plugins_PointItem(char direction)
    {
       wmove(plugin_window, scroll_yp + Old_Plug_Pointer - Plug_Base_Pointer, LMARGIN);
       whline(plugin_window,' ', W_SELECTPLUG);                          //deletes the previous position
-      wprintw(plugin_window, "%3d) %8s   v %.1f -- %s", Old_Plug_Pointer+1,
-                                                         Plugin_Getname(Plugins_list[Old_Plug_Pointer].name),
-                                                         Plugins_list[Old_Plug_Pointer].version,
-                                                         Plugins_list[Old_Plug_Pointer].description);
+
+      if (Plugins_Array[Old_Plug_Pointer].status == 'A')
+         wbkgdset(plugin_window, COLOR_PAIR(BOTTOM_COLOR));
+
+      wprintw(plugin_window, "%3d) %-11s  %.1f %c -- %s", Old_Plug_Pointer+1,
+                                                         Plugins_Array[Old_Plug_Pointer].name,
+                                                         Plugins_Array[Old_Plug_Pointer].version,
+                                                         Plugins_Array[Old_Plug_Pointer].status,
+                                                         Plugins_Array[Old_Plug_Pointer].description);
+      wbkgdset(plugin_window, COLOR_PAIR(NORM_COLOR));
    }
 
    if (has_colors())
@@ -247,10 +268,11 @@ void Interface_Plugins_PointItem(char direction)
 
    wmove(plugin_window, scroll_yp + Plug_Pointer - Plug_Base_Pointer, LMARGIN);
    whline(plugin_window, ' ', W_SELECTPLUG);                         //select new position
-   wprintw(plugin_window, "%3d) %8s   v %.1f -- %s", Plug_Pointer+1,
-                                                      Plugin_Getname(Plugins_list[Plug_Pointer].name),
-                                                      Plugins_list[Plug_Pointer].version,
-                                                      Plugins_list[Plug_Pointer].description);
+   wprintw(plugin_window, "%3d) %-11s  %.1f %c -- %s", Plug_Pointer+1,
+                                                      Plugins_Array[Plug_Pointer].name,
+                                                      Plugins_Array[Plug_Pointer].version,
+                                                      Plugins_Array[Plug_Pointer].status,
+                                                      Plugins_Array[Plug_Pointer].description);
 
 
    pnoutrefresh(plugin_window, scroll_yp, 0, W_MAINY1 + 3, 3, W_MAINY2 - 3 , W_MAINX2 - 2);
@@ -297,12 +319,8 @@ void Interface_Plugins_Scroll(short direction)
 
 
 
-
-
 void Interface_Plugins_SelectItem(void)
 {
-   char plugin_file[100];
-   FILE *test;
    int KeyPress;
 
 #ifdef DEBUG
@@ -319,62 +337,281 @@ void Interface_Plugins_SelectItem(void)
    pnoutrefresh(plugin_window, scroll_yp, 0, W_MAINY1 + 3, 3, W_MAINY2 - 3 , W_MAINX2 - 2);
    doupdate();
 
-   strcpy(plugin_file, Plugins_list[Plug_Pointer].path);
-   strcat(plugin_file, Plugins_list[Plug_Pointer].name);
-
-   if ( (test = fopen(plugin_file, "r")) )
-      Plugin_RunPlugIn(plugin_file, NULL);
-   else
+   if (Plugins_Array[Plug_Pointer].status == 'E')
    {
-      Interface_Plugins_PopUp("Can't find the dinamyc plugin file !!");
-      return;
+      if (!Plugin_RunExt(Plugins_Array[Plug_Pointer].name))
+      {
+         Interface_Plugins_PopUp("There was an error loading this plugin !!");
+         return;
+      }
+
+      loop
+      {
+         KeyPress = wgetch(plugin_window);
+
+         switch(KeyPress)
+         {
+               case KEY_DOWN:
+                     Interface_Plugins_Scroll( +1 );
+                     break;
+
+               case KEY_UP:
+                     Interface_Plugins_Scroll( -1 );
+                     break;
+
+               case KEY_NPAGE:
+                     Interface_Plugins_Scroll( W_MAINY2-10 );     //PGDOWN
+                     break;
+
+               case KEY_PPAGE:
+                     Interface_Plugins_Scroll( -(W_MAINY2-10) );  //PGUP
+                     break;
+
+               case KEY_F(10):
+               case 'Q':
+               case 'q':
+                        scroll_yp = PAD_BUFFER - (W_MAINY2-10);
+                        Interface_Plugins_DrawScroller();
+                        return;
+                        break;
+         }
+      }
    }
+   else  // it is an hooking plugin
+   {
+      Plugins_Array[Plug_Pointer].status = (Plugins_Array[Plug_Pointer].status == 'A') ? ' ' : 'A';
+   }
+
+}
+
+
+void Interface_Plugins_Output_DrawScroller(void)
+{
+   short sheight = (W_MAINY2-1)*(W_MAINY2-1)/PAD_BUFFER;
+   short vpos = (W_MAINY2+1)*scroll_out_yp/PAD_BUFFER;
+
+   sheight = (sheight < 1) ? 1 : sheight;
+
+   vpos = (vpos == 0) ? 1 : vpos;
+   vpos = (vpos > W_MAINY2-sheight) ? W_MAINY2-sheight : vpos;
+
+   mvwvline(o_win, 1, W_MAINX2, ACS_VLINE, W_MAINY2-1);
+
+   wattron(o_win, A_REVERSE);
+   mvwvline(o_win, vpos, W_MAINX2, ' ', sheight);
+   wnoutrefresh(o_win);
+   wattroff(o_win, A_REVERSE);
+
+}
+
+void Interface_Plugins_Output_Scroll(short direction)
+{
+
+   scroll_out_yp += direction;
+   scroll_out_yp = (scroll_out_yp < 0) ? 0 : scroll_out_yp;
+   scroll_out_yp = (scroll_out_yp > PAD_BUFFER - (W_MAINY2-1)) ? PAD_BUFFER - (W_MAINY2-1) : scroll_out_yp;
+
+   pnoutrefresh(output_window, scroll_out_yp, 0, 1, 2, W_MAINY2-1 , W_MAINX2 - 2);
+   Interface_Plugins_Output_DrawScroller();
+
+   doupdate();
+}
+
+
+void Interface_Plugins_Output(void)
+{
+   fd_set msk_fd;
+   struct timeval TimeOut;
+   short saved_yp = scroll_out_yp;
+   int KeyPress;
+
+   memset(&TimeOut, 0, sizeof(TimeOut));  //  timeout = 0
+   FD_ZERO(&msk_fd);
+
+   if (o_win != NULL)
+   {
+      delwin(o_win);
+      o_win = NULL;
+   }
+
+   if (o_win == NULL)
+   {
+      o_win = newwin(W_MAINY2+1, W_MAINX2+1, 0, 0);
+      wbkgdset(o_win, COLOR_PAIR(HELP_COLOR));
+      wattron(o_win, A_BOLD);
+      box(o_win,ACS_VLINE,ACS_HLINE);
+      mvwprintw(o_win,  0, 2, "Plugin output :");
+      keypad(o_win, TRUE);
+   }
+
+   if (scroll_out_yp == 0) scroll_out_yp = PAD_BUFFER - 4;
+   scroll_out_yp -= (W_MAINY2-4);
+
+   Interface_Plugins_Output_DrawScroller();
+   wnoutrefresh(o_win);
+   pnoutrefresh(output_window, scroll_out_yp, 0, 1, 2, W_MAINY2-1 , W_MAINX2 - 2);
+   doupdate();
 
    loop
    {
-      KeyPress = wgetch(plugin_window);
+      FD_SET(0, &msk_fd);
 
-      switch(KeyPress)
+      select(FOPEN_MAX, &msk_fd, (fd_set *) 0, (fd_set *) 0, &TimeOut);
+
+      if (FD_ISSET(0, &msk_fd))
       {
-            case KEY_DOWN:
-                  Interface_Plugins_Scroll( +1 );
+         KeyPress = wgetch(o_win);
+
+         switch(KeyPress)
+         {
+               case KEY_DOWN:
+                     Interface_Plugins_Output_Scroll( +1 );
+                     break;
+
+               case KEY_UP:
+                     Interface_Plugins_Output_Scroll( -1 );
+                     break;
+
+               case KEY_NPAGE:
+                     Interface_Plugins_Output_Scroll( W_MAINY2 );     //PGDOWN
+                     break;
+
+               case KEY_PPAGE:
+                     Interface_Plugins_Output_Scroll( -(W_MAINY2) );  //PGUP
+                     break;
+
+               case 'D':
+               case 'd':
+                     {
+                        FILE *bin_dump, *str_dump;
+                        int ch, ok = 0;
+
+                        bin_dump = tmpfile();
+                        putwin(output_window, bin_dump);
+                        str_dump = fopen("plugin_output_dump.log", "wb");
+                        rewind(bin_dump);
+                        fseek(bin_dump, 0x48, SEEK_CUR); // skip the header file
+                        while( (ch = fgetc(bin_dump)) != EOF)
+                        {
+                           if (isprint(ch))
+                           {
+                              if (!ok && ch != 0x20) ok = 1;
+                              if (ok)
+                              {
+                                 if (ok++ == (W_MAINX2-2))
+                                 {
+                                    fputc('\n', str_dump);
+                                    ok = 1;
+                                 }
+                                 else
+                                    fputc(ch, str_dump);
+                              }
+                           }
+                        }
+                        Interface_PopUp("Data dumped to : plugin_output_dump.log");
+                        touchwin(o_win);
+                        wnoutrefresh(o_win);
+                     }
+                     break;
+
+               case KEY_F(1):
+               case 'H':
+               case 'h':{
+                           static char *help[] = {
+                              "[qQ][F10] - quit",
+                              "[arrows]  - scroll the output",
+                              "[dD]      - dump plugins output to a file",
+                              "[hH]      - this help screen",
+                              NULL};
+                           Interface_HelpWindow(help);
+                        }
+                  Interface_Plugins_Redraw();
                   break;
 
-            case KEY_UP:
-                  Interface_Plugins_Scroll( -1 );
-                  break;
-
-            case KEY_NPAGE:
-                  Interface_Plugins_Scroll( W_MAINY2-10 );     //PGDOWN
-                  break;
-
-            case KEY_PPAGE:
-                  Interface_Plugins_Scroll( -(W_MAINY2-10) );  //PGUP
-                  break;
-
-            case KEY_F(10):
-            case 'Q':
-            case 'q':
-                     scroll_yp = PAD_BUFFER - (W_MAINY2-10);
-                     Interface_Plugins_DrawScroller();
+               case KEY_F(10):
+               case 'Q':
+               case 'q':
+                     delwin(o_win);
+                     o_win = NULL;
+                     scroll_out_yp = saved_yp;
                      return;
                      break;
+         }
+      }
+      else
+      {
+         Interface_Plugins_PluginOutput();
+         pnoutrefresh(output_window, scroll_out_yp, 0, 1, 2, W_MAINY2-1 , W_MAINX2 - 2);
+         doupdate();
       }
    }
 
 }
 
 
+void Interface_Plugins_PluginOutput(void)
+{
+   int mesglen = 0;
+   char message[500];
+
+   Buffer_Get(pipe_with_plugins, &mesglen, sizeof(int));
+   Buffer_Get(pipe_with_plugins, message, mesglen);
+
+   message[mesglen] = 0;
+
+   if (!mesglen) return;
+
+   if (o_win == NULL)
+   {
+      o_win = newwin(5, W_MAINX2+1, 0, 0);
+      if (output_window == NULL)
+      {
+         output_window = newpad(PAD_BUFFER, W_MAINX2 - 2);
+         scroll_out_yp = PAD_BUFFER - 4;
+         wmove(output_window, scroll_out_yp, 0);
+         scrollok(output_window, TRUE);
+      }
+      wbkgdset(o_win, COLOR_PAIR(HELP_COLOR));
+      wattron(o_win, A_BOLD);
+      box(o_win,ACS_VLINE,ACS_HLINE);
+      mvwprintw(o_win,  0, 2, "Plugin output :");
+      wbkgdset(output_window, COLOR_PAIR(BOTTOM_COLOR));
+   }
+
+   touchwin(o_win);
+   wprintw(output_window, "%s", message);
+   wnoutrefresh(o_win);
+   pnoutrefresh(output_window, scroll_out_yp, 0, 1, 2, 3 , W_MAINX2 - 2);
+   //doupdate();
+}
+
+
+
+void Interface_Plugins_HidePluginOutput(void)
+{
+   if (o_win != NULL)
+   {
+      delwin(o_win);
+      o_win = NULL;
+      touchwin(top_window);
+      touchwin(b_top_window);
+      wnoutrefresh(b_top_window);
+      wnoutrefresh(top_window);
+      doupdate();
+   }
+}
+
+
 
 void Interface_Plugins_Run(void)
 {
-   int KeyPress;
+   int KeyPress, i;
 
 #ifdef DEBUG
    Debug_msg("Interface_Plugins_Run");
 #endif
 
-   P_Sel_Number = Sel_Number - 6;
+   P_Sel_Number = Sel_Number - 4;
 
    b_plugin_window = newwin(W_MAINY2 - W_MAINY1 - 3, W_MAINX2 - W_MAINX1 - 3, W_MAINY1 + 2, W_MAINX1 + 2);
    plugin_window = newpad(PAD_BUFFER, W_MAINX2 - 6);
@@ -399,7 +636,7 @@ void Interface_Plugins_Run(void)
    pnoutrefresh(plugin_window, scroll_yp, 0, W_MAINY1 + 3, 3, W_MAINY2 - 3 , W_MAINX2 - 2);
    doupdate();
 
-   number_of_plugins = Plugin_MakePlugList();
+   if (!Plugins_Array) number_of_plugins = Plugin_ExtArray();
 
    Interface_Plugins_DrawScroller();
    Interface_Plugins_InitList();
@@ -438,7 +675,8 @@ void Interface_Plugins_Run(void)
          case 'h':{
                      static char *help[] = {
                         "[qQ][F10] - quit",
-                        "[return]  - run the selected plug in",
+                        "[return]  - if external: run the selected plug in",
+                        "          - if hooking : activate/deactivate it",
                         "[hH]      - this help screen",
                         NULL};
                      Interface_HelpWindow(help);
@@ -453,10 +691,17 @@ void Interface_Plugins_Run(void)
 
          case 'q':
          case 'Q':
+         case KEY_F(10):
                   #ifdef DEBUG
                      Debug_msg("Interface_Plugin_Run_END");
                   #endif
-                  if (Options.silent && Options.plugin) Interface_WExit("They are safe!!  for now... ");
+
+                  for(i=0; i< number_of_plugins; i++)    // activate the plugins
+                     if (Plugins_Array[i].status == 'A')
+                        Plugin_SetActivation(Plugins_Array[i].name, 1);
+                     else if (Plugins_Array[i].status == ' ')
+                        Plugin_SetActivation(Plugins_Array[i].name, 0);
+
                   delwin(plugin_window);
                   delwin(b_plugin_window);
                   touchwin(main_window);
@@ -465,15 +710,6 @@ void Interface_Plugins_Run(void)
                   plugin_window = NULL; // for winch see ec_signal.c
                   return;
                   break;
-
-         case KEY_F(10):
-                  #ifdef DEBUG
-                     Debug_msg("Interface_Plugin_Run_END");
-                     Debug_msg("Interface_Run_END");
-                  #endif
-                  Interface_WExit("Quitting... ");
-                  break;
-
       }
    }
 
