@@ -17,21 +17,19 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_interface_sniff.c,v 1.2 2001/08/30 10:05:19 alor Exp $
+    $Id: ec_interface_sniff.c,v 1.11 2001/12/09 20:24:51 alor Exp $
 */
 
 #include "include/ec_main.h"
 
 #ifdef HAVE_NCURSES  // don't compile if ncurses interface is not supported
 
-#include <string.h>
 #ifdef HAVE_NCURSES_H
    #include <ncurses.h>
 #else
    #include <curses.h>
 #endif
-#include <unistd.h>
-#include <signal.h>
+
 #include <sys/time.h>
 
 #include "include/ec_interface.h"
@@ -50,10 +48,7 @@
 #include "include/ec_buffer.h"
 #include "include/ec_error.h"
 #include "include/ec_logtofile.h"
-
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
+#include "include/ec_thread.h"
 
 #ifdef PERMIT_PLUGINS
    #include "include/ec_interface_plugins.h"
@@ -79,6 +74,7 @@ void Interface_Sniff_InitList(void);
 void Interface_Sniff_RefreshList(void);
 void Interface_Sniff_KillConn(void);
 void Interface_Sniff_ActiveDissecor(short mode);
+void * Interface_Sniff_Timeouter(void *dummy);
 
 // global variables
 
@@ -105,7 +101,6 @@ void Interface_Sniff_InitList(void)
    int Conn_Top_Pointer;
    char info[250];
 
-
    if (has_colors())
       wbkgdset(main_window, COLOR_PAIR(MAIN_COLOR));
    else
@@ -160,6 +155,7 @@ void Interface_Sniff_InitList(void)
    wnoutrefresh(bottom_window);
    wnoutrefresh(main_window);
    doupdate();
+
 }
 
 
@@ -263,7 +259,7 @@ void Interface_Sniff_RefreshList(void)
    wnoutrefresh(message_window);
    doupdate();
 
-   number_of_connections = Decodedata_RefreshConnectionList();
+   Decodedata_RefreshConnectionList();
    Conn_Pointer = 0;
    Conn_Base_Pointer = 0;
 
@@ -274,6 +270,9 @@ void Interface_Sniff_RefreshList(void)
 
    doupdate();
 
+#ifdef DEBUG
+   Debug_msg("Interface_Sniff_RefreshList -- end");
+#endif
 }
 
 
@@ -334,24 +333,63 @@ void Interface_Sniff_ActiveDissecor(short mode)
 }
 
 
+void * Interface_Sniff_Timeouter(void *dummy)      // Timeouter thread
+{
+   int i;
+   time_t now;
+   extern pthread_mutex_t connection_mutex;
+
+   loop
+   {
+      pthread_testcancel();
+
+      if (number_of_connections <= 0)
+      {
+         usleep(5000);
+         continue;
+      }
+
+      now = time(NULL);
+
+      pthread_mutex_lock(&connection_mutex);
+
+      for (i = 0; i < number_of_connections; i++)
+      {
+         if (Conn_Between_Hosts[i].proto == 'T')
+         {
+            if (!strcmp(Conn_Between_Hosts[i].status, "ACTIVE") && Conn_Between_Hosts[i].timestamp <= now - CONN_TIME_SILENT)
+               strcpy(Conn_Between_Hosts[i].status, "silent");
+
+            if (strcmp(Conn_Between_Hosts[i].status, "KILLED") &&
+                strcmp(Conn_Between_Hosts[i].status, "CLOSED") && Conn_Between_Hosts[i].timestamp <= now - CONN_TIME_TIMEOUT) // 10 min of timeout
+               strcpy(Conn_Between_Hosts[i].status, "timeout");
+         }
+      }
+
+      pthread_mutex_unlock(&connection_mutex);
+
+      sleep(2);
+   }
+}
+
+
 
 void Interface_Sniff_Run(short mode)
 {
    int KeyPress, i;
-   int Illithid_pid = 0, Dopple_pid = 0;
+   pthread_t Illithid_pid = 0, Dopple_pid = 0, time_pid = 0;
    #if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
-      int Grell_pid = 0;
+      pthread_t Grell_pid = 0;
    #endif
    fd_set msk_fd;
    struct timeval TimeOut;
-   CONNECTION data_from_illithid;
 
 #ifdef DEBUG
    Debug_msg("Interface_Sniff_Run -- mode %d", mode);
 #endif
 
-   if (pipe_with_illithid == -1) pipe_with_illithid = Buffer_Create(5.0e5);      // creates the buffers
-   if (pipe_with_illithid_data == -1) pipe_with_illithid_data = Buffer_Create(1.0e6); // 1Mb in shared memory !!!
+   if (pipe_with_illithid_data == -1) pipe_with_illithid_data = Buffer_Create(1.0e6);
+   if (pipe_with_plugins == -1) pipe_with_plugins = Buffer_Create(1.5e5);  // 150 Kbyte
 
    wbkgdset(main_window, COLOR_PAIR(MAIN_COLOR));
    werase(main_window);
@@ -372,8 +410,6 @@ void Interface_Sniff_Run(short mode)
    Interface_Sniff_RefreshList();
    Interface_Sniff_InitList();
 
-   Buffer_Flush(pipe_with_illithid);
-
    switch (mode)
    {
       case ARPBASED:
@@ -383,9 +419,9 @@ void Interface_Sniff_Run(short mode)
                      for(i=0; i<number_of_hosts_in_lan; i++)
                      {
                         if ( !strcmp(Host_Source.ip, Host_In_LAN[i].ip) )
-                           strcpy(Host_Source.mac, Host_In_LAN[i].mac);
+                           strlcpy(Host_Source.mac, Host_In_LAN[i].mac, sizeof(Host_Source.mac));
                         if ( !strcmp(Host_Dest.ip, Host_In_LAN[i].ip) )
-                           strcpy(Host_Dest.mac, Host_In_LAN[i].mac);
+                           strlcpy(Host_Dest.mac, Host_In_LAN[i].mac, sizeof(Host_Dest.mac));
                      }
 
                      if ( !strcmp(Host_Source.mac, "") && !strcmp(Host_Dest.mac, "") )
@@ -416,26 +452,27 @@ void Interface_Sniff_Run(short mode)
    memset(&TimeOut, 0, sizeof(TimeOut));  //  timeout = 0
    FD_ZERO(&msk_fd);
 
+   time_pid = ECThread_create("timeouter", &Interface_Sniff_Timeouter, NULL);
+
    loop
    {
-      int datalen;
       FD_SET(0, &msk_fd);
 
       select(FOPEN_MAX, &msk_fd, (fd_set *) 0, (fd_set *) 0, &TimeOut);
 
-      datalen = Buffer_Get(pipe_with_illithid, &data_from_illithid, sizeof(CONNECTION));
+      Interface_Sniff_InitList();
 
-      if (datalen > 0)
-      {
-         number_of_connections = Decodedata_MakeConnectionList(data_from_illithid);
-         Interface_Sniff_InitList();
-      }
-      else  // workaround for the 100% CPU usage...
-         usleep(5);
+#ifdef PERMIT_PLUGINS
+      Interface_Plugins_PluginOutput();
+#endif
 
       if (FD_ISSET(0, &msk_fd))
       {
 
+#ifdef PERMIT_PLUGINS
+         Interface_Plugins_HidePluginOutput();
+#endif
+
          KeyPress = wgetch(main_window);
 
          switch (KeyPress)
@@ -461,12 +498,8 @@ void Interface_Sniff_Run(short mode)
                      wmove(bottom_window, 0, 0);
                      mvwprintw(top_window, 2, 52, "                    ");
 
-                     kill(Illithid_pid, SIGUSR2);
-                     #if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
-                        if (Grell_pid)
-                           if ( kill(-Grell_pid, SIGUSR2) == -1)   // send the signal to the Grell's group (-PID)
-                              kill(Grell_pid, SIGUSR2);
-                     #endif
+                     Connection_Mode = 0;
+
                      Interface_Sniff_Data_Run(Conn_Between_Hosts[Conn_Pointer].source_ip,
                                               Conn_Between_Hosts[Conn_Pointer].source_port,
                                               Conn_Between_Hosts[Conn_Pointer].dest_ip,
@@ -477,26 +510,19 @@ void Interface_Sniff_Run(short mode)
                                               Conn_Between_Hosts[Conn_Pointer].type,
                                               mode);
                      // wait and then...
-                     kill(Illithid_pid, SIGUSR2);
-                     #if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
-                        if (Grell_pid)
-                           if ( kill(-Grell_pid, SIGUSR2) == -1)   // send the signal to the Grell's group (-PID)
-                              kill(Grell_pid, SIGUSR2);
-                     #endif
+                     Connection_Mode = 1;
 
                      Interface_Sniff_ActiveDissecor(mode);
                      if (filter_on_source && FilterDrop_CheckMode(Filter_Array_Source, mode))
                      {
                         Interface_PopUp("The Source filter chain can be used only in ARPBASED mode !!");
                         filter_on_source = 0;
-                        write(pipe_filter[1], "S", 1);
                         Interface_Inject_FilterStatus();
                      }
                      if (filter_on_dest && FilterDrop_CheckMode(Filter_Array_Dest, mode))
                      {
                         Interface_PopUp("The Dest filter chain can be used only in ARPBASED mode !!");
                         filter_on_dest = 0;
-                        write(pipe_filter[1], "D", 1);
                         Interface_Inject_FilterStatus();
                      }
                      Interface_Sniff_InitList();
@@ -515,10 +541,11 @@ void Interface_Sniff_Run(short mode)
                         char source[40];
                         char dest[40];
                         if (Conn_Between_Hosts[Conn_Pointer].source_ip == NULL) break;   // no connections
-                        snprintf(source, 40, " %s (%s) <-->", Inet_HostName(Conn_Between_Hosts[Conn_Pointer].source_ip),
+                        snprintf(source, 39, " %s (%s) <-->", Inet_HostName(Conn_Between_Hosts[Conn_Pointer].source_ip),
                                                               Conn_Between_Hosts[Conn_Pointer].source_ip);
-                        snprintf(dest, 40, " %s (%s)", Inet_HostName(Conn_Between_Hosts[Conn_Pointer].dest_ip),
+                        snprintf(dest, 39, " %s (%s)", Inet_HostName(Conn_Between_Hosts[Conn_Pointer].dest_ip),
                                                        Conn_Between_Hosts[Conn_Pointer].dest_ip);
+                        source[39] = dest[39] = 0;
                         Interface_PopUp("%s%s", source, dest);
                      }
                      break;
@@ -544,6 +571,13 @@ void Interface_Sniff_Run(short mode)
             case 'P':
             case 'p':
                      Interface_Plugins_Run();
+                     Buffer_Flush(pipe_with_plugins);
+                     break;
+
+            case 'I':
+            case 'i':
+                     Interface_Plugins_Output();
+                     Interface_Redraw();
                      break;
 #endif
 
@@ -551,8 +585,10 @@ void Interface_Sniff_Run(short mode)
             case 'a':
                      if (mode <= PUBLICARP)
                      {
-                        kill(Illithid_pid, SIGUSR1);  // activate/deactivate special dissector
-                        kill(getpid(), SIGUSR1);
+                        active_dissector = (active_dissector) ? 0 : 1;     // activate/deactivate active dissector (arp based)
+                        #ifdef DEBUG
+                           Debug_msg("\tactive_dissector %d", active_dissector);
+                        #endif
                         Interface_Sniff_ActiveDissecor(mode);
                      }
                      else
@@ -589,7 +625,8 @@ void Interface_Sniff_Run(short mode)
                            "[lL]      - log all collected passwords to a file",
                            "[kK]      - kill the connection (be careful !)",
 #ifdef PERMIT_PLUGINS
-                           "[pP]      - run a plugin",
+                           "[pP]      - plugin management",
+                           "[iI]      - plugin output window",
 #endif
                            "[oO]      - passive scanning of the LAN",
                            "[dD]      - resolve ip via DNS",
@@ -610,11 +647,29 @@ void Interface_Sniff_Run(short mode)
                         if ((answer != 'y') && (answer != 'Y'))
                            break;
                      }
-                     kill(Illithid_pid, SIGTERM);
-                     if (Dopple_pid) kill(Dopple_pid, SIGTERM);
-                     #if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
-                        if (Grell_pid) kill(Grell_pid, SIGTERM);
-                     #endif
+                     {
+                        WINDOW *message_window;
+                        char mess[29] = "shutting down all threads...";
+
+                        message_window = newwin(5, strlen(mess) + 4,0,0);
+                        mvwin(message_window, W_MAINY1+(W_MAINY2-W_MAINY1)/2-2, W_MAINX2/2 - (strlen(mess) + 4)/2 );
+                        wbkgdset(message_window, COLOR_PAIR(TITLE_COLOR));
+                        wattron(message_window, A_BOLD);
+                        box(message_window,ACS_VLINE,ACS_HLINE);
+                        mvwprintw(message_window,  2, 2, "%s", mess);
+                        wnoutrefresh(message_window);
+                        doupdate();
+
+                        ECThread_destroy(Illithid_pid);
+                        ECThread_destroy(time_pid);
+                        if (Dopple_pid) ECThread_destroy(Dopple_pid);
+                        #if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
+                           if (Grell_pid) ECThread_destroy(Grell_pid);
+                        #endif
+                        delwin(message_window);
+                        touchwin(main_window);
+                        wnoutrefresh(main_window);
+                     }
                      #ifdef DEBUG
                         Debug_msg("Interface_Sniff_END");
                      #endif
@@ -622,6 +677,7 @@ void Interface_Sniff_Run(short mode)
                      werase(bottom_window);
                      wnoutrefresh(bottom_window);
                      doupdate();
+                     Decodedata_RefreshConnectionList();
                      number_of_connections = -1;
                      return;
                      break;
@@ -631,15 +687,35 @@ void Interface_Sniff_Run(short mode)
                         Debug_msg("Interface_Sniff_END");
                         Debug_msg("Interface_Run_END");
                      #endif
-                     kill(Illithid_pid, SIGTERM);
-                     if (Dopple_pid) kill(Dopple_pid, SIGTERM);
-                     #if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
-                        if (Grell_pid) kill(Grell_pid, SIGTERM);
-                     #endif
+                     {
+                        WINDOW *message_window;
+                        char mess[29] = "shutting down all threads...";
+
+                        message_window = newwin(5, strlen(mess) + 4,0,0);
+                        mvwin(message_window, W_MAINY1+(W_MAINY2-W_MAINY1)/2-2, W_MAINX2/2 - (strlen(mess) + 4)/2 );
+                        wbkgdset(message_window, COLOR_PAIR(TITLE_COLOR));
+                        wattron(message_window, A_BOLD);
+                        box(message_window,ACS_VLINE,ACS_HLINE);
+                        mvwprintw(message_window,  2, 2, "%s", mess);
+                        wnoutrefresh(message_window);
+                        doupdate();
+
+                        ECThread_destroy(Illithid_pid);
+                        ECThread_destroy(time_pid);
+                        if (Dopple_pid) ECThread_destroy(Dopple_pid);
+                        #if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
+                           if (Grell_pid) ECThread_destroy(Grell_pid);
+                        #endif
+                        delwin(message_window);
+                        touchwin(main_window);
+                        wnoutrefresh(main_window);
+                     }
                      Interface_WExit("They are safe!!  for now... ");
                      break;
          }
       }
+      else  // workaround for the 100% CPU usage...
+         usleep(1);
 
    }
 
