@@ -17,27 +17,26 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_interface_sniff_data.c,v 1.2 2001/09/06 20:07:42 alor Exp $
+    $Id: ec_interface_sniff_data.c,v 1.8 2001/12/09 20:24:51 alor Exp $
 */
 
 #include "include/ec_main.h"
 
 #ifdef HAVE_NCURSES  // don't compile if ncurses interface is not supported
 
-#include <string.h>
 #ifdef HAVE_NCURSES_H
    #include <ncurses.h>
 #else
    #include <curses.h>
 #endif
+
 #include <sys/time.h>
 #include <sys/types.h>
-#include <unistd.h>
-#include <signal.h>
 #include <time.h>
 
 #include "include/ec_interface.h"
 #include "include/ec_interface_inject.h"
+#include "include/ec_interface_plugins.h"
 #include "include/ec_decodedata.h"
 #include "include/ec_dissector.h"
 #include "include/ec_illithid.h"
@@ -45,10 +44,6 @@
 #include "include/ec_error.h"
 #include "include/ec_inet_structures.h"
 
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
-
 #define PAD_BUFFER 1000        // buffered lines for sniffing
 
 
@@ -58,7 +53,7 @@
 #define POINT_COLOR  4
 #define SEL_COLOR    5
 #define HELP_COLOR   6
-#define SNIFF_COLOR  7
+#define SNIFF_COLOR  6
 
 #define KEY_TAB      '\t'     // they aren't defined in ncurses.h :(
 #define KEY_RETURN   10
@@ -66,14 +61,14 @@
 
 #define ASCII_VIEW   0        // data in the sniffing windows...
 #define HEX_VIEW     1
-
+#define TEXT_VIEW    2
 
 // protos...
 
-void Interface_Sniff_Data_Run(char *ipsource, int portsource, char *ipdest, int portdest, char *macsource, char *macdest, char proto, char *app, short mode);
+void Interface_Sniff_Data_Run(char *ips, int portsource, char *ipd, int portdest, char *macs, char *macd, char proto, char *type, short mode);
 void Interface_Sniff_Data_Redraw(void);
 void Interface_Sniff_Data_KeyTab(char *ipsource, int portsource, char *ipdest, int portdest);
-void Interface_Sniff_Data_SniffData(char *source, char *dest, int portsource, int portdest, char proto);
+void Interface_Sniff_Data_SniffData(void);
 void Interface_Sniff_Data_LogToFile(char proto, char *ipsource, int portsource, char *ipdest, int portdest);
 void Interface_Sniff_Data_Winch(void);
 void Interface_Sniff_Data_StopCont(void);
@@ -172,6 +167,11 @@ void Interface_Sniff_Data_KeyTab(char *ipsource, int portsource, char *ipdest, i
                         mvwprintw(data_source_win, W_MAINY2 - W_MAINY1 - 4, 2, "HEX");
                         mvwprintw(data_dest_win, W_MAINY2 - W_MAINY1 - 4, 2, "HEX");
                         break;
+
+       case TEXT_VIEW:
+                        mvwprintw(data_source_win, W_MAINY2 - W_MAINY1 - 4, 2, "TEXT");
+                        mvwprintw(data_dest_win, W_MAINY2 - W_MAINY1 - 4, 2, "TEXT");
+                        break;
    }
 
    if (stop) mvwprintw(data_source_win, W_MAINY2 - W_MAINY1 - 4, 2, "STOPPED");
@@ -211,17 +211,17 @@ void Interface_Sniff_Data_LogToFile(char proto, char *ipsource, int portsource,
    tt = time(NULL);
    dd = localtime(&tt);
 
-   sprintf(date, "%04d%02d%02d", dd->tm_year+1900, dd->tm_mon+1, dd->tm_mday);
+   snprintf(date, sizeof(date), "%04d%02d%02d", dd->tm_year+1900, dd->tm_mon+1, dd->tm_mday);
 
    if (win_pointer == data_source_win)
    {
       strcpy(question, "Do U want to log SOURCE-to-DEST to this file ? (y/n)");
-      sprintf(filename, "%s-%c-%s:%d-%s:%d.log", date, proto, ipsource, portsource, ipdest, portdest);
+      snprintf(filename, sizeof(filename), "%s-%c-%s:%d-%s:%d.log", date, proto, ipsource, portsource, ipdest, portdest);
    }
    else
    {
       strcpy(question, "Do U want to log DEST-to-SOURCE to this file ? (y/n)");
-      sprintf(filename, "%s-%c-%s:%d-%s:%d.log", date, proto, ipdest, portdest, ipsource, portsource);
+      snprintf(filename, sizeof(filename), "%s-%c-%s:%d-%s:%d.log", date, proto, ipdest, portdest, ipsource, portsource);
    }
 
    question_window = newwin(7, strlen(question) + 4,0,0);
@@ -229,12 +229,12 @@ void Interface_Sniff_Data_LogToFile(char proto, char *ipsource, int portsource,
    wbkgdset(question_window, COLOR_PAIR(TITLE_COLOR));
    wattron(question_window, A_BOLD);
    box(question_window,ACS_VLINE,ACS_HLINE);
-   mvwprintw(question_window,  2, 2, question);
+   mvwprintw(question_window,  2, 2, "%s", question);
    wnoutrefresh(question_window);
    doupdate();
 
    wbkgdset(question_window, COLOR_PAIR(HELP_COLOR));
-   mvwprintw(question_window,  4, 4, filename);
+   mvwprintw(question_window,  4, 4, "%s", filename);
    answer = wgetch(question_window);
    delwin(question_window);
 
@@ -342,6 +342,13 @@ void Interface_Sniff_Data_View(short mode)
                         mvwprintw(data_source_win, W_MAINY2 - W_MAINY1 - 4, 2, "HEX");
                         mvwprintw(data_dest_win, W_MAINY2 - W_MAINY1 - 4, 2, "HEX");
                         break;
+
+      case TEXT_VIEW:
+                        wmove(data_source_win, W_MAINY2 - W_MAINY1 - 4, 2);   whline(data_source_win, ACS_HLINE, 7);
+                        wmove(data_dest_win, W_MAINY2 - W_MAINY1 - 4, 2); whline(data_dest_win, ACS_HLINE, 7);
+                        mvwprintw(data_source_win, W_MAINY2 - W_MAINY1 - 4, 2, "TEXT");
+                        mvwprintw(data_dest_win, W_MAINY2 - W_MAINY1 - 4, 2, "TEXT");
+                        break;
    }
 
    view = mode;
@@ -360,26 +367,20 @@ void Interface_Sniff_Data_View(short mode)
 
 
 
-void Interface_Sniff_Data_SniffData(char *source, char *dest, int portsource, int portdest, char proto)
+void Interface_Sniff_Data_SniffData(void)
 {
    SNIFFED_DATA data_from_illithid;
    int datalen;
 
-   memset(&data_from_illithid, 0, sizeof(SNIFFED_DATA));
-
    datalen = Buffer_Get(pipe_with_illithid_data, &data_from_illithid, sizeof(SNIFFED_DATA));
 
    if (datalen<=0)
    {
-       usleep(1000);
+       usleep(1);
        return;
    }
 
-   // Not from our stream
-   if (!Illithid_ToBeSniffed(source, portsource, dest, portdest, &data_from_illithid) || proto!=data_from_illithid.proto)
-       return;
-
-   if (!strcmp(source, data_from_illithid.source_ip))
+   if ( current_illithid_data.source_ip == data_from_illithid.fast_source_ip)
    {
       if (log_source) { write(fileno(file_source), &data_from_illithid.data, data_from_illithid.datasize); fflush(file_source);}
       if (!stop)
@@ -389,6 +390,12 @@ void Interface_Sniff_Data_SniffData(char *source, char *dest, int portsource, in
                            wprintw(data_source, "%s", Decodedata_GetAsciiData(data_from_illithid.data, data_from_illithid.datasize));
                            pnoutrefresh(data_source, scroll_ys,0, W_MAINY1 + 3, 3, W_MAINY2 - 3 , W_MAINX2 / 2 - 2);
                            break;
+
+            case TEXT_VIEW:
+                           wprintw(data_source, "%s", Decodedata_GetTextData(data_from_illithid.data, data_from_illithid.datasize));
+                           pnoutrefresh(data_source, scroll_ys,0, W_MAINY1 + 3, 3, W_MAINY2 - 3 , W_MAINX2 / 2 - 2);
+                           break;
+
             case HEX_VIEW:
                            if (data_from_illithid.proto == 'T')
                               wprintw(data_source, "\n> S %lx A %lx (%s) <\n%s",
@@ -413,6 +420,12 @@ void Interface_Sniff_Data_SniffData(char *source, char *dest, int portsource, in
                            wprintw(data_dest, "%s", Decodedata_GetAsciiData(data_from_illithid.data, data_from_illithid.datasize));
                            pnoutrefresh(data_dest, scroll_yd,0, W_MAINY1 + 3, W_MAINX2 / 2 + 2, W_MAINY2 - 3 , W_MAINX2 - 2);
                            break;
+
+            case TEXT_VIEW:
+                           wprintw(data_dest, "%s", Decodedata_GetTextData(data_from_illithid.data, data_from_illithid.datasize));
+                           pnoutrefresh(data_dest, scroll_yd,0, W_MAINY1 + 3, W_MAINX2 / 2 + 2, W_MAINY2 - 3 , W_MAINX2 - 2);
+                           break;
+
             case HEX_VIEW:
                            if (data_from_illithid.proto == 'T')
                               wprintw(data_dest, "\n> S %lx A %lx (%s) <\n%s",
@@ -582,22 +595,30 @@ void Interface_Sniff_Data_Kill(char proto, char *ipsource, short portsource, cha
 
 
 
-void Interface_Sniff_Data_Run(char *ipsource, int portsource, char *ipdest, int portdest, char *macsource, char *macdest, char proto, char *app, short mode)
+void Interface_Sniff_Data_Run(char *ips, int portsource, char *ipd, int portdest, char *macs, char *macd, char proto, char *type, short mode)
 {
    int KeyPress;
    int dimY = W_MAINY2 - W_MAINY1 - 3;
    int dimX = W_MAINX2 / 2 - 2;
    fd_set msk_fd;
    struct timeval TimeOut;
+   struct in_addr addr;
+   char ipsource[16];
+   char ipdest[16];
+   char macsource[20];
+   char macdest[20];
+   char app[18];
 
-   Buffer_Flush(pipe_with_illithid_data);
+   strlcpy(ipsource, ips, sizeof(ipsource));     // save locally because other threads can
+   strlcpy(ipdest, ipd, sizeof(ipdest));       // realloc the array containing these values
+   strlcpy(macsource, macs, sizeof(macsource));
+   strlcpy(macdest, macd, sizeof(macdest));
+   strlcpy(app, type, sizeof(app));
 
 #ifdef DEBUG
    Debug_msg("Interface_Sniff_Data_Run -- %d -- %c [%s:%d] [%s:%d] [%s] [%s]", mode, proto, ipsource, portsource, ipdest, portdest, macsource, macdest );
 #endif
 
-   init_pair(SNIFF_COLOR, COLOR_CYAN, COLOR_BLACK);
-
    data_source_win = newwin(dimY, dimX, W_MAINY1 + 2, 2);
    data_dest_win = newwin(dimY, dimX, W_MAINY1 + 2, W_MAINX2 / 2 + 1);
    data_source = newpad(PAD_BUFFER, dimX - 2);
@@ -645,7 +666,7 @@ void Interface_Sniff_Data_Run(char *ipsource, int portsource, char *ipdest, int
    wbkgdset(bottom_window, COLOR_PAIR(SNIFF_COLOR));
    wprintw(bottom_window, "Application: ");
    wbkgdset(bottom_window, COLOR_PAIR(BOTTOM_COLOR));
-   wprintw(bottom_window, app);
+   wprintw(bottom_window, "%s", app);
 
    redrawwin(main_window);
    wnoutrefresh(main_window);
@@ -667,6 +688,17 @@ void Interface_Sniff_Data_Run(char *ipsource, int portsource, char *ipdest, int
    memset(&TimeOut, 0, sizeof(TimeOut));  //  timeout = 0
    FD_ZERO(&msk_fd);
 
+   current_illithid_data.proto = proto;
+   current_illithid_data.source_port = portsource;
+   current_illithid_data.dest_port = portdest;
+
+   if (inet_aton(ipsource, &addr))
+      current_illithid_data.source_ip = ntohl(addr.s_addr);
+   if (inet_aton(ipdest, &addr))
+      current_illithid_data.dest_ip =  ntohl(addr.s_addr);
+
+   Buffer_Flush(pipe_with_illithid_data);
+
    loop
    {
 
@@ -674,10 +706,19 @@ void Interface_Sniff_Data_Run(char *ipsource, int portsource, char *ipdest, int
 
       select(FOPEN_MAX, &msk_fd, (fd_set *) 0, (fd_set *) 0, &TimeOut);
 
-      Interface_Sniff_Data_SniffData(ipsource, ipdest, portsource, portdest, proto);
+      Interface_Sniff_Data_SniffData();
+
+#ifdef PERMIT_PLUGINS
+      Interface_Plugins_PluginOutput();
+#endif
 
       if (FD_ISSET(0, &msk_fd))
       {
+
+#ifdef PERMIT_PLUGINS
+         Interface_Plugins_HidePluginOutput();
+#endif
+
          KeyPress = wgetch(main_window);
 
          switch (KeyPress)
@@ -713,6 +754,11 @@ void Interface_Sniff_Data_Run(char *ipsource, int portsource, char *ipdest, int
                   Interface_Sniff_Data_View(HEX_VIEW);
                   break;
 
+            case 'T':
+            case 't':
+                  Interface_Sniff_Data_View(TEXT_VIEW);
+                  break;
+
             case 'S':
             case 's':
                   Interface_Sniff_Data_StopCont();
@@ -757,6 +803,14 @@ void Interface_Sniff_Data_Run(char *ipsource, int portsource, char *ipdest, int
                   Interface_Sniff_Data_Redraw();
                   break;
 
+#ifdef PERMIT_PLUGINS
+            case 'P':
+            case 'p':
+                     Interface_Plugins_Run();
+                     Interface_Sniff_Data_Redraw();
+                     break;
+#endif
+
             case KEY_F(1):
             case 'H':
             case 'h':{
@@ -767,9 +821,13 @@ void Interface_Sniff_Data_Run(char *ipsource, int portsource, char *ipdest, int
                         "[pgDw]    - scroll forward (as well as DownArrow)",
                         "[iI]      - inject characters in the connection",
                         "[fF]      - set/edit filters chains",
+#ifdef PERMIT_PLUGINS
+                        "[pP]      - run a plugin",
+#endif
                         "[kK]      - kill the connection (be careful !)",
                         "[aA]      - ASCII view",
                         "[xX]      - HEX view",
+                        "[tT]      - TEXT (readable chars) only view",
                         "[sS]      - stop/cont the sniff (only visualization)",
                         "[lL]      - Log to file",
                         NULL};
