@@ -17,18 +17,14 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_main.c,v 1.5 2001/09/10 16:41:33 alor Exp $
+    $Id: ec_main.c,v 1.15 2001/12/08 16:38:43 alor Exp $
 */
 
 #include "include/ec_main.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
 #include <sys/ioctl.h>
 #ifdef HAVE_TERMIOS_H
-	#include <termios.h>
+   #include <termios.h>
 #endif
 #ifdef HAVE_GETOPT_H
    #include <getopt.h>
@@ -43,28 +39,26 @@
 #include "include/ec_signal.h"
 #include "include/ec_parser.h"
 #include "include/ec_filterdrop.h"
+#include "include/ec_thread.h"
+#include "include/ec_plugins.h"
 
 #ifdef HAVE_NCURSES
     #include "include/ec_interface.h"
 #endif
 
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
-
 // global variables
 
-char *program_argv0;
-
 HOST *Host_In_LAN = NULL;              // ec_main.h
 int number_of_hosts_in_lan;
 
 CONNECTION *Conn_Between_Hosts = NULL; // ec_main.h
 int number_of_connections = -1;
 
-PASSIVE_DATA *Passive_Host;				// ec_main.h
+PASSIVE_DATA *Passive_Host;            // ec_main.h
 int number_of_passive_hosts;
 
+CURRENT_SNIFFED_DATA current_illithid_data;
+
 HOST Host_Source;
 HOST Host_Dest;
 
@@ -75,11 +69,9 @@ char *Execute_Plugin;
 
 OPTIONS Options;
 
-int pipe_with_dryad = -1;
-int pipe_with_illithid = -1;
 int pipe_with_illithid_data = -1;
+int pipe_with_plugins = -1;
 int pipe_inject[2];
-int pipe_filter[2];
 int pipe_kill[2];
 
 char active_dissector = 1;    // ec_main.h
@@ -92,7 +84,7 @@ char filter_on_dest = 0;
 void Main_Usage(void);
 void Main_Interactive(void);
 void Main_Normal(void);
-void Main_CheckForRun(void);
+void Main_CheckForRun(char *program_argv0);
 void Main_CheckForFilters(void);
 void Main_ParseParameters(char *first, char *second, char *third, char *fourth);
 void Main_Check_NewRelease(void);
@@ -104,14 +96,14 @@ void Main_Devel_Release(void);
 int main(int argc, char *argv[])
 {
 
-   program_argv0 = argv[0];
-
 #ifdef DEBUG
    Debug_Init();
    Debug_msg("Main");
 #endif
 
-   Main_CheckForRun();     // it is ok ?
+   ECThread_register(pthread_self(), "ettercap");
+
+   Main_CheckForRun(argv[0]);     // it is ok ?
 
    Signal_SigBuster();     // signal masking
 
@@ -152,7 +144,6 @@ int main(int argc, char *argv[])
    if (Options.filter) Main_CheckForFilters();
 
    pipe(pipe_inject);      // create the pipes with illithid
-   pipe(pipe_filter);
    pipe(pipe_kill);
 
    if (Options.normal)
@@ -194,7 +185,7 @@ void Main_Usage(void)
    fprintf (stdout, "\nGeneral options:\n");
    fprintf (stdout, "  -N, --simple                 NON interactive mode (without ncurses)\n");
    fprintf (stdout, "  -z, --silent                 silent mode (no arp storm on start up)\n");
-	fprintf (stdout, "  -O, --passive                passive scanning of the LAN\n");
+   fprintf (stdout, "  -O, --passive                passive scanning of the LAN\n");
    fprintf (stdout, "  -b, --broadping              broadcast ping instead of arp storm on start up\n");
    fprintf (stdout, "  -D, --delay <n sec>          the dalay between arp replies (default is 30 sec)\n");
    fprintf (stdout, "  -Z, --stormdelay <n usec>    the dalay between arp request (def is 1500 usec)\n");
@@ -204,6 +195,8 @@ void Main_Usage(void)
    fprintf (stdout, "  -i, --iface <iface>          network interface to be used\n");
    fprintf (stdout, "  -n, --netmask <netmask>      the netmask used to scan the lan\n");
    fprintf (stdout, "  -e, --etterconf <filename>   load options from a config file\n");
+   fprintf (stdout, "  -j, --loadhosts <filename>   load hosts list from a file\n");
+   fprintf (stdout, "  -k, --savehosts              save hosts list to a file\n");
    fprintf (stdout, "  -v, --version                check for the latest ettercap version\n");
    fprintf (stdout, "  -y, --yes                    in combination with -v auto answer yes\n");
    fprintf (stdout, "  -h, --help                   this help screen\n");
@@ -222,10 +215,10 @@ void Main_Usage(void)
    fprintf (stdout, "  -x, --hexview                display data in hex mode\n");
    fprintf (stdout, "  -L, --logtofile              logs all data to specific file(s)\n");
    fprintf (stdout, "  -q, --quiet                  \"demonize\" ettercap (useful with -L)\n");
-   fprintf (stdout, "  -k, --newcert                create a new SSL cert file for HTTPS dissector\n");
+   fprintf (stdout, "  -w, --newcert                create a new SSL cert file for HTTPS dissector\n");
    fprintf (stdout, "  -F, --filter <filename>      load  \"filename\" as the filter chain file\n");
    fprintf (stdout, "  -c, --check                  check for other poisoners in the LAN\n");
-   fprintf (stdout, "  -t, --linktype               am I in a switched LAN or not ?\n");
+   fprintf (stdout, "  -t, --linktype               tries to indentify the LAN type (switch or hub)\n");
    fprintf (stdout, "\n");
 
    exit (0);
@@ -268,6 +261,10 @@ void Main_Interactive(void)
          Error_msg("Screen must be at least 25x80 !!");
    }
 
+#ifdef PERMIT_PLUGINS
+   Plugin_LoadAll();
+#endif
+
    if (!Options.silent) printf("Building host list for netmask %s, please wait...\n", Inet_MySubnet());
    number_of_hosts_in_lan = Inet_HostInLAN();
 
@@ -300,13 +297,25 @@ void Main_Normal(void)
    printf ("\n\033[01m\033[1m%s %s (c) 2001 %s\033[0m\n\n", PROGRAM, VERSION, AUTHORS);
    printf ("Your IP: %s with MAC: %s on Iface: %s\n", Inet_MyIPAddress(), Inet_MyMACAddress(), Options.netiface);
 
+#ifdef PERMIT_PLUGINS
+   if (Options.plugin || Options.arpsniff || Options.sniff  || Options.macsniff)
+      Plugin_LoadAll();
+#endif
 
-   if (Options.list || Options.check || Options.arpsniff || Options.sniff  || Options.macsniff || Options.link || Options.passive)
+   if (Options.list || Options.check || Options.arpsniff ||
+       Options.sniff  || Options.macsniff || Options.link ||
+       Options.passive || Options.hoststofile)
    {
       if (!Options.silent) printf("Building host list for netmask %s, please wait...\n", Inet_MySubnet());
       number_of_hosts_in_lan = Inet_HostInLAN();
    }
 
+   if (Options.hoststofile)
+   {
+      fprintf(stdout, "\nHost list dumped into file: %s\n\n", Inet_Save_Host_List());
+      exit(0);
+   }
+
    if (Options.list)
       Simple_HostList();
 
@@ -327,8 +336,8 @@ void Main_Normal(void)
    if (Options.arpsniff || Options.sniff  || Options.macsniff)
       Simple_Run();
 
-	if (Options.passive)
-		Simple_PassiveScan();
+   if (Options.passive)
+      Simple_PassiveScan();
 
    printf("\n");
 #ifdef DEBUG
@@ -340,20 +349,18 @@ exit(0);
 
 
 
-void Main_CheckForRun(void)
+void Main_CheckForRun(char *program_argv0)
 {
 
 #ifdef DEBUG
    Debug_msg("Main_CheckForRun");
 #endif
 
-#ifdef PERMIT_SUID
-   if (geteuid() != 0)
-      Error_msg("Sorry EUID %d, must be EUID = 0 to run %s !!", geteuid(), PROGRAM);
-#else
    if (getuid() != 0)
+   {
+      Options.normal = 1;
       Error_msg("Sorry UID %d, must be root to run %s !!", getuid(), PROGRAM);
-#endif
+   }
 
    if (!strstr(program_argv0, PROGRAM))      // just for script-kiddies ;)
       Error_msg("ehi guy ! my name is \"%s\" ! I really don't like \"%s\"...", PROGRAM, program_argv0);
@@ -425,8 +432,8 @@ void Main_Check_NewRelease(void)
    int sock;
    char *ptr;
    char *latest;
-   char getmsg[500];
-   char buffer[5000];
+   char getmsg[512];
+   char buffer[4096];
    char host[] = "ettercap.sourceforge.net";
    char page[] = "/latest.php";
 //   char host[] = "ghibli.alor.org";
@@ -458,10 +465,10 @@ void Main_Check_NewRelease(void)
       sock = Inet_OpenSocket(host, 80);
 
       fprintf (stdout, "Requesting %s...\n\n", page);
-      sprintf(getmsg, "GET %s HTTP/1.0\r\n"
-                      "Host: %s\r\n"
-                      "User-Agent: %s (%s).\r\n"
-                      "\r\n", page, host, PROGRAM, VERSION );
+      snprintf(getmsg, sizeof(getmsg), "GET %s HTTP/1.0\r\n"
+                                       "Host: %s\r\n"
+                                       "User-Agent: %s (%s).\r\n"
+                                       "\r\n", page, host, PROGRAM, VERSION );
       Inet_Http_Send(sock, getmsg);
 
 #ifdef DEBUG
@@ -469,7 +476,7 @@ void Main_Check_NewRelease(void)
    Debug_msg("Main_Check_NewRelease - ENDSEND --------------------");
 #endif
 
-      Inet_Http_Receive(sock, buffer);
+      Inet_Http_Receive(sock, buffer, sizeof(buffer));
 
 #ifdef DEBUG
    Debug_msg("Main_Check_NewRelease - RECEIVE --------------------\n\n%s\n\n", buffer);
@@ -513,8 +520,9 @@ void Main_Check_NewRelease(void)
          if (answer == 'y' || answer == 'Y')
          {
             char wget[100];
-            sprintf(wget, "wget http://%s/download/ettercap-%s.tar.gz", host, latest);
-            system(wget);
+            snprintf(wget, sizeof(wget), "http://%s/download/ettercap-%s.tar.gz", host, latest);
+            if ( execl( WGET_PATH, "wget", wget, NULL) == -1 )
+               Error_msg("Cannot execute wget ! Auto update cannot download the file...\n");
          }
       }
    }
