@@ -17,16 +17,12 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_parser.c,v 1.6 2001/09/10 16:41:33 alor Exp $
+    $Id: ec_parser.c,v 1.15 2001/12/08 16:38:43 alor Exp $
 */
 
 
 #include "include/ec_main.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
 #include <ctype.h>
 #ifdef HAVE_GETOPT_H
    #include <getopt.h>
@@ -41,19 +37,18 @@
 #include "include/ec_inet_structures.h"
 #include "include/ec_filterdrop.h"
 
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
-
 extern char *Execute_Plugin;
 
 char *list_to_parse;    // host list expanded from wildcards
+char *loading_plugs;    // list of plugin to be loaded.  it has this form: "|dummy|foo|bar|..."
 
 // protos...
 
 extern void Main_Usage(void);
 void Parser_ParseConfFile(char *filename);
 void Parser_Dissectors(char *toparse);
+void Parser_Plugins(char *toparse);
+char Parser_Activated_Plugin(char *name);
 void Parser_ParseParameters(char *first, char *second, char *third, char *fourth);
 int Parser_ParseOptions(int counter, char **values);
 char * Parser_PrintFilter(DROP_FILTER *ptr, int i);
@@ -63,6 +58,7 @@ char * Parser_AddStr(char *list, char *string);
 void Parser_Expand(char *to_parse);
 int match_pattern(const char *s, const char *pattern);
 void Parser_Filters(char *line, DROP_FILTER *filter);
+char *Parser_StrSpacetoUnder(char *h_name);
 
 //-----------------------------------
 
@@ -94,7 +90,7 @@ int Parser_ParseOptions(int counter, char **values)
       { "quiet", no_argument, NULL, 'q' },
       { "etterconf", required_argument, NULL, 'e' },
       { "dontresolve", no_argument, NULL, 'd' },
-      { "newcert", no_argument, NULL, 'k' },
+      { "newcert", no_argument, NULL, 'w' },
       { "filter", required_argument, NULL, 'F' },
       { "hosts", required_argument, NULL, 'H' },
       { "yes", no_argument, NULL, 'y' },
@@ -103,6 +99,8 @@ int Parser_ParseOptions(int counter, char **values)
       { "spoof", required_argument, NULL, 'S' },
       { "stormdelay", required_argument, NULL, 'Z' },
       { "passive", no_argument, NULL, 'O' },
+      { "loadhosts", required_argument, NULL, 'j' },
+      { "savehosts", no_argument, NULL, 'k' },
       { 0 , 0 , 0 , 0}
    };
 
@@ -113,9 +111,9 @@ int Parser_ParseOptions(int counter, char **values)
    optind = 0;
 
 #ifdef PERMIT_PLUGINS
-   while ((c = getopt_long (counter, values, "hvyNlasmci:p:xzuftCbn:Lqe:dkF:H:D:RS:Z:O",long_options, (int *)0)) != EOF) {
+   while ((c = getopt_long (counter, values, "hvyNlasmci:p:xzuftCbn:Lqe:dwF:H:D:RS:Z:Oj:k",long_options, (int *)0)) != EOF) {
 #else
-   while ((c = getopt_long (counter, values, "hvyNlasmci:xzuftCbn:Lqe:dkF:H:D:RS:Z:O",long_options, (int *)0)) != EOF) {    // no plugin
+   while ((c = getopt_long (counter, values, "hvyNlasmci:xzuftCbn:Lqe:dwF:H:D:RS:Z:Oj:k",long_options, (int *)0)) != EOF) {    // no plugin
 #endif
 
       switch (c) {
@@ -174,11 +172,20 @@ int Parser_ParseOptions(int counter, char **values)
 #endif
 
          case 'i':
-            strncpy(Options.netiface, optarg, 10);
+            strlcpy(Options.netiface, optarg, sizeof(Options.netiface));
+         break;
+
+         case 'j':
+            Options.hostsfromfile = 1;
+            Options.hostfile = strdup(optarg);
+         break;
+
+         case 'k':
+            Options.hoststofile = 1;
          break;
 
          case 'n':
-            strncpy(Options.netmask, optarg, 16);
+            strlcpy(Options.netmask, optarg, sizeof(Options.netmask));
          break;
 
          case 'D':
@@ -188,7 +195,7 @@ int Parser_ParseOptions(int counter, char **values)
 
          case 'Z':
             Options.storm_delay = atoi(optarg);
-            if (Options.storm_delay == 0) Options.storm_delay = 1;      // at least one second...
+            if (Options.storm_delay == 0) Options.storm_delay = 1;      // at least one usec...
          break;
 
          case 'S':
@@ -202,7 +209,7 @@ int Parser_ParseOptions(int counter, char **values)
                return 1;
          break;
 
-         case 'k':
+         case 'w':
             Options.normal = 1;
             Simple_CreateCertFile();
          break;
@@ -249,6 +256,8 @@ exit_debug:
 
 #define R(a,b,c) (a & b) | ((a ^ b) & c)     // returns true if more than one was selected
 
+   if (!Options.normal) Options.reverse = 0;
+
    if (Options.quiet && !Options.normal && !Options.logtofile )
       Error_msg("Demonization is only useful with -NL or -NLC !!\n\n");
 
@@ -409,6 +418,7 @@ char * Parser_AddStr(char *list, char *string)
    if (list == NULL)
       ERROR_MSG("realloc()");
    strcat(list, string);
+   list[strlen(list)+strlen(string)] = 0;
 
    return list;
 }
@@ -426,6 +436,9 @@ void Parser_ParseConfFile(char *filename)
    int pargc = 0;
    char *pargv[256];
    char dissectors = 0;
+#ifdef PERMIT_PLUGINS
+   char plugins = 0;
+#endif
 
 #ifdef DEBUG
    Debug_msg("Parser_ParseConfFile - %s", filename);
@@ -571,6 +584,19 @@ void Parser_ParseConfFile(char *filename)
          fprintf (stdout, "Setting dissectors handlers...\n");
          dissectors = 1;
       }
+#ifdef PERMIT_PLUGINS
+      if (!strncasecmp(line, "</hooking plugins>", 18))
+         plugins = 0;
+
+      if (plugins)
+         Parser_Plugins(line);
+
+      if (!strncasecmp(line, "<hooking plugins>", 17))
+      {
+         fprintf (stdout, "Plugins to be loaded...\n");
+         plugins = 1;
+      }
+#endif
 
    } while (!feof(etterconf));
 
@@ -604,6 +630,8 @@ void Parser_ParseConfFile(char *filename)
 }
 
 
+
+
 void Parser_Dissectors(char *toparse)
 {
    char name[15];
@@ -618,9 +646,9 @@ void Parser_Dissectors(char *toparse)
    memset(name, 0, sizeof(name));
    memset(arguments, 0, sizeof(arguments));
 
-   strncpy(name, strtok(toparse, "="), 15);
+   strlcpy(name, strtok(toparse, "="), sizeof(name));
 
-   strncpy(arguments, strtok(NULL, "="), 24);
+   strlcpy(arguments, strtok(NULL, "="), sizeof(arguments));
 
    if (!strncmp(arguments, "OFF", 3))
    {
@@ -669,6 +697,46 @@ void Parser_Dissectors(char *toparse)
 
 
 
+void Parser_Plugins(char *toparse)
+{
+   char name[20], l_name[22];
+   char args[4];
+
+   if (!strchr(toparse, '='))    // malformed line
+      return;
+
+   if (!loading_plugs) loading_plugs = (char *)calloc(1,1);
+
+   memset(name, 0, sizeof(name));
+   strlcpy(name, strtok(toparse, "="), 20);
+   strlcpy(args, strtok(NULL, "="), 4);
+
+   if (!strncmp(args, "ON", 2))
+   {
+      snprintf(l_name, sizeof(l_name), "|%s|", name);
+      loading_plugs = (char *)realloc(loading_plugs, strlen(loading_plugs)+strlen(l_name)+2);
+      strcat(loading_plugs, l_name);
+      fprintf(stdout, "%s\n", name);
+   }
+}
+
+
+
+
+char Parser_Activated_Plugin(char *name)
+{
+   char l_name[22];
+   snprintf(l_name, sizeof(l_name), "|%s|", name);
+
+   if (!loading_plugs) return 0;
+
+   if (strstr(loading_plugs, l_name)) return 1;
+
+   return 0;
+}
+
+
+
 void Parser_Filters(char *line, DROP_FILTER *filter)
 {
    int i, j;
@@ -682,9 +750,9 @@ void Parser_Filters(char *line, DROP_FILTER *filter)
       q = strstr(p, "</search>");
       i = ((int)q-(int)p) - strlen("<search>");
       if (i==0) return;
-      sprintf(tmp, "<search>%%%dc</search>", i);
+      snprintf(tmp, sizeof(tmp), "<search>%%%dc</search>", i);
       sscanf(p, tmp, filter->display_search);
-      filter->wildcard = FilterDrop_ParseWildcard(tmp_search, filter->display_search);
+      filter->wildcard = FilterDrop_ParseWildcard(tmp_search, filter->display_search, sizeof(tmp_search));
       filter->slen = FilterDrop_strescape(filter->search, tmp_search);
       return;
    }
@@ -694,7 +762,7 @@ void Parser_Filters(char *line, DROP_FILTER *filter)
       q = strstr(p, "</replace>");
       i = ((int)q-(int)p) - strlen("<replace>");
       if (i==0) return;
-      sprintf(tmp, "<replace>%%%dc</replace>", i);
+      snprintf(tmp,sizeof(tmp) ,"<replace>%%%dc</replace>", i);
       sscanf(p, tmp, filter->display_replace);
       filter->rlen = FilterDrop_strescape(filter->replace, filter->display_replace);
       return;
@@ -758,8 +826,8 @@ void Parser_LoadFilters(char *filename)
       etterfilter = fopen(line, "r");
       if (etterfilter == NULL)
       {
-         strcpy(line, INSTALL_PATH);
-         strcat(line, "etter.filter");
+         strlcpy(line, DATA_PATH, sizeof(line));
+         strlcat(line, "/etter.filter", sizeof(line));
          etterfilter = fopen(line, "r");
       }
       Filter_File = strdup(line);
@@ -777,10 +845,10 @@ void Parser_LoadFilters(char *filename)
    }
 
    if (etterfilter == NULL)
-         Error_msg("CAN'T find a filter file in ./ or in %s", INSTALL_PATH);
+         Error_msg("CAN'T find a filter file in ./ or in %s", DATA_PATH);
 
-   Filter_Source=0;
-   Filter_Dest=0;
+   Filter_Source = 0;
+   Filter_Dest = 0;
 
    if (Filter_Array_Source) free(Filter_Array_Source);
    if (Filter_Array_Dest) free(Filter_Array_Dest);
@@ -856,7 +924,7 @@ char * Parser_PrintFilter(DROP_FILTER *ptr, int i)
    static char tmp[100];
    int j;
 
-   j = sprintf(tmp, "%2d | %5d:%-5d %c [%-10.10s] %c ", i,  ptr[i].source, ptr[i].dest, ptr[i].proto, ptr[i].display_search, ptr[i].type);
+   j = snprintf(tmp, sizeof(tmp), "%2d | %5d:%-5d %c [%-10.10s] %c ", i,  ptr[i].source, ptr[i].dest, ptr[i].proto, ptr[i].display_search, ptr[i].type);
    if (ptr[i].type == 'R')
       j += sprintf(tmp+j, "[%-10.10s] ", ptr[i].display_replace);
    else
@@ -884,8 +952,8 @@ void Parser_Expand(char *to_parse)
    char ip[25];
    char *pattern;
 
-   memset(new_parse, 0, 25);
-   memset(ip, 0, 25);
+   memset(new_parse, 0, sizeof(new_parse));
+   memset(ip, 0, sizeof(ip));
    pattern = strdup(to_parse);
 
    if (strstr(pattern, "*") || strstr(pattern, "?"))
@@ -895,20 +963,22 @@ void Parser_Expand(char *to_parse)
          i++;
          if (!found && (strstr(q, "*") || strstr(q, "?")) )
          {
-            strlcat(new_parse, "%d", 25);
-            if (i<4) strlcat(new_parse, ".", 25);
+            strlcat(new_parse, "%d", sizeof(new_parse));
+            if (i<4) strlcat(new_parse, ".", sizeof(new_parse));
             found=1;
          }
          else
          {
-            strlcat(new_parse, q, 25);
-            if (i<4) strlcat(new_parse, ".", 25);
+            strlcat(new_parse, q, sizeof(new_parse));
+            if (i<4) strlcat(new_parse, ".", sizeof(new_parse));
          }
-
       }
    }
    else
-      strcpy(new_parse, to_parse);
+   {
+      strncpy(new_parse, to_parse, sizeof(new_parse)-1);
+      new_parse[sizeof(new_parse)-1]='\0';
+   }
 
    free(pattern);
 
@@ -929,7 +999,7 @@ void Parser_Expand(char *to_parse)
 
    for(i=0; i<256; i++)
    {
-      sprintf(ip, new_parse, i);
+      snprintf(ip, sizeof(ip), new_parse, i);
       if (match_pattern(ip, to_parse))
          Parser_Expand(ip);
    }
@@ -1032,5 +1102,21 @@ int match_pattern(const char *s, const char *pattern)
 }
 
 
+char *Parser_StrSpacetoUnder(char *h_name)
+{
+   int i;
+   static char toggle_name[200];
+
+   strcpy(toggle_name, h_name);
+   i = strlen(toggle_name);
+
+   for (i--;i>=0;i--)
+      if (toggle_name[i]==' ')
+         toggle_name[i]='_';
+
+   return toggle_name;
+}
+
+
 /* EOF */
 
