@@ -17,22 +17,14 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_signal.c,v 1.1.1.1 2001/08/27 19:23:43 alor Exp $
+    $Id: ec_signal.c,v 1.3 2001/11/19 09:30:23 alor Exp $
 */
 
 #include "include/ec_main.h"
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <signal.h>
 #include <sys/time.h>
 #include <sys/resource.h>
-#include <unistd.h>
-#if defined (LINUX) && !defined (__USE_GNU)
-   #define __USE_GNU    // for strsignal()
-#endif
-#include <string.h>
-
 
 #ifdef HAVE_NCURSES
    #ifdef HAVE_NCURSES_H
@@ -55,20 +47,14 @@
 #endif
 
 #include "include/ec_error.h"
-
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
+#include "include/ec_thread.h"
 
 // protos...
 
 void Signal_SigBuster(void);
-RETSIGTYPE Signal_ILL(int sig);
 RETSIGTYPE Signal_SEGV(int sig);
 RETSIGTYPE Signal_TERM(int sig);
 RETSIGTYPE Signal_WINCH(int sig);
-RETSIGTYPE Signal_USR1(int sig);
-RETSIGTYPE Signal_USR2(int sig);
 
 //-----------------------
 
@@ -78,7 +64,6 @@ void Signal_SigBuster(void)
    Debug_msg("Signal_Buster");
 #endif
 
-   signal(SIGILL,  Signal_ILL);     // this is received by ettercap if one of its sons get a SIGSEGV
    signal(SIGBUS,  Signal_SEGV);
    signal(SIGSEGV,  Signal_SEGV);
    signal(SIGHUP,   Signal_TERM);
@@ -87,26 +72,6 @@ void Signal_SigBuster(void)
    signal(SIGCHLD,  SIG_IGN);       // if I kill a forked process it doesn't become a zombie...
    signal(SIGPIPE,  Signal_TERM);   // if ettercap SIGSEGV, illithid receives this sig... (pipe_with_illithid)
 
-   signal(SIGUSR1, Signal_USR1);
-   signal(SIGUSR2, Signal_USR2);
-}
-
-
-RETSIGTYPE Signal_ILL(int sig)
-{
-#ifdef DEBUG
-   Debug_msg("[%s] One of my sons got an error...", program_argv0 );
-#endif
-#ifdef HAVE_NCURSES
-   if (ScreenMode)    // restore the cursor
-      curs_set(1);
-
-   #ifdef HAVE_TERMIOS_H
-      tcsetattr(0, TCSANOW, &original_term_info);
-   #endif
-#endif
-
-   exit(666);
 }
 
 
@@ -115,13 +80,11 @@ RETSIGTYPE Signal_SEGV(int sig)
 #ifdef DEBUG
    struct rlimit corelimit = {RLIM_INFINITY, RLIM_INFINITY};
    if (sig == SIGBUS)
-      Debug_msg("[%s] Bus Error...", program_argv0 );
+      Debug_msg("[%s] Bus Error...", ECThread_getname(pthread_self()) );
    else if (sig == SIGSEGV)
-      Debug_msg("[%s] Segmentation Fault...", program_argv0 );
+      Debug_msg("[%s] Segmentation Fault...", ECThread_getname(pthread_self()) );
 #endif
 
-   if (!strstr(program_argv0, PROGRAM)) kill(getppid(), SIGILL);    // kill the main process
-
 #ifdef HAVE_NCURSES
    if (ScreenMode)    // close the ncurses screen
       Interface_CloseScreen();
@@ -130,9 +93,9 @@ RETSIGTYPE Signal_SEGV(int sig)
 
    fprintf (stderr, "\n\033[01m\033[1m Ooops !! Somewhere in the stack a pointer got crazy...\n\n");
    if (sig == SIGBUS)
-      fprintf (stderr, " [%s] Bus Error...\033[0m\n\n\n", program_argv0);
+      fprintf (stderr, " [%s] Bus Error...\033[0m\n\n\n", ECThread_getname(pthread_self()));
    else if (sig == SIGSEGV)
-      fprintf (stderr, " [%s] Segmentation Fault...\033[0m\n\n", program_argv0);
+      fprintf (stderr, " [%s] Segmentation Fault...\033[0m\n\n", ECThread_getname(pthread_self()));
 
    fprintf (stderr, "===========================================================================\n");
    fprintf (stderr, " To report this error follow these steps:\n\n");
@@ -153,10 +116,10 @@ RETSIGTYPE Signal_SEGV(int sig)
    fprintf (stderr, "  3) make a report : \"tar zcvf error.tar.gz ettercap_debug.log \"\n\n");
    fprintf (stderr, "  4) get the gdb backtrace :\n"
                     "  \t - \"gdb ettercap core\"\n"
-                    " \t - at the gdb prompt \"bt\"\n"
-                    " \t - at the gdb prompt \"quit\" and return to the shell\n"
-                    " \t - copy and paste this output.\n\n");
-   fprintf (stderr, " 5) mail us the output of gdb and the error.tar.gz\n");
+                    "  \t - at the gdb prompt \"bt\"\n"
+                    "  \t - at the gdb prompt \"quit\" and return to the shell\n"
+                    "  \t - copy and paste this output.\n\n");
+   fprintf (stderr, "  5) mail us the output of gdb and the error.tar.gz\n");
    fprintf (stderr, "============================================================================\n");
 
 #ifndef DEBUG
@@ -177,13 +140,12 @@ RETSIGTYPE Signal_TERM(int sig)
 {
 #ifdef DEBUG
    #ifdef HAVE_STRSIGNAL
-      Debug_msg("[%s] Signal handler... (caught SIGNAL: %d) | %s", program_argv0, sig, strsignal(sig));
+      Debug_msg("[%s] Signal handler... (caught SIGNAL: %d) | %s", ECThread_getname(pthread_self()), sig, strsignal(sig));
    #else
-      Debug_msg("[%s] Signal handler... (caught SIGNAL: %d)", program_argv0, sig);
+      Debug_msg("[%s] Signal handler... (caught SIGNAL: %d)", ECThread_getname(pthread_self()), sig);
    #endif
 #endif
 
-   if (!strstr(program_argv0, PROGRAM)) exit(0);    // is not the main process...
 
 #ifdef HAVE_NCURSES
    if (ScreenMode)
@@ -196,7 +158,6 @@ RETSIGTYPE Signal_TERM(int sig)
       printf("\n Signal handler... (caught SIGNAL: %d)\n\n", sig);
    #endif
 
-   kill(-1*getpid(), SIGTERM);  // kill all my forked processes
    exit(1);
 
 }
@@ -238,25 +199,4 @@ RETSIGTYPE Signal_WINCH(int sig)
 }
 
 
-
-RETSIGTYPE Signal_USR1(int sig)
-{
-   active_dissector = (active_dissector) ? 0 : 1;     // activate/deactivate active dissector (arp based)
-   #ifdef DEBUG
-      Debug_msg("\tactive_dissector %d", active_dissector);
-   #endif
-   signal(SIGUSR1, Signal_USR1);
-}
-
-
-
-RETSIGTYPE Signal_USR2(int sig)
-{
-   Connection_Mode = (Connection_Mode) ? 0 : 1;
-   #ifdef DEBUG
-      Debug_msg("\tConnection_Mode %d", Connection_Mode);
-   #endif
-   signal(SIGUSR2, Signal_USR2);
-}
-
 /* EOF */
