@@ -17,19 +17,17 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_simple.c,v 1.7 2001/09/13 20:15:48 alor Exp $
+    $Id: ec_simple.c,v 1.14 2001/12/08 11:22:20 alor Exp $
 */
 
-#include <stdio.h>
-#include <string.h>
-#include <unistd.h>
-#include <stdlib.h>
+#include "include/ec_main.h"
+
 #include <fcntl.h>
 #include <time.h>
 #include <sys/time.h>
+#include <sys/wait.h>
 #include <signal.h>
 
-#include "include/ec_main.h"
 #include "include/ec_error.h"
 #include "include/ec_inet.h"
 #include "include/ec_inet_structures.h"
@@ -45,10 +43,7 @@
 #include "include/ec_logtofile.h"
 #include "include/ec_filterdrop.h"
 #include "include/ec_dryad.h"
-
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
+#include "include/ec_thread.h"
 
 #ifdef HAVE_TERMIOS_H
 
@@ -64,6 +59,7 @@
 
 #define ASCII_VIEW      0
 #define HEX_VIEW        1
+#define TEXT_VIEW       2
 
 extern char *Execute_Plugin;
 
@@ -128,28 +124,49 @@ void Simple_FingerPrint(void)
 void Simple_CreateCertFile(void)
 {
 
+#if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
+
+   FILE *fta, *ftr;
+   char line[512];
+
 #ifdef DEBUG
    Debug_msg("Simple_CreateCertFile");
 #endif
 
-#if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
-
    fprintf(stdout, "\nGenerating Openssl [%s] certificate...\n\n", CERT_FILE);
 
-   // XXX - i am cheap and dirty...  as dug song said in the webmitm source code...
-   // ripped form it
+   if ( !fork() )
+   {
+      if ( execl( OPENSSL_PATH, "openssl", "genrsa", "-out", CERT_FILE, "1024", NULL) == -1)
+         Error_msg("Could not launch openssl --> " OPENSSL_PATH);
+   }
+   else
+      wait(NULL);
 
-   if (system("openssl genrsa -out " CERT_FILE " 1024") != 0)
-      Error_msg("could not launch openssl (check the path)");
+   if ( !fork() )
+   {
+      if ( execl( OPENSSL_PATH, "openssl", "req", "-new", "-key", CERT_FILE, "-out", CERT_FILE ".csr", NULL) == -1)
+         Error_msg("Could not launch openssl --> " OPENSSL_PATH);
+   }
+   else
+      wait(NULL);
 
-   if (system("openssl req -new -key " CERT_FILE " -out " CERT_FILE ".csr") != 0)
-      Error_msg("could not launch openssl (check the path)");
+   if ( !fork() )
+   {
+      if ( execl( OPENSSL_PATH, "openssl", "x509", "-req", "-days", "365", "-in", CERT_FILE ".csr", "-signkey", CERT_FILE, "-out", CERT_FILE ".new", NULL) == -1)
+         Error_msg("Could not launch openssl --> " OPENSSL_PATH);
+   }
+   else
+      wait(NULL);
 
-   if (system("openssl x509 -req -days 365 -in " CERT_FILE ".csr -signkey " CERT_FILE " -out " CERT_FILE ".new"))
-      Error_msg("could not launch openssl (check the path)");
+   fta = fopen(CERT_FILE, "a");
+   ftr = fopen(CERT_FILE ".new", "r");
 
-   if (system("cat " CERT_FILE ".new >> " CERT_FILE) != 0)
-      Error_msg("could not launch cat");
+   while (fgets(line, sizeof(line), ftr))
+      fprintf(fta, "%s", line);
+
+   fclose(fta);
+   fclose(ftr);
 
    unlink(CERT_FILE ".new");
    unlink(CERT_FILE ".csr");
@@ -203,19 +220,19 @@ void Simple_CheckForPoisoner(void)
       if (SniffList[i].mode == 1)
       {
          found = 1;
-         printf("%s is replying for %s\n", Host_In_LAN[SniffList[i].Host_Index1].ip, Host_In_LAN[SniffList[i].Host_Index2].ip);
+         printf(" %s is replying for %s\n", Host_In_LAN[SniffList[i].Host_Index1].ip, Host_In_LAN[SniffList[i].Host_Index2].ip);
       }
 
       if (SniffList[i].mode == 2)
       {
          found = 1;
-         printf("MAC of %s and %s are identical !\n",Host_In_LAN[SniffList[i].Host_Index1].ip,Host_In_LAN[SniffList[i].Host_Index2].ip);
+         printf(" MAC of %s and %s are identical !\n",Host_In_LAN[SniffList[i].Host_Index1].ip,Host_In_LAN[SniffList[i].Host_Index2].ip);
       }
 
    }
 
    if (!found)
-      printf("No poisoners found in this lan (seems to be safe)\n\n");
+      printf(" No poisoners found in this lan (seems to be safe)\n\n");
 
    free(SniffList);
 }
@@ -224,14 +241,10 @@ void Simple_CheckForPoisoner(void)
 
 void Simple_Plugin(void)
 {
-   char plugin[strlen(Execute_Plugin)+6];
-   short i, n, found=-1;
-
 #ifdef DEBUG
    Debug_msg("Simple_Plugin -- %s", Execute_Plugin);
 #endif
 
-
    if (strcmp(Host_Source.name, ""))
       strcpy(Host_Source.ip, Inet_NameToIp(Host_Source.name));
 
@@ -242,39 +255,31 @@ void Simple_Plugin(void)
    Debug_msg("Simple_Plugin -- [%s] [%s] [%s] [%s]", Host_Source.name, Host_Source.ip, Host_Dest.name, Host_Dest.ip );
 #endif
 
-   sprintf(plugin, "ec_%s.so", Execute_Plugin);
-
-   n = Plugin_MakePlugList();
-
    if (!strcmp(Execute_Plugin, "list"))
    {
+      char ** plug_list, ** current;
 
-      if (n == 0)
-         Error_msg("\n NO available plugin found in %s or ./ !!\n", INSTALL_PATH);
+      plug_list = Plugin_ExtList();
+
+      if (!*plug_list)
+         Error_msg("\n NO available plugins found in %s or ./ !!\n", PLUGIN_PATH);
 
       fprintf(stdout, "\n Available Plugins :\n\n");
 
-      for(i=0; i<n; i++)
-         fprintf(stdout, "%2d) %8s     v %.1f -- %s\n", i+1, Plugin_Getname(Plugins_list[i].name), Plugins_list[i].version, Plugins_list[i].description);
-      return;
-   }
-
-   for(i=0; i<n; i++)
-   {
-      if ( !strcmp(Plugins_list[i].name, plugin) ) found = i;
-   }
-
-   if (found == -1)
-   {
-      fprintf(stderr, "\nPlugin %s NOT found in %s or in ./ !\n", plugin, INSTALL_PATH);
-      fprintf(stderr, "\nuse -p list to view the available ones.\n\n");
+      for (current = plug_list; *current; current++)
+      {
+        fprintf(stdout, "%s", *current);
+        free(*current);
+      }
+      free(plug_list);
    }
    else
    {
-      char toexec[strlen(plugin)+strlen(Plugins_list[found].path)];
-      strcpy(toexec, Plugins_list[found].path);
-      strcat(toexec, plugin);
-      Plugin_RunPlugIn(toexec, NULL);
+      if (!Plugin_RunExt(Execute_Plugin))
+      {
+         fprintf(stderr, "\nPlugin \"%s\" NOT found in %s or in ./ !\n", Execute_Plugin, PLUGIN_PATH);
+         fprintf(stderr, "\n'ettercap -Np list' to view the available ones.\n\n");
+      }
    }
 
 }
@@ -336,9 +341,9 @@ void reset_term()          // taken from readchar.c, by M. Andreoli (2000)
 
 void Simple_Run(void)
 {
-   int Illithid_pid = 0, Dopple_pid = 0;
+   pthread_t Illithid_pid = 0, Dopple_pid = 0;
    #if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
-      int Grell_pid = 0;
+      pthread_t Grell_pid = 0;
    #endif
    int i, index_source=-1, index_dest=-1;
    char view = ASCII_VIEW;
@@ -346,10 +351,10 @@ void Simple_Run(void)
    fd_set msk_fd;
    struct timeval TimeOut;
    SNIFFED_DATA data_from_illithid;
-   CONNECTION data_from_illithid_conn;
-   int datalen;
+   int datalen = 0;
    char source[20], dest[20];
    char proto = 'T', mode = IPBASED;
+   struct in_addr addr;
    TIME_DECLARE;
 
    if (Options.udp) proto = 'U';
@@ -461,8 +466,7 @@ void Simple_Run(void)
 
    if (!Options.quiet) printf("\nPress 'h' for help...\n\n");
 
-   pipe_with_illithid_data = Buffer_Create(1.0e6); // 1Mb in shared memory !!!
-   pipe_with_illithid = Buffer_Create(5.0e5);      // 500Kb
+   pipe_with_illithid_data = Buffer_Create(5.0e5); // 500 Kb
 
    if (!Options.quiet && Options.reverse) printf(" Reverse");
 
@@ -512,242 +516,270 @@ void Simple_Run(void)
 
    if (!Options.quiet && Options.collect) printf("Collecting passwords... \n\n");
 
-      memset(&TimeOut, 0, sizeof(TimeOut));  //  timeout = 0
-      FD_ZERO(&msk_fd);
+   memset(&TimeOut, 0, sizeof(TimeOut));  //  timeout = 0
+   FD_ZERO(&msk_fd);
 
-      if(!Options.quiet)
-      {
-         #ifdef HAVE_TERMIOS_H
-            set_raw_term();   // non blocking stdin... yes this work !!
-         #else
-            #ifdef DEBUG
-               Debug_msg("Simple_Run -- NO TERMIOS_H");
-            #endif
-            fcntl(0, F_SETFL, O_NONBLOCK);   // stdin non blocking... seems to be a non working method for me...
+   if(!Options.quiet)
+   {
+      #ifdef HAVE_TERMIOS_H
+         set_raw_term();   // non blocking stdin... yes this work !!
+      #else
+         #ifdef DEBUG
+            Debug_msg("Simple_Run -- NO TERMIOS_H");
          #endif
+         fcntl(0, F_SETFL, O_NONBLOCK);   // stdin non blocking... seems to be a non working method for me...
+      #endif
+   }
+
+   current_illithid_data.proto = proto;
+   current_illithid_data.source_port = Host_Source.port;
+   current_illithid_data.dest_port = Host_Dest.port;
+
+   if (inet_aton(Host_Source.ip, &addr))
+      current_illithid_data.source_ip = ntohl(addr.s_addr);
+   if (inet_aton(Host_Dest.ip, &addr))
+      current_illithid_data.dest_ip =  ntohl(addr.s_addr);
+
+   TIME_START;
+
+   loop
+   {
+
+      FD_SET(0, &msk_fd);
+
+      select(FOPEN_MAX, &msk_fd, (fd_set *) 0, (fd_set *) 0, &TimeOut);
+
+      if (FD_ISSET(0, &msk_fd))
+      {
+         char ch = 0;
+         ch = getchar();
+         switch(ch)
+         {
+            case 'A':
+            case 'a':
+                  if (!Options.collect)
+                  {
+                     printf("\n\nAscii mode...\n\n");
+                     view = ASCII_VIEW;
+                  }
+                  break;
+
+            case 'X':
+            case 'x':
+                  if (!Options.collect)
+                  {
+                     printf("\n\nHex mode...\n\n");
+                     view = HEX_VIEW;
+                  }
+                  break;
+
+            case 'T':
+            case 't':
+                  if (!Options.collect)
+                  {
+                     printf("\n\nTEXT only mode...\n\n");
+                     view = TEXT_VIEW;
+                  }
+                  break;
+
+            case 'L':
+            case 'l':
+                  if (Options.logtofile)
+                  {
+                     printf("\n\nStop logging to file(s)...\n\n");
+                     Options.logtofile = 0;
+                  }
+                  else
+                  {
+                     printf("\n\nLogging to file(s)...\n\n");
+                     Options.logtofile = 1;
+                  }
+                  break;
+
+            case ' ':
+                  if (stop == 1)
+                  {
+                     printf("\n\nLet's go...\n\n");
+                     stop = 0;
+                  }
+                  else
+                  {
+                     printf("\n\nStopped...\n\n");
+                     stop = 1;
+                  }
+                  break;
+
+            case 'S':
+            case 's':
+                  if (Options.filter)
+                  {
+                     filter_on_source = (filter_on_source) ? 0 : 1;
+                     printf("\n\nFILTERS: on source %d | on dest %d\n\n", filter_on_source, filter_on_dest);
+                  }
+                  break;
+
+            case 'D':
+            case 'd':
+                  if (Options.filter)
+                  {
+                     filter_on_dest = (filter_on_dest) ? 0 : 1;
+                     printf("\n\nFILTERS: on source %d | on dest %d\n\n", filter_on_source, filter_on_dest);
+                  }
+                  break;
+
+            case 'H':
+            case 'h':
+                  printf("\n\n[qQ]  - quit\n");
+                  if (!Options.collect)
+                  {
+                     printf(    "[aA]  - dump data in ascii mode\n");
+                     printf(    "[xX]  - dump data in hex mode\n");
+                     printf(    "[tT]  - dump data in text only mode\n");
+                  }
+                  if (Options.filter)
+                  {
+                     printf(    "[sS]  - set/unset filter on source\n");
+                     printf(    "[dD]  - set/unset filter on dest\n");
+                  }
+                  printf(    "[lL]  - log all trafic to file(s)\n");
+                  printf(    "space - stop/cont sniffing\n\n");
+                  break;
+
+            case 'Q':
+            case 'q':
+#ifdef HAVE_TERMIOS_H
+                  if(!Options.quiet) reset_term();
+#endif
+                  printf("\n\n\nShutting down all threads... ");
+                  fflush(stdout);
+                  ECThread_destroy(Illithid_pid);
+                  if (Dopple_pid) ECThread_destroy(Dopple_pid);
+                  #if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
+                     if (Grell_pid) ECThread_destroy(Grell_pid);
+                  #endif
+                  printf("Done.\n\n");
+                  exit(0);
+                  break;
+         }
       }
 
-      TIME_START;
+      if (!Options.collect)
+         datalen = Buffer_Get(pipe_with_illithid_data, &data_from_illithid, sizeof(SNIFFED_DATA));
 
-      loop
+      if (datalen > 0 || Connection_Mode)
       {
-
-         FD_SET(0, &msk_fd);
-
-         select(FOPEN_MAX, &msk_fd, (fd_set *) 0, (fd_set *) 0, &TimeOut);
-
-         if (FD_ISSET(0, &msk_fd))
-         {
-            char ch = 0;
-            ch = getchar();
-            switch(ch)
-            {
-               case 'A':
-               case 'a':
-                     if (!Options.collect)
-                     {
-                        printf("\n\nAscii mode...\n\n");
-                        view = ASCII_VIEW;
-                     }
-                     break;
-
-               case 'X':
-               case 'x':
-                     if (!Options.collect)
-                     {
-                        printf("\n\nHex mode...\n\n");
-                        view = HEX_VIEW;
-                     }
-                     break;
-
-               case 'L':
-               case 'l':
-                     if (Options.logtofile)
-                     {
-                        printf("\n\nStop logging to file(s)...\n\n");
-                        Options.logtofile = 0;
-                     }
-                     else
-                     {
-                        printf("\n\nLogging to file(s)...\n\n");
-                        Options.logtofile = 1;
-                     }
-                     break;
-
-               case ' ':
-                     if (stop == 1)
-                     {
-                        printf("\n\nLet's go...\n\n");
-                        stop = 0;
-                     }
-                     else
-                     {
-                        printf("\n\nStopped...\n\n");
-                        stop = 1;
-                     }
-                     break;
-
-               case 'S':
-               case 's':
-                     if (Options.filter)
-                     {
-                        filter_on_source = (filter_on_source) ? 0 : 1;
-                        write(pipe_filter[1], "S", 1);
-                        printf("\n\nFILTERS: on source %d | on dest %d\n\n", filter_on_source, filter_on_dest);
-                     }
-                     break;
-
-               case 'D':
-               case 'd':
-                     if (Options.filter)
-                     {
-                        filter_on_dest = (filter_on_dest) ? 0 : 1;
-                        write(pipe_filter[1], "D", 1);
-                        printf("\n\nFILTERS: on source %d | on dest %d\n\n", filter_on_source, filter_on_dest);
-                     }
-                     break;
-
-               case 'H':
-               case 'h':
-                     printf("\n\n[qQ]  - quit\n");
-                     if (!Options.collect)
-                     {
-                        printf(    "[aA]  - dump data in ascii mode\n");
-                        printf(    "[xX]  - dump data in hex mode\n");
-                     }
-                     if (Options.filter)
-                     {
-                        printf(    "[sS]  - set/unset filter on source\n");
-                        printf(    "[dD]  - set/unset filter on dest\n");
-                     }
-                     printf(    "[lL]  - log all trafic to file(s)\n");
-                     printf(    "space - stop/cont sniffing\n\n");
-                     break;
-
-               case 'Q':
-               case 'q':
-#ifdef HAVE_TERMIOS_H
-                     if(!Options.quiet) reset_term();
-#endif
-                     printf("\n\n\nQuitting...\n\n");
-                     if (Illithid_pid) kill(Illithid_pid, SIGTERM);
-                     if (Dopple_pid) kill(Dopple_pid, SIGTERM);
-                     #if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
-                        if (Grell_pid) kill(Grell_pid, SIGTERM);
-                     #endif
-                     exit(0);
-                     break;
-            }
-         }
+         time_t tt = time(NULL);
+         struct tm *dd = localtime(&tt);
 
          if (Options.collect)
-            datalen = Buffer_Get(pipe_with_illithid, &data_from_illithid_conn, sizeof(CONNECTION));
-         else
-            datalen = Buffer_Get(pipe_with_illithid_data, &data_from_illithid, sizeof(SNIFFED_DATA));
-
-         if (datalen>0)
          {
-            time_t tt = time(NULL);
-            struct tm *dd = localtime(&tt);
+            int j;
 
-            if (Options.collect)
+            for (j=0; j < number_of_connections; j++)
             {
-               int j;
-               number_of_connections = Decodedata_MakeConnectionList(data_from_illithid_conn);
-
-               for (j=0; j < number_of_connections; j++)
+               if (Conn_Between_Hosts[j].user[0] != 0 &&  Conn_Between_Hosts[j].pass[0] != 0)
                {
-                  if (Conn_Between_Hosts[j].user[0] != 0 &&  Conn_Between_Hosts[j].pass[0] != 0)
+                  if (Options.logtofile) LogToFile_Collect(&Conn_Between_Hosts[j]);
+
+                  if (!Options.quiet)
                   {
-                     if (Options.logtofile) LogToFile_Collect(&Conn_Between_Hosts[j]);
+                     printf("\n\n%02d:%02d:%02d  %s:%d <--> %s:%d%15s\n\n", dd->tm_hour, dd->tm_min, dd->tm_sec,
+                           Conn_Between_Hosts[j].source_ip,
+                           Conn_Between_Hosts[j].source_port,
+                           Conn_Between_Hosts[j].dest_ip,
+                           Conn_Between_Hosts[j].dest_port,
+                           Conn_Between_Hosts[j].type
+                           );
 
-                     if (!Options.quiet)
-                     {
-                        printf("\n\n%02d:%02d:%02d  %s:%d <--> %s:%d%15s\n\n", dd->tm_hour, dd->tm_min, dd->tm_sec,
-                              Conn_Between_Hosts[j].source_ip,
-                              Conn_Between_Hosts[j].source_port,
-                              Conn_Between_Hosts[j].dest_ip,
-                              Conn_Between_Hosts[j].dest_port,
-                              Conn_Between_Hosts[j].type
-                              );
-
-                        printf("%s\n", Conn_Between_Hosts[j].user);
-                        printf("%s\n", Conn_Between_Hosts[j].pass);
-                        if (strlen(Conn_Between_Hosts[j].info))
-                           printf("\n%s\n", Conn_Between_Hosts[j].info);
-                     }
-                     memset(&Conn_Between_Hosts[j], 0, sizeof(CONNECTION));  // dont display in the future...
-                     #ifdef DEBUG
-                        Debug_msg("\tConnection %d displayed and deleted...", j+1);
-                     #endif
+                     printf("%s\n", Conn_Between_Hosts[j].user);
+                     printf("%s\n", Conn_Between_Hosts[j].pass);
+                     if (strlen(Conn_Between_Hosts[j].info))
+                        printf("\n%s\n", Conn_Between_Hosts[j].info);
                   }
-               }  // end for
-               TIME_FINISH;
-               if (TIME_ELAPSED >= 300)
-               {
+                  Decodedata_SetArrayIndex(&Conn_Between_Hosts[j], -1); // dont display in the future...
+                  memset(&Conn_Between_Hosts[j].user, 0, sizeof(Conn_Between_Hosts[j].user));
+                  memset(&Conn_Between_Hosts[j].pass, 0, sizeof(Conn_Between_Hosts[j].pass));
+                  memset(&Conn_Between_Hosts[j].info, 0, sizeof(Conn_Between_Hosts[j].info));
                   #ifdef DEBUG
-                     Debug_msg("\tConnection List refreshed after %.0f minutes...", (TIME_ELAPSED)/60);
+                     Debug_msg("\tConnection %d displayed and deleted...", j+1);
                   #endif
-                  TIME_START;                         // after 300 seconds (5 min)...
-                  Decodedata_RefreshConnectionList(); // periodically refresh the list...
                }
-            }
-            else // !collect
-            if ( Options.reverse ^ (proto == data_from_illithid.proto && Illithid_ToBeSniffed("", Host_Source.port, "", Host_Dest.port, &data_from_illithid)))
+            }  // end for
+            TIME_FINISH;
+            if (TIME_ELAPSED >= 300)
             {
+               #ifdef DEBUG
+                  Debug_msg("\tConnection List refreshed after %.0f minutes...", (TIME_ELAPSED)/60);
+               #endif
+               TIME_START;                         // after 300 seconds (5 min)...
+               Decodedata_RefreshConnectionList(); // periodically refresh the list...
+            }
+            usleep(5000);
+         }
+         else // !collect
+         {
 
-               if (Options.logtofile) LogToFile(&data_from_illithid);
-               if (!stop && !Options.quiet )
+            if (Options.logtofile) LogToFile(&data_from_illithid);
+            if (!stop && !Options.quiet )
+            {
+               switch (view)
                {
-                  switch (view)
-                  {
-                     case ASCII_VIEW:
-                        printf("\n\n\n%02d:%02d:%02d  %s:%d --> %s:%d\n\n%s", dd->tm_hour, dd->tm_min, dd->tm_sec,
+                  case ASCII_VIEW:
+                     printf("\n\n\n%02d:%02d:%02d  %s:%d --> %s:%d\n\n%s", dd->tm_hour, dd->tm_min, dd->tm_sec,
+                        data_from_illithid.source_ip,
+                        data_from_illithid.source_port,
+                        data_from_illithid.dest_ip,
+                        data_from_illithid.dest_port,
+                        Decodedata_GetAsciiData(data_from_illithid.data, data_from_illithid.datasize)
+                     );
+                     break;
+
+                  case TEXT_VIEW:
+                     printf("\n\n\n%02d:%02d:%02d  %s:%d --> %s:%d\n\n%s", dd->tm_hour, dd->tm_min, dd->tm_sec,
+                        data_from_illithid.source_ip,
+                        data_from_illithid.source_port,
+                        data_from_illithid.dest_ip,
+                        data_from_illithid.dest_port,
+                        Decodedata_GetTextData(data_from_illithid.data, data_from_illithid.datasize)
+                     );
+                     break;
+
+                  case HEX_VIEW:
+                     if (data_from_illithid.proto == 'T')
+                        printf("\n\n%02d:%02d:%02d  %s:%d --> %s:%d | seq %lx ack %lx | flags %s |\n%s ", dd->tm_hour, dd->tm_min, dd->tm_sec,
                            data_from_illithid.source_ip,
                            data_from_illithid.source_port,
                            data_from_illithid.dest_ip,
                            data_from_illithid.dest_port,
-                           Decodedata_GetAsciiData(data_from_illithid.data, data_from_illithid.datasize)
+                           data_from_illithid.seq,
+                           data_from_illithid.ack_seq,
+                           Decodedata_TCPFlags(data_from_illithid.flags),
+                           Decodedata_GetHexData(data_from_illithid.data, data_from_illithid.datasize, 80)
                         );
-                        break;
-
-                     case HEX_VIEW:
-                        if (data_from_illithid.proto == 'T')
-                           printf("\n\n%02d:%02d:%02d  %s:%d --> %s:%d | seq %lx ack %lx | flags %s |\n%s ", dd->tm_hour, dd->tm_min, dd->tm_sec,
-                              data_from_illithid.source_ip,
-                              data_from_illithid.source_port,
-                              data_from_illithid.dest_ip,
-                              data_from_illithid.dest_port,
-                              data_from_illithid.seq,
-                              data_from_illithid.ack_seq,
-                              Decodedata_TCPFlags(data_from_illithid.flags),
-                              Decodedata_GetHexData(data_from_illithid.data, data_from_illithid.datasize, 80)
-                           );
-                        else
-                           printf("\n\n%02d:%02d:%02d  %s:%d --> %s:%d | UDP |\n%s ", dd->tm_hour, dd->tm_min, dd->tm_sec,
-                              data_from_illithid.source_ip,
-                              data_from_illithid.source_port,
-                              data_from_illithid.dest_ip,
-                              data_from_illithid.dest_port,
-                              Decodedata_GetHexData(data_from_illithid.data, data_from_illithid.datasize, 80)
-                           );
-                        break;
-                  }
-                  fflush(stdout);
+                     else
+                        printf("\n\n%02d:%02d:%02d  %s:%d --> %s:%d | UDP |\n%s ", dd->tm_hour, dd->tm_min, dd->tm_sec,
+                           data_from_illithid.source_ip,
+                           data_from_illithid.source_port,
+                           data_from_illithid.dest_ip,
+                           data_from_illithid.dest_port,
+                           Decodedata_GetHexData(data_from_illithid.data, data_from_illithid.datasize, 80)
+                        );
+                     break;
                }
+               fflush(stdout);
             }
          }
-         else
-             usleep(1000);
       }
+      else
+         usleep(1);
+   }
 }
 
 
 void Simple_PassiveScan(void)
 {
-   int Dryad_pid = 0;
-   int datalen, i;
-   PASSIVE_DATA data_from_dryad;
+   pthread_t Dryad_pid;
+   int i;
    fd_set msk_fd;
    struct timeval TimeOut;
    TIME_DECLARE;
@@ -761,8 +793,6 @@ void Simple_PassiveScan(void)
 
    if (!Options.quiet) printf("\nPress 'h' for help...\n\n");
 
-   pipe_with_dryad = Buffer_Create(2.0e5); // 200Kb in shared memory !!!
-
    Dryad_pid = Dryad_Run();
 
    if (!Options.quiet) printf(" Passive Scanning : ANY <--> ANY\n\n");
@@ -890,16 +920,16 @@ void Simple_PassiveScan(void)
 #ifdef HAVE_TERMIOS_H
                      if(!Options.quiet) reset_term();
 #endif
-                     printf("\n\n\nQuitting...\n\n");
-                     if (Dryad_pid) kill(Dryad_pid, SIGTERM);
+                     printf("\n\n\nShutting down all threads... ");
+                     fflush(stdout);
+                     ECThread_destroy(Dryad_pid);
                      Decodedata_FreePassiveList();
+                     printf("Done.\n\n");
                      exit(0);
                      break;
             }
          }
 
-         datalen = Buffer_Get(pipe_with_dryad, &data_from_dryad, sizeof(PASSIVE_DATA));
-
          if (Options.logtofile)
          {
             TIME_FINISH;
@@ -914,31 +944,24 @@ void Simple_PassiveScan(void)
             }
          }
 
-         if (datalen > 0)
+         if (!Options.quiet)
          {
-            number_of_passive_hosts = Decodedata_MakePassiveList(data_from_dryad);
+            u_short hash = Inet_Forge_Checksum((u_short *)Passive_Host, 0xe77e, sizeof(PASSIVE_DATA)*number_of_passive_hosts, 0, 0);
 
-            if (!Options.quiet)
+            if (passive_csum != hash && number_of_passive_hosts > 0)
             {
-               u_short hash = Inet_Forge_Checksum((u_short *)Passive_Host, 0xe77e, sizeof(PASSIVE_DATA)*number_of_passive_hosts, 0, 0);
+               int j;
 
-               if (passive_csum != hash )
+               Decodedata_Passive_SortList();
+               printf ("\n\n>>> PASSIVE INFORMATION <<<\n\n");
+               for(j=0; j< number_of_passive_hosts; j++)
                {
-                  int j;
-
-                  Decodedata_Passive_SortList();
-                  printf ("\n\n>>> PASSIVE INFORMATION <<<\n\n");
-                  for(j=0; j< number_of_passive_hosts; j++)
-                  {
-                     printf(" --> %15s : %2s : %s\n", Passive_Host[j].ip, Passive_Host[j].type, Passive_Host[j].os);
-                  }
-                  passive_csum = hash;
+                  printf(" --> %15s : %2s : %s\n", Passive_Host[j].ip, Passive_Host[j].type, Passive_Host[j].os);
                }
+               passive_csum = hash;
             }
-
          }
-         else  // datalen = 0
-             usleep(1000);
+         usleep(1);
       }
 }
 
