@@ -16,12 +16,15 @@ extern int Inet_Fake_Host(void);
 
 extern int Inet_OpenSocket(char *host, short port);
 extern int Inet_Http_Send(int s, char *payload);
-extern int Inet_Http_Receive(int s, char *payload);
+extern int Inet_Http_Receive(int s, char *payload, size_t size);
+
+extern int Inet_SendLargeTCPPacket(int sock, char *buffer, int len, int MTU);
+extern char * Inet_Save_Host_List(void);
 
 // Following are architecture dependent !! implementations are in ./src/`uname`/ec_inet_`uname`.c
 
 extern int Inet_FindIFace(char *iface);
-extern char Inet_CorrectIface(char *iface);
+extern int Inet_CorrectIface(char *iface);
 
 extern int Inet_GetIfaceInfo(char *iface, int *MTU, char *MyMAC, unsigned long *IP, unsigned long *NetMask);
 extern int Inet_SetPromisc(char *iface);
