@@ -1,8 +1,3 @@
-
-#if !defined(EC_INET_FORGE_H)
-#define EC_INET_FORGE_H
-
-
 #include <sys/types.h>
 
 extern u_char * Inet_Forge_packet( u_short size ); // size of the allocated mem
@@ -66,9 +61,4 @@ extern u_short Inet_Forge_Checksum( u_short *buffer,
 extern u_long Inet_Forge_CRC(u_char *buffer, // the buffer
                              int len);       // buffer lenght
 
-#endif
-
 /* EOF */
-
-// vim:ts=3:expandtab
-
