@@ -1,5 +1,5 @@
 
-extern FIELD *make_label(int frow, int fcol, NCURSES_CONST char *label);
+extern FIELD *make_label(int frow, int fcol, char *label);
 extern FIELD *make_field(int frow, int fcol, int rows, int cols, bool secure);
 extern void display_form(FORM *f);
 extern void erase_form(FORM *f);
