@@ -1,17 +1,8 @@
 
-#if !defined(EC_INTERFACE_PLUGINS_H)
-#define EC_INTERFACE_PLUGINS_H
-
-
 extern void Interface_Plugins_Run(void);
 extern void Interface_Plugins_Winch(void);
 extern void Interface_Plugins_Output(void);
 extern void Interface_Plugins_PluginOutput(void);
 extern void Interface_Plugins_HidePluginOutput(void);
 
-#endif
-
 /* EOF */
-
-// vim:ts=3:expandtab
-
