@@ -1,8 +1,4 @@
 
-#if !defined(EC_INTERFACE_SNIFF_DATA_H)
-#define EC_INTERFACE_SNIFF_DATA_H
-
-
 extern void Interface_Sniff_Data_Run(char *ips,
                                      int portsource,
                                      char *ipd,
@@ -11,14 +7,8 @@ extern void Interface_Sniff_Data_Run(char *ips,
                                      char *macd,
                                      char proto,
                                      char *type,
-                                     short mode,
-				     int connection);
+                                     short mode);
 extern void Interface_Sniff_Data_Winch(void);
 extern void Interface_Sniff_Data_Redraw(void);
 
-#endif
-
 /* EOF */
-
-// vim:ts=3:expandtab
-
