@@ -1,22 +1,37 @@
 
-/* $Id: ec_main.h,v 1.8 2001/09/17 07:34:35 alor Exp $ */
+/* $Id: ec_main.h,v 1.16 2001/12/13 17:05:20 alor Exp $ */
 
 #ifdef HAVE_CONFIG_H
    #include <config.h>     // autoheader   -I.
 #endif
 
 #define DEVEL_RELEASE 0
-#define VERSION "0.6.0"
+#define VERSION "0.6.3.1"
 #define PROGRAM "ettercap"
 #define AUTHORS "ALoR & NaGA"
 
 #define loop for(;;)
 
-extern char *program_argv0;
-
+#include <stdio.h>
+#include <stdlib.h>
+#include <unistd.h>
+#if defined (LINUX) && !defined (__USE_GNU)  // for memmem(), strsignal(), etc etc...
+   #define __USE_GNU
+#endif
+#include <string.h>
+#if defined (__USE_GNU)
+   #undef __USE_GNU
+#endif
+#include <strings.h>
 #include <sys/types.h>
+#include <time.h>
+#include <pthread.h>
 #include "ec_queue.h"
 
+#ifdef DEBUG
+   #include "ec_debug.h"
+#endif
+
 #ifndef HAVE_STRLCAT             // missing functions
    #include "../missing/strlcat.h"
 #endif
@@ -30,11 +45,8 @@ extern char *program_argv0;
    #include "../missing/memmem.h"
 #endif
 
-#ifdef HAVE_ATEXIT
-   #define exit_func(x)    atexit(x)
-#else
-   #define exit_func(x)
-#endif
+#define exit_func(x)    pthread_cleanup_push(x, (void *)NULL)
+#define exit_func_end() pthread_cleanup_pop(0)
 
 
 typedef struct host_arp          // hosts in LAN info
@@ -76,6 +88,9 @@ typedef struct options_list      // arguments passed on command line
    char yes:1;
    char reverse;
    char passive:1;
+   char hostsfromfile:1;
+   char hoststofile:1;
+   char *hostfile;
    char netiface[10];
    char netmask[16];
    short delay;
@@ -122,6 +137,8 @@ typedef struct sniffed_data   // data through the pipe with illithid
 {
    char source_ip[16];
    char dest_ip[16];
+   u_long fast_source_ip;
+   u_long fast_dest_ip;
    u_short source_port;
    u_short dest_port;
    u_long seq;
@@ -133,6 +150,17 @@ typedef struct sniffed_data   // data through the pipe with illithid
 } SNIFFED_DATA;
 
 
+typedef struct current_sniffed_data
+{
+   u_long source_ip;
+   u_long dest_ip;
+   u_short source_port;
+   u_short dest_port;
+   char proto;
+} CURRENT_SNIFFED_DATA;
+
+extern CURRENT_SNIFFED_DATA current_illithid_data;
+
 typedef struct {
    int Host_Index1;
    int Host_Index2;
@@ -146,6 +174,8 @@ typedef struct connection        // connection list
    char dest_ip[16];
    char source_mac[20];
    char dest_mac[20];
+   u_long fast_source_ip;
+   u_long fast_dest_ip;
    u_short source_port;
    u_short dest_port;
    u_long source_seq;
@@ -154,12 +184,19 @@ typedef struct connection        // connection list
    char proto;
    short datalen;
    char status[8];
+   time_t timestamp;
    char type[18];          // from /etc/services
    char user[30];          // pay attention on buffer overflow !!
    char pass[30];
    char info[150];         // additional info... ( smb domain, http page ...)
 } CONNECTION;
 
+typedef struct dissect
+{
+   u_char *layer4;
+   CONNECTION *connection;
+} DISSECTION;
+
 
 #define MAX_INJECT 1000          // max number of injected chars
 #define MAX_FILTER 200           // max filter length
@@ -206,11 +243,9 @@ typedef struct {                 // killing structure
 extern CONNECTION *Conn_Between_Hosts;
 extern int number_of_connections;
 
-extern int pipe_with_dryad;
-extern int pipe_with_illithid;
 extern int pipe_with_illithid_data;
+extern int pipe_with_plugins;
 extern int pipe_inject[2];
-extern int pipe_filter[2];
 extern int pipe_kill[2];
 
 extern char active_dissector;    // ec_main.c
@@ -250,8 +285,11 @@ extern int host_to_be_scanned;
 #define DEFAULT_DELAY 30            // in seconds, delay between arp replies
 #define DEFAULT_STORM_DELAY 1500    // in microseconds, delay between inizial arp request
 
-#include "ec_install_path.h"  // where ettercap was installed
+#include "ec_install_path.h"
 
 #define OS_FILE "etter.passive.os.fp"
 
+#define CONN_TIME_SILENT   1        // in seconds, the time to change the connection status from ACTIVE to silent
+#define CONN_TIME_TIMEOUT  1200
+
 /* EOF */
