@@ -29,6 +29,8 @@
 #include <sys/types.h>
 #include <string.h>
 
+void * memmem(const char *, size_t, const char *, size_t);
+
 /*
  *  DESCRIPTION
  *        The memmem() function finds the start of the first occurrence
@@ -41,8 +43,8 @@
  */
 
 
-void * memmem(const u_char *haystack, size_t haystacklen,
-              const u_char *needle, size_t needlelen)
+void * memmem(const char *haystack, size_t haystacklen,
+              const char *needle, size_t needlelen)
 {
 	register const char *h = haystack;
 	register const char *n = needle;
@@ -51,7 +53,7 @@ void * memmem(const u_char *haystack, size_t haystacklen,
 	register size_t i;
 
 	if (nl == 0) return (void *) haystack;    /* The first occurrence of the empty string is deemed to occur at
-                                                the beginning of the string.  */
+                                                     the beginning of the string.  */
 
 	for( i = 0; (i < hl) && (i + nl <= hl ); i++ )
 		if (h[i] == *n)	/* first match */
